@@ -3,7 +3,7 @@
 //  celestia
 //
 //  Created by Hank Ramsey on Fri Oct 29 2004.
-//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
+//  Copyright (C) 2007, Celestia Development Team
 //
 
 #import "CelestiaSettings.h"
@@ -14,6 +14,7 @@
 #include "celestia.h"
 #include "render.h"
 
+
 class MacOSXWatcher : public CelestiaWatcher
 {
     private:
@@ -74,17 +75,14 @@ static NSMutableDictionary* tagMap;
         [[super init] release];
         return sharedInstance;
     }
-    
+
     self = [super init];
-    
+
     tagMap = [[ NSMutableDictionary dictionaryWithCapacity: 100 ] retain];
-    
+
     appCore = (CelestiaCore*) [[CelestiaAppCore sharedAppCore] appCore];
 
-     MacOSXWatcher* theWatcher = new MacOSXWatcher(appCore,self);
-     appCore->addWatcher(theWatcher);
-
-    tagDict = [[NSDictionary dictionaryWithObjectsAndKeys:
+    tagDict = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
 //        TAGDEF(101,@"time")
 //        TAGDEF(999,@"timeScale")
         TAGDEF(300,@"fullScreenMode")
@@ -106,7 +104,8 @@ static NSMutableDictionary* tagMap;
         TAGDEF(409,@"showSmoothLines")
         TAGDEF(410,@"showEclipseShadows")
         TAGDEF(412,@"showRingShadows")
-        TAGDEF(411,@"showStarsAsPoints")
+        TAGDEF(419,@"showCloudShadows")
+//        TAGDEF(411,@"showStarsAsPoints")
         TAGDEF(414,@"showAutoMag")
         TAGDEF(406,@"showCelestialSphere")
         // object labels
@@ -121,12 +120,15 @@ static NSMutableDictionary* tagMap;
         TAGDEF(508,@"showCometLabels")
         TAGDEF(509,@"showNebulaLabels")
         TAGDEF(510,@"showOpenClusterLabels")
+        TAGDEF(511,@"showLatinConstellationLabels")
         // popups
         TAGDEF(600,@"altSurface")
         TAGDEF(610,@"hudDetail")
         TAGDEF(620,@"starStyle")
         TAGDEF(630,@"renderPath")
         TAGDEF(640,@"resolution")
+        TAGDEF(650,@"timeZone")
+        TAGDEF(660,@"dateFormat")
         // orbits
 //        TAGDEF(999,@"minimumOrbitSize")
         TAGDEF(700,@"showPlanetOrbits")
@@ -134,6 +136,7 @@ static NSMutableDictionary* tagMap;
         TAGDEF(702,@"showAsteroidOrbits")
         TAGDEF(704,@"showSpacecraftOrbits")
         TAGDEF(703,@"showCometOrbits")
+        TAGDEF(705,@"showStellarOrbits")
         // feature labels
         TAGDEF(903,@"minimumFeatureSize")
         TAGDEF(800,@"showCityLabels")
@@ -168,25 +171,28 @@ static NSMutableDictionary* tagMap;
         // stars
 //        TAGDEF(999,@"distanceLimit")
         TAGDEF(900,@"ambientLightLevel")
-        TAGDEF(901,@"brightnessBias")
+//        TAGDEF(901,@"brightnessBias")
         TAGDEF(902,@"faintestVisible")
         TAGDEF(904,@"galaxyBrightness")
 //        TAGDEF(999,@"saturationMagnitude")
 
-        nil ] retain];
+        nil];
     keyArray = [[tagDict allValues] retain];
-    
+
+#if REFMARKS
+    NSDictionary *volatileTagDict = [NSDictionary dictionaryWithObjectsAndKeys:
+        TAGDEF(1000,@"showBodyAxes")
+        TAGDEF(1001,@"showFrameAxes")
+        TAGDEF(1002,@"showSunDirection")
+        TAGDEF(1003,@"showVelocityVector")
+        nil];
+    [tagDict addEntriesFromDictionary: volatileTagDict];
+#endif
 
     return self;
 }
 
--(void)addWatcher: (CelestiaController*) control
-{
-     MacOSXWatcher* theWatcher = new MacOSXWatcher(appCore,self);
-     appCore->addWatcher(theWatcher);
-}
-
--(void) setControl: (CelestiaController*) _control
+-(void) setControl: (id) _control
 {
     control = _control;
 }
@@ -251,17 +257,17 @@ static NSMutableDictionary* tagMap;
 -(void) loadUserDefaults 
 {
 	id key;
-//        NSLog(@"loading user defaults");
 	NSDictionary* defaultsDictionary = [self findUserDefaults ];
 	if ( defaultsDictionary == nil ) return;
 	NSEnumerator* keys = [ defaultsDictionary keyEnumerator];
 	while ( nil != (key = [keys nextObject]) )
 	{
-//                NSLog([NSString stringWithFormat: @"loading default dict entry %@ %@", key, [defaultsDictionary objectForKey: key] ]);
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
+		[self setValue: [defaultsDictionary objectForKey: key] forKey: key];
+#else
 		[self takeValue: [defaultsDictionary objectForKey: key] forKey: key];
-//                NSLog([NSString stringWithFormat: @"loaded default dict entry %@ %@", key, [self valueForKey: key] ]);
+#endif
 	}
-//        NSLog(@"loaded user defaults");
 }
 
 -(void) loadAppDefaults
@@ -299,47 +305,84 @@ static NSMutableDictionary* tagMap;
 
 -(id) valueForTag: (int) tag { 
     return [self valueForKey: [tagDict objectForKey: [NSNumber numberWithInt: tag] ] ];
-    };
+}
 
 -(void) takeValue: (id) value forTag: (int) tag 
 {
-//      NSLog([NSString stringWithFormat: @"take value for tag %@ %d", value, tag]);
-      id key = [tagDict objectForKey: [NSNumber numberWithInt: tag] ];
-//      NSLog([NSString stringWithFormat: @"take value for key %@ %@", value, key]);
-     if (key!= nil) [self takeValue: value forKey: key ];
-};
+    id key = [tagDict objectForKey: [NSNumber numberWithInt: tag] ];
+    if (key!= nil)
+    {
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
+        [self setValue: value forKey: key ];
+#else
+        [self takeValue: value forKey: key ];
+#endif
+    }
+}
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
+- (id)valueForUndefinedKey: (NSString *) key
+{
+#ifdef DEBUG
+    if ( key ) NSLog(@"unbound value for %@", key);
+#endif
+    return nil;
+}
+
+- (void)setValue: (id) value forUndefinedKey: (NSString *) key
+{
+#ifdef DEBUG
+    NSLog(@"unbound key set for %@", key);
+#endif
+}
+
+- (void)setNilValueForKey: (NSString *) key
+{
+#ifdef DEBUG
+    NSLog(@"nil value for %@", key);
+#endif
+}
+
+#else
 
 -(id)handleQueryWithUnboundKey: (NSString*) key
 {
-    if ( key != NULL) NSLog([NSString stringWithFormat: @"unbound key query for %@",key]);
+#ifdef DEBUG
+    if ( key ) NSLog(@"unbound key query for %@", key);
+#endif
     return nil;
-};
+}
 
 -(void)handleTakeValue: (id) value forUnboundKey: (NSString*) key
 {
-    NSLog([NSString stringWithFormat: @"unbound key set for %@",key]);
-};
+#ifdef DEBUG
+    NSLog(@"unbound key set for %@", key);
+#endif
+}
 
 -(void)unableToSetNilForKey: (NSString*) key
 {
-    NSLog([NSString stringWithFormat: @"nl key for %@",key]);
-};
+#ifdef DEBUG
+    NSLog(@"nil key for %@", key);
+#endif
+}
+#endif
 
 // Time Settings
 
--(NSNumber *) time { return [NSNumber numberWithDouble: appCore->getSimulation()->getTime()]; }
--(void) setTime: (NSNumber *) value { appCore->getSimulation()->setTime(value ? [value doubleValue] : 0.0); }
+-(double) time { return appCore->getSimulation()->getTime(); }
+-(void) setTime: (double) value { appCore->getSimulation()->setTime(value); }
 
--(NSNumber *) timeScale { return [NSNumber numberWithDouble: appCore->getSimulation()->getTimeScale()]; }
--(void) setTimeScale: (NSNumber *) value { appCore->getSimulation()->setTimeScale(value ? [value doubleValue] : 0.0); }
+-(double) timeScale { return appCore->getSimulation()->getTimeScale(); }
+-(void) setTimeScale: (double) value { appCore->getSimulation()->setTimeScale(value); }
 
--(BOOL) synchTime { return appCore->getSimulation()->getSyncTime(); };
--(void) setSynchTime: (BOOL) value { appCore->getSimulation()->setSyncTime(value); };
+-(BOOL) synchTime { return appCore->getSimulation()->getSyncTime(); }
+-(void) setSynchTime: (BOOL) value { appCore->getSimulation()->setSyncTime(value); }
 
 // Gaze Settings
 
--(NSNumber *) fieldOfView { return [NSNumber numberWithFloat: appCore->getSimulation()->getObserver().getFOV()]; }
--(void)  setFieldOfView: (NSNumber *) value { appCore->getSimulation()->getObserver().setFOV(value ? [value floatValue] : 0.0f); }
+-(float) fieldOfView { return appCore->getSimulation()->getObserver().getFOV(); }
+-(void)  setFieldOfView: (float) value { appCore->getSimulation()->getObserver().setFOV(value); }
 
 // Observer Settings
 
@@ -356,7 +399,7 @@ static NSMutableDictionary* tagMap;
 //    NSLog([NSString stringWithFormat: @"setValue %d forBit: %d inSet: %d = %d",value,bit,set,result]);
 //    NSLog([NSString stringWithFormat: @"bit was: %d bit is: %d ",(set&bit),(result&bit)]);
     return result;
-};
+}
 
 // Visibility Settings
 
@@ -377,7 +420,7 @@ RENDERMETHODS(PartialTrajectories)
 RENDERMETHODS(SmoothLines)
 RENDERMETHODS(EclipseShadows)
 RENDERMETHODS(RingShadows)
-RENDERMETHODS(StarsAsPoints)
+RENDERMETHODS(CloudShadows)
 RENDERMETHODS(AutoMag)
 RENDERMETHODS(CelestialSphere)
 RENDERMETHODS(Nebulae)
@@ -399,6 +442,13 @@ LABELMETHODS(Comet)
 LABELMETHODS(Nebula)
 LABELMETHODS(OpenCluster)
 
+-(BOOL) showLatinConstellationLabels {
+    return (appCore->getRenderer()->getLabelMode() & Renderer::I18nConstellationLabels) == 0;
+}
+-(void) setShowLatinConstellationLabels : (BOOL) value {
+    appCore->getRenderer()->setLabelMode( [self setValue: (!value) forBit:  Renderer::I18nConstellationLabels inSet: appCore->getRenderer()->getLabelMode()] );
+}
+
 // Orbit Settings
 
 #define ORBITMETHODS(flag)  -(BOOL) show##flag##Orbits { return (appCore->getRenderer()->getOrbitMask()&Body::flag) != 0; } -(void) setShow##flag##Orbits: (BOOL) value  { appCore->getRenderer()->setOrbitMask([self setValue: value forBit: Body::flag inSet: appCore->getRenderer()->getOrbitMask()]); } 
@@ -408,10 +458,11 @@ ORBITMETHODS(Moon)
 ORBITMETHODS(Asteroid)
 ORBITMETHODS(Spacecraft)
 ORBITMETHODS(Comet)
+ORBITMETHODS(Stellar)
 
 
--(NSNumber *) minimumOrbitSize { return [NSNumber numberWithFloat: appCore->getRenderer()->getMinimumOrbitSize()]; }
--(void)  setMinimumOrbitSize: (NSNumber *) value { appCore->getRenderer()->setMinimumOrbitSize(value ? [value floatValue] : 0.0f); }
+-(float) minimumOrbitSize { return appCore->getRenderer()->getMinimumOrbitSize(); }
+-(void)  setMinimumOrbitSize: (float) value { appCore->getRenderer()->setMinimumOrbitSize(value); }
 
 // Feature Settings
 
@@ -447,132 +498,124 @@ FEATUREMETHODS(Fluctus)
 FEATUREMETHODS(Farrum)
 FEATUREMETHODS(Other)
 
--(NSNumber *) minimumFeatureSize { return [NSNumber numberWithFloat: appCore->getRenderer()->getMinimumFeatureSize()]; }
--(void)  setMinimumFeatureSize: (NSNumber *) value { appCore->getRenderer()->setMinimumFeatureSize(value ? [value floatValue] : 0.0f); }
+-(float) minimumFeatureSize { return appCore->getRenderer()->getMinimumFeatureSize(); }
+-(void)  setMinimumFeatureSize: (float) value { appCore->getRenderer()->setMinimumFeatureSize(value); }
+
+#if REFMARKS
+// Refmark Settings
+
+#define REFMARKMETHODS(flag)  -(BOOL) show##flag { return (appCore->getSimulation()->getSelection().body()->getVisibleReferenceMarks()&Body::flag) != 0; } -(void) setShow##flag: (BOOL) value  { appCore->getSimulation()->getSelection().body()->setVisibleReferenceMarks([self setValue: value forBit: Body::flag inSet: appCore->getSimulation()->getSelection().body()->getVisibleReferenceMarks()]); } 
+
+REFMARKMETHODS(BodyAxes)
+REFMARKMETHODS(FrameAxes)
+REFMARKMETHODS(SunDirection)
+REFMARKMETHODS(VelocityVector)
+#endif
 
 // Lighting Settings
 
--(NSNumber *) ambientLightLevel { return [NSNumber numberWithFloat: appCore->getRenderer()->getAmbientLightLevel()]; }
--(void)  setAmbientLightLevel: (NSNumber *) value { appCore->getRenderer()->setAmbientLightLevel(value ? [value floatValue] : 0.0f); }
+-(float) ambientLightLevel { return appCore->getRenderer()->getAmbientLightLevel(); }
+-(void)  setAmbientLightLevel: (float) value { appCore->getRenderer()->setAmbientLightLevel(value); }
 
--(NSNumber *) galaxyBrightness { return [NSNumber numberWithFloat: Galaxy::getLightGain()]; }
--(void)  setGalaxyBrightness: (NSNumber *) value { Galaxy::setLightGain(value ? [value floatValue] : 0.0f); }
+-(float) galaxyBrightness { return Galaxy::getLightGain(); }
+-(void)  setGalaxyBrightness: (float) value { Galaxy::setLightGain(value); }
 
 // Star Settings
 
--(NSNumber *) distanceLimit { return [NSNumber numberWithFloat: appCore->getRenderer()->getDistanceLimit()]; }
--(void)  setDistanceLimit: (NSNumber *) value { appCore->getRenderer()->setDistanceLimit(value ? [value floatValue] : 0.0f); }
+-(float) distanceLimit { return appCore->getRenderer()->getDistanceLimit(); }
+-(void)  setDistanceLimit: (float) value { appCore->getRenderer()->setDistanceLimit(value); }
 
--(NSNumber *) faintestVisible 
+-(float) faintestVisible 
 { 
 //    return appCore->getSimulation()->getFaintestVisible(); 
     if ((appCore->getRenderer()->getRenderFlags() & Renderer::ShowAutoMag) == 0)
     {
-        return [NSNumber numberWithFloat: appCore->getSimulation()->getFaintestVisible()];
+        return appCore->getSimulation()->getFaintestVisible();
     }
     else
     {
-        return [NSNumber numberWithFloat: appCore->getRenderer()->getFaintestAM45deg()];
+        return appCore->getRenderer()->getFaintestAM45deg();
     }                
 }
 
--(void)  setFaintestVisible: (NSNumber *) value 
+-(void)  setFaintestVisible: (float) value 
 {
-    float theValue = value ? [value floatValue] : 0.0f;
-
     if ((appCore->getRenderer()->getRenderFlags() & Renderer::ShowAutoMag) == 0)
     {
-        appCore->setFaintest(theValue);
+        appCore->setFaintest(value);
     }
     else
     {
-        appCore->getRenderer()->setFaintestAM45deg(theValue);
+        appCore->getRenderer()->setFaintestAM45deg(value);
         appCore->setFaintestAutoMag();
     }                
 }
 
-// Brightness Settings
-
--(NSNumber *) saturationMagnitude { return [NSNumber numberWithFloat: appCore->getRenderer()->getSaturationMagnitude()]; }
--(void)  setSaturationMagnitude: (NSNumber *) value { appCore->getRenderer()->setSaturationMagnitude(value ? [value floatValue] : 0.0f); }
-
--(NSNumber *) brightnessBias { return [NSNumber numberWithFloat: appCore->getRenderer()->getBrightnessBias()]; }
--(void)  setBrightnessBias: (NSNumber *) value { appCore->getRenderer()->setBrightnessBias(value ? [value floatValue] : 0.0f); }
-
--(int)  starStyle { return appCore->getRenderer()->getStarStyle(); }  ;
--(void) setStarStyle: (int) value { appCore->getRenderer()->setStarStyle((Renderer::StarStyle)value); };
+-(int)  starStyle { return appCore->getRenderer()->getStarStyle(); }
+-(void) setStarStyle: (int) value { appCore->getRenderer()->setStarStyle((Renderer::StarStyle)value); }
 
 // Texture Settings
 
--(int)  resolution { return appCore->getRenderer()->getResolution(); }  ;
--(void) setResolution: (int) value { appCore->getRenderer()->setResolution(value); };
+-(int)  resolution { return appCore->getRenderer()->getResolution(); }
+-(void) setResolution: (int) value { appCore->getRenderer()->setResolution(value); }
 
 // Overlay Settings
 
--(int)  hudDetail { return appCore->getHudDetail(); }  ;
--(void) setHudDetail: (int) value { appCore->setHudDetail(value); };
+-(int)  hudDetail { return appCore->getHudDetail(); }
+-(void) setHudDetail: (int) value { appCore->setHudDetail(value); }
+
+// Time settings
+
+-(int)  timeZone { return appCore->getTimeZoneBias()==0 ? 1 : 0; }
+-(void) setTimeZone: (int) value
+{
+    NSTimeZone *tz;
+    NSDate *date = [NSDate dateWithJulian: [NSNumber numberWithDouble: [self time]]];
+    if (0 == value)
+    {
+        tz = [NSTimeZone defaultTimeZone];
+    }
+    else
+    {
+        tz = [NSTimeZone timeZoneWithAbbreviation: @"GMT"];
+    }
+    [[CelestiaAppCore sharedAppCore] setTimeZone: tz withDate: date];
+}
+
+-(int)  dateFormat { return appCore->getDateFormat(); }
+-(void) setDateFormat: (int) value { appCore->setDateFormat((astro::Date::Format)value); }
 
 // Other Settings
 
--(int)  renderPath { return appCore->getRenderer()->getGLContext()->getRenderPath(); }  ;
--(void) setRenderPath: (int) value { appCore->getRenderer()->getGLContext()->setRenderPath((GLContext::GLRenderPath)value); };
+-(int)  renderPath { return appCore->getRenderer()->getGLContext()->getRenderPath(); }
+-(void) setRenderPath: (int) value { appCore->getRenderer()->getGLContext()->setRenderPath((GLContext::GLRenderPath)value); }
 
 -(int)  fullScreenMode { return [[control valueForKey: @"isFullScreen"] intValue]; }
--(void) setFullScreenMode: (int) value { [control takeValue: [NSNumber numberWithBool: (value != 0)] forKey: @"isFullScreen"]; }
+-(void) setFullScreenMode: (int) value
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
+    [control setValue: [NSNumber numberWithBool: (value != 0)] forKey: @"isFullScreen"];
+#else
+    [control takeValue: [NSNumber numberWithBool: (value != 0)] forKey: @"isFullScreen"];
+#endif
+}
 
 // Alt Surface Setting
 
-/*- (BOOL) validateAltSurface: (int) index
-{
-    if (index == 0) 
-    {
-        string displayedSurfName = appCore->getSimulation()->getActiveObserver()->getDisplayedSurface();
-        return displayedSurfName == string("");
-    }
-    index--;
-    // Validate items for the alternate surface submenu
-    Selection sel = appCore->getSimulation()->getSelection();
-    if (sel.body() != NULL)
-    {
-        vector<string>* surfNames = sel.body()->getAlternateSurfaceNames();
-        if (surfNames != NULL)
-        {
-            string surfName, displayedSurfName;
-            if (index >= 0 && index < (int)surfNames->size())
-                surfName = surfNames->at(index);
-            displayedSurfName = appCore->getSimulation()->getActiveObserver()->getDisplayedSurface();
-            if (surfName  == displayedSurfName )
-            { 
-                delete surfNames;
-                return YES;
-            }
-        }
-    }
-    return NO;
-}
-*/
-
 - (int) altSurface
 {
     // return alternate surface index
     string displayedSurfName = appCore->getSimulation()->getActiveObserver()->getDisplayedSurface();
-    Selection sel = appCore->getSimulation()->getSelection();
-    if (sel.body() != NULL)
+    CelestiaSelection *sel = [[[CelestiaAppCore sharedAppCore] simulation] selection];
+    if ([sel body])
     {
-        vector<string>* surfNames = sel.body()->getAlternateSurfaceNames();
-        if (surfNames != NULL)
+        NSArray *surfaces = [[sel body] alternateSurfaceNames];
+        if ( surfaces && [surfaces count] > 0 )
         {
-            int index;
-            string surfName;
-            for (index = 0 ; index < (int)surfNames->size(); index++)
-            {
-                surfName = surfNames->at(index);
-                if (surfName  == displayedSurfName )
-                { 
-                    delete surfNames;
-                    return index+1;
-                }
-            }
+            NSString *displayedSurfStr = [NSString stringWithStdString: displayedSurfName];
+            unsigned index = [surfaces indexOfObject: displayedSurfStr];
+            if (index != NSNotFound)
+                return index+1;
         }
     }
     return 0;
@@ -581,95 +624,26 @@ FEATUREMETHODS(Other)
 - (void) setAltSurface: (int) index
 {
     // Handle the alternate surface submenu
-    Selection sel = appCore->getSimulation()->getSelection();
-    if (sel.body() != NULL)
+    CelestiaSelection *sel = [[[CelestiaAppCore sharedAppCore] simulation] selection];
+    if ([sel body])
     {
         if ( index == 0 )
         {
             appCore->getSimulation()->getActiveObserver()->setDisplayedSurface(string(""));
             return;
         }
-        vector<string>* surfNames = sel.body()->getAlternateSurfaceNames();
-        if (surfNames != NULL)
+        NSArray *surfaces = [[sel body] alternateSurfaceNames];
+        if ( surfaces && [surfaces count] > 0 )
         {
             string surfName;
             index--;
-            if (index >= 0 && index < (int)surfNames->size())
-                surfName = surfNames->at(index);
+            if (index >= 0 && (unsigned)index < [surfaces count])
+                surfName = [[surfaces objectAtIndex: index] stdString];
             appCore->getSimulation()->getActiveObserver()->setDisplayedSurface(surfName);
-            delete surfNames;
         }
     }
 }
 
-- (void) disableSurfaceMenu: (NSMenu*) contextMenu
-{
-        NSMenuItem* surfaceItem = [ contextMenu itemWithTitle: @"Show Alternate Surface"];
-//        [ surfaceItem setSubmenu: NULL ];
-//        [ surfaceItem setAction: NULL ];
-//        [ contextMenu setAutoenablesItems: NO ];
-        [ surfaceItem setEnabled: NO ];
-        NSLog(@"disabling surface menu\n");
-}
-
-- (void) addSurfaceMenu: (NSMenu*) contextMenu
-{
-    Selection sel = appCore->getSimulation()->getSelection();
-    if (sel.body() == NULL)
-    {
-        [self disableSurfaceMenu: contextMenu ];
-        return;
-    }
-    else
-    {
-        NSMenuItem* firstItem = [ contextMenu itemAtIndex: 0];
-        NSMenuItem* surfaceItem = [ contextMenu itemWithTitle: @"Show Alternate Surface"];
-        NSMenu* surfaceMenu = [[NSMenu alloc ] initWithTitle: @"altsurf" ];
-        vector<string>* surfaces = sel.body()->getAlternateSurfaceNames();
-        if ( surfaces->size() == 0 ) 
-        {
-            [self disableSurfaceMenu: contextMenu];
-            return;
-        }
-            [ surfaceItem setEnabled: YES ];
-            [ contextMenu setAutoenablesItems: YES ];
-            [ surfaceMenu setAutoenablesItems: YES ];
-            NSMenuItem* newItem = [ [NSMenuItem alloc] init ]; 
-            [newItem setTitle: @"default" ];
-            [newItem setTag:  600 ];
-            [newItem setTarget:  [firstItem target] ];
-            [newItem setAction:  [firstItem action] ];
-            [ surfaceMenu addItem: newItem ];
-        for (int i = 0; i < (int)surfaces->size(); i++)
-        {
-            NSMenuItem* newItem = [ [NSMenuItem alloc] init ]; 
-            [newItem setTitle: [ NSString stringWithStdString: (*surfaces)[i] ] ];
-            [newItem setTag:  601+i ];
-            [newItem setEnabled:  YES ];
-            [newItem setTarget:  [firstItem target] ];
-            [newItem setAction:  [firstItem action] ];
-            [ surfaceMenu addItem: newItem ];
-        }
-        [ surfaceItem setSubmenu: surfaceMenu ];
-        [ surfaceItem setEnabled: YES ];
-        [ surfaceMenu update ];
-        delete surfaces;
-        return;
-    }
-}
-
-/* not yet...
-
-// Shader Settings
-
-    bool getFragmentShaderEnabled() const;
-    void setFragmentShaderEnabled(bool);
-    bool fragmentShaderSupported() const;
-    bool getVertexShaderEnabled() const;
-    void setVertexShaderEnabled(bool);
-    bool vertexShaderSupported() const;
-
-*/
 
 // GUI Tag Methods ----------------------------------------------------------
 
@@ -691,10 +665,10 @@ FEATUREMETHODS(Other)
     id value;
 
     if ( tag <= 128 ) tag = [self tagForKey: tag ]; // handle menu item for setting; obsolete?
-    if ( tag <= 128 ) { appCore->charEntered(tag); return; };
+    if ( tag <= 128 ) { appCore->charEntered(tag); return; }
     switch ( tag/100)
     {
-        case 4: case 5: case 7: case 8: // 400, 500, 700, 800
+        case 4: case 5: case 7: case 8: case 10: // 400, 500, 700, 800, 1000
             [self takeValue: [NSNumber numberWithInt: (1-[[self valueForTag: tag] intValue])] forTag: tag ];
         break;
         case 6: // 600
@@ -752,27 +726,27 @@ FEATUREMETHODS(Other)
     }
     else 
     {
-	switch ( tag/100)
-	{
-		case 4: case 5: case 7: case 8:
-		// 400s, 500s, 700s, 800s (checkboxes)
-		        [item setState: [[self valueForTag: tag] intValue] ? NSOnState : NSOffState ];
+        switch ( tag/100)
+        {
+		case 4: case 5: case 7: case 8: case 10:
+		// 400s, 500s, 700s, 800s, 1000s (checkboxes)
+            [item setState: [[self valueForTag: tag] intValue] ? NSOnState : NSOffState ];
 		break;
 		case 6:
 		// 600s (popups)
-                    if ( tag >= 600 && tag < 610 ) // altSurface menu
-                    {
-                        int index = tag-600;
-                        [item setState:  ([self altSurface] == index ) ? NSOnState: NSOffState ];            
-                    } 
-                    else
-                        [self validateButton: item atIndex: tag%10 withValue: [[self valueForTag: tag-(tag%10)] intValue] ];
+            if ( tag >= 600 && tag < 610 ) // altSurface menu
+            {
+                int index = tag-600;
+                [item setState:  ([self altSurface] == index ) ? NSOnState: NSOffState ];            
+            } 
+            else
+                [self validateButton: item atIndex: tag%10 withValue: [[self valueForTag: tag-(tag%10)] intValue] ];
 		break;
 		case 9:
 		// 900s (floats)
-	                [item setFloatValue: [[self valueForTag: tag] floatValue] ];                
+            [item setFloatValue: [[self valueForTag: tag] floatValue] ];                
 		break;
-	}
+        }
     }
     return YES;
 }
@@ -817,7 +791,7 @@ FEATUREMETHODS(Other)
             case  25: tag = 414; break;  // AutoMag
             case  20: tag = 415; break;  // CometTails
             case  11: tag = 416; break;  // Markers
-            case  19: tag = 411; break;  // StarsAsPoints
+//            case  19: tag = 411; break;  // StarsAsPoints
             case  24: tag = 409; break;  // SmoothLines
             default : tag = key; break; // Special or not a setting
     }
@@ -956,7 +930,7 @@ FEATUREMETHODS(Other)
 //        NSLog(@"subviews items = %@\n", [item subviews]);
         item = [ [item subviews] objectEnumerator ];
 //        NSLog(@"view items = %@\n", item);
-    };
+    }
 
     if ( [item isKindOfClass: [NSEnumerator class] ] )
     {
