@@ -7,11 +7,15 @@
 //
 
 #ifndef gettext
-#define gettext(s)  s
+#include "POSupport.h"
+#define gettext(s)      localizedUTF8String(s)
+#define dgettext(d,s)   localizedUTF8StringWithDomain(d,s)
 #endif
 
+#define __AIFF__
+#include <TargetConditionals.h>
+
 #ifdef __OBJC__
-# define __AIFF__
 # import <Cocoa/Cocoa.h>
 # undef DPRINTF
 #endif
