@@ -1,6 +1,6 @@
 // body.cpp
 //
-// Copyright (C) 2001 Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2006 Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -17,13 +17,16 @@
 #include "mesh.h"
 #include "meshmanager.h"
 #include "body.h"
+#include "frame.h"
 
 using namespace std;
 
-
 Body::Body(PlanetarySystem* _system) :
     orbit(NULL),
-    orbitRefPlane(astro::BodyEquator),
+    orbitBarycenter(_system ? _system->getPrimaryBody() : NULL),
+    orbitFrame(NULL),
+    bodyFrame(NULL),
+    rotationModel(NULL),
     radius(10000.0f),
     mass(0.0f),
     oblateness(0),
@@ -43,7 +46,9 @@ Body::Body(PlanetarySystem* _system) :
     classification(Unknown),
     altSurfaces(NULL),
     locations(NULL),
-    locationsComputed(false)
+    locationsComputed(false),
+    referenceMarks(0),
+    frameRefStar(NULL)
 {
     system = _system;
 }
@@ -54,6 +59,11 @@ Body::~Body()
     // clean up orbit, atmosphere, etc.
     if (system != NULL)
         system->removeBody(this);
+
+    if (orbitFrame != NULL)
+        orbitFrame->release();
+    if (bodyFrame != NULL)
+        bodyFrame->release();
 }
 
 
@@ -91,15 +101,65 @@ void Body::setOrbit(Orbit* _orbit)
 }
 
 
-astro::ReferencePlane Body::getOrbitReferencePlane() const
+// TODO: orbitBarycenter is superceded by frames and should be removed.
+const Body* Body::getOrbitBarycenter() const
 {
-    return orbitRefPlane;
+    return orbitBarycenter;
 }
 
 
-void Body::setOrbitReferencePlane(astro::ReferencePlane refPlane)
+void Body::setOrbitBarycenter(const Body* barycenterBody)
 {
-    orbitRefPlane = refPlane;
+    assert(barycenterBody == NULL || barycenterBody->getSystem()->getStar() == getSystem()->getStar());
+    if (barycenterBody != NULL)
+    {
+    }
+    orbitBarycenter = barycenterBody;
+}
+
+const ReferenceFrame* Body::getOrbitFrame() const
+{
+    return orbitFrame;
+}
+
+
+void
+Body::setOrbitFrame(const ReferenceFrame* f)
+{
+    // Update reference counts
+    if (f != NULL)
+        f->addRef();
+    if (orbitFrame != NULL)
+        orbitFrame->release();
+
+    orbitFrame = f;
+
+    // Temporary hack: keep track of the star this frame is ultimately
+    // referenced to if it is different from the star at the root of
+    // the body's name hierarchy.
+    Star* frameRoot = getFrameReferenceStar();
+    Star* refStar = getReferenceStar();
+    if (frameRoot != refStar)
+        frameRefStar = frameRoot;
+}
+
+
+const ReferenceFrame* Body::getBodyFrame() const
+{
+    return bodyFrame;
+}
+
+
+void
+Body::setBodyFrame(const ReferenceFrame* f)
+{
+    // Update reference counts
+    if (f != NULL)
+        f->addRef();
+    if (bodyFrame != NULL)
+        bodyFrame->release();
+
+    bodyFrame = f;
 }
 
 
@@ -116,8 +176,8 @@ void Body::setRadius(float _radius)
 
 
 // For an irregular object, the radius is defined to be the largest semi-axis
-// of the axis-aligned bounding box.  The radius of the smallest sphere containing
-// the object is potentially larger by a factor of sqrt(3)
+// of the axis-aligned bounding box.  The radius of the smallest sphere
+// containing the object is potentially larger by a factor of sqrt(3)
 float Body::getBoundingRadius() const
 {
     if (model == InvalidResource)
@@ -175,15 +235,14 @@ void Body::setOrientation(const Quatf& q)
 }
 
 
-RotationElements Body::getRotationElements() const
+const RotationModel* Body::getRotationModel() const
 {
-    return rotationElements;
+    return rotationModel;
 }
 
-
-void Body::setRotationElements(const RotationElements& re)
+void Body::setRotationModel(const RotationModel* rm)
 {
-    rotationElements = re;
+    rotationModel = rm;
 }
 
 
@@ -260,127 +319,190 @@ void Body::setAtmosphere(const Atmosphere& _atmosphere)
 
 
 // Get a matrix which converts from local to heliocentric coordinates
-Mat4d Body::getLocalToHeliocentric(double when) const
+Mat4d Body::getLocalToHeliocentric(double tjd) const
 {
-    double ascendingNode = (double) rotationElements.ascendingNode +
-        rotationElements.precessionRate * (when - astro::J2000);
+    Point3d pos = orbit->positionAtTime(tjd);
 
-    Point3d pos = orbit->positionAtTime(when);
-    Mat4d frame;
-
-    switch (orbitRefPlane)
+    if (orbitFrame != NULL)
     {
-    case astro::BodyEquator:
-        frame = (Mat4d::xrotation(-rotationElements.obliquity) *
-                 Mat4d::yrotation(-ascendingNode) *
-                 Mat4d::translation(pos));
-        break;
-    case astro::Ecliptic_J2000:
-        frame = Mat4d::translation(pos);
-        break;
-    case astro::Equator_J2000:
-        frame = Mat4d::translation(pos);
-        break;
-    default:
-        assert(0);
-    }
- 
-    // Recurse up the hierarchy . . .
-    if (system != NULL && system->getPrimaryBody() != NULL)
-        frame = frame * system->getPrimaryBody()->getLocalToHeliocentric(when);
+        Point3d p = orbitFrame->convertFromAstrocentric(pos, tjd);
 
-    return frame;
+        // Temporary hack; won't be necessary post-1.5.0 when this function
+        // is redefined to return position with respect to frame root object
+        // instead of name hierarchy root.
+        if (frameRefStar != NULL)
+        {
+            Vec3d frameOffset(0.0, 0.0, 0.0);
+            Star* refStar = getReferenceStar();
+            if (refStar != NULL)
+            {
+                frameOffset = (frameRefStar->getPosition(tjd) - refStar->getPosition(tjd)) *
+                    astro::microLightYearsToKilometers(1.0);
+            }
+
+            return Mat4d::translation(p + frameOffset);
+        }
+        else
+        {
+            return Mat4d::translation(p);
+        }
+    }
+    else
+    {
+        Mat4d frame;
+
+        // TODO: inconsistent with orbitFrame != NULL case; shouldn't apply
+        // rotation model of body, only of parents.
+        frame = getRotationModel()->equatorOrientationAtTime(tjd).toMatrix4() *
+            Mat4d::translation(pos);
+
+        // Recurse up the hierarchy . . .
+        if (orbitBarycenter != NULL)
+            frame = frame * orbitBarycenter->getLocalToHeliocentric(tjd);
+        return frame;
+    }
 }
 
 
 // Return the position of the center of the body in heliocentric coordinates
-Point3d Body::getHeliocentricPosition(double when) const
+Point3d Body::getHeliocentricPosition(double tjd) const
 {
-    return Point3d(0.0, 0.0, 0.0) * getLocalToHeliocentric(when);
+    Point3d pos = orbit->positionAtTime(tjd);
+
+    if (orbitFrame != NULL)
+    {
+        Point3d p = orbitFrame->convertFromAstrocentric(pos, tjd);
+
+        // Temporary hack; won't be necessary post-1.5.0 when this function
+        // is redefined to return position with respect to frame root object
+        // instead of name hierarchy root.
+        if (frameRefStar != NULL)
+        {
+            Vec3d frameOffset(0.0, 0.0, 0.0);
+            Star* refStar = getReferenceStar();
+            if (refStar != NULL)
+            {
+                frameOffset = (frameRefStar->getPosition(tjd) - refStar->getPosition(tjd)) *
+                    astro::microLightYearsToKilometers(1.0);
+            }
+
+            return p + frameOffset;
+        }
+        else
+        {
+            return p;
+        }
+    }
+    else
+    {
+        // No orbit frame was specified; use the default orbit frame, which
+        // is typically the mean equatorial frame of the parent object. The
+        // exception is when the parent object is a star; in this case, the
+        // orbit frame is just the ecliptical frame.
+        if (orbitBarycenter != NULL)
+        {
+            // Object orbits a non-stellar object; get the parent's equatorial
+            // frame and convert to ecliptical coordinates.
+            Quatd orientation = orbitBarycenter->getEclipticalToEquatorial(tjd);
+            return orbitBarycenter->getHeliocentricPosition(tjd) +
+                pos * orientation.toMatrix3();
+        }
+        else
+        {
+            // Parent is a star; pos is already in ecliptical coordinates
+            return pos;
+        }
+    }
 }
 
 
-Quatd Body::getEclipticalToEquatorial(double when) const
+Quatd Body::getEclipticalToFrame(double tjd) const
 {
-    double ascendingNode = (double) rotationElements.ascendingNode +
-        rotationElements.precessionRate * (when - astro::J2000);
+    Quatd q(1.0);
 
-    Quatd q =
-        Quatd::xrotation(-rotationElements.obliquity) *
-        Quatd::yrotation(-ascendingNode);
-
-    // Recurse up the hierarchy . . .
-    if (system != NULL && system->getPrimaryBody() != NULL)
-        q = q * system->getPrimaryBody()->getEclipticalToEquatorial(when);
+    if (bodyFrame != NULL)
+    {
+        q = bodyFrame->getOrientation(tjd);
+    }
+    else
+    {
+        if (orbitBarycenter != NULL)
+            q = orbitBarycenter->getEclipticalToEquatorial(tjd);
+    }
 
     return q;
 }
 
 
-Quatd Body::getEclipticalToGeographic(double when) const
+Quatd Body::getEclipticalToEquatorial(double tjd) const
 {
-    return getEquatorialToGeographic(when) * getEclipticalToEquatorial(when);
+    Quatd q = getRotationModel()->equatorOrientationAtTime(tjd);
+        
+    if (bodyFrame != NULL)
+    {
+        return q * bodyFrame->getOrientation(tjd);
+    }
+    else
+    {
+        // Recurse up the hierarchy . . .
+        if (orbitBarycenter != NULL)
+            q = q * orbitBarycenter->getEclipticalToEquatorial(tjd);
+    }
+        
+    return q;
 }
 
 
-// The geographic coordinate system has an origin at the center of the
+Quatd Body::getEclipticalToBodyFixed(double when) const
+{
+    return getEquatorialToBodyFixed(when) * getEclipticalToEquatorial(when);
+}
+
+
+// The body-fixed coordinate system has an origin at the center of the
 // body, y-axis parallel to the rotation axis, x-axis through the prime
-// meridian, and z-axis at a right angle the xy plane.  An object with
-// constant geographic coordinates will thus remain fixed with respect
-// to a point on the surface of the body.
-Quatd Body::getEquatorialToGeographic(double when) const
+// meridian, and z-axis at a right angle the xy plane.
+Quatd Body::getEquatorialToBodyFixed(double when) const
 {
-    double t = when - rotationElements.epoch;
-    double rotations = t / (double) rotationElements.period;
-    double wholeRotations = floor(rotations);
-    double remainder = rotations - wholeRotations;
-
-    // Add an extra half rotation because of the convention in all
-    // planet texture maps where zero deg long. is in the middle of
-    // the texture.
-    remainder += 0.5;
-    
-    Quatd q(1);
-    q.yrotate(-remainder * 2 * PI - rotationElements.offset);
-    return q;
+    return rotationModel->spin(when);
 }
 
 
-Mat4d Body::getGeographicToHeliocentric(double when) const
+Mat4d Body::getBodyFixedToHeliocentric(double when) const
 {
-    return getEquatorialToGeographic(when).toMatrix4() *
+    return getEquatorialToBodyFixed(when).toMatrix4() *
         getLocalToHeliocentric(when);
 }
 
 
-Vec3f Body::planetocentricToCartesian(float lon, float lat, float alt) const
+Vec3d Body::planetocentricToCartesian(double lon, double lat, double alt) const
 {
-    float phi = -degToRad(lat) + (float) PI / 2;
-    float theta = degToRad(lon) - (float) PI;
+    double phi = -degToRad(lat) + PI / 2;
+    double theta = degToRad(lon) - PI;
 
-    Vec3f pos((float) (cos(theta) * sin(phi)),
-              (float) (cos(phi)),
-              (float) (-sin(theta) * sin(phi)));
+    Vec3d pos(cos(theta) * sin(phi),
+              cos(phi),
+              -sin(theta) * sin(phi));
 
     return pos * (getRadius() + alt);
 }
 
 
-Vec3f Body::planetocentricToCartesian(const Vec3f& lonLatAlt) const
+Vec3d Body::planetocentricToCartesian(const Vec3d& lonLatAlt) const
 {
     return planetocentricToCartesian(lonLatAlt.x, lonLatAlt.y, lonLatAlt.z);
 }
 
 
-Vec3f Body::cartesianToPlanetocentric(const Vec3f& v) const
+Vec3d Body::cartesianToPlanetocentric(const Vec3d& v) const
 {
-    Vec3f w = v;
+    Vec3d w = v;
     w.normalize();
 
-    double lat = (float) PI / 2.0f - acos(w.y);
+    double lat = PI / 2.0 - acos(w.y);
     double lon = atan2(w.z, -w.x);
 
-    return Vec3f((float) lon, (float) lat, v.length() - getRadius());
+    return Vec3d(lon, lat, v.length() - getRadius());
 }
 
 
@@ -445,6 +567,15 @@ float Body::getApparentMagnitude(const Star& sun,
 }
 
 
+float Body::getApparentMagnitude(float sunLuminosity,
+                                 float distanceFromSun,
+                                 float distanceFromViewer) const
+{
+    return astro::lumToAppMag(getLuminosity(sunLuminosity, distanceFromSun),
+                              astro::kilometersToLightYears(distanceFromViewer));
+}
+
+
 // Return the apparent magnitude of the body, corrected for the phase.
 float Body::getApparentMagnitude(const Star& sun,
                                  const Vec3d& sunPosition,
@@ -608,6 +739,95 @@ void Body::computeLocations()
 }
 
 
+bool
+Body::referenceMarkVisible(uint32 refmark) const
+{
+    return (referenceMarks & refmark) != 0;
+}
+
+
+uint32
+Body::getVisibleReferenceMarks() const
+{
+    return referenceMarks;
+}
+
+
+void
+Body::setVisibleReferenceMarks(uint32 refmarks)
+{
+	referenceMarks = refmarks;
+}
+
+
+// Get the star at the root of the name hierarchy
+// NOTE: This method shouldn't be required after cleanup of frame and name
+// hierarchies post 1.5.0.
+Star*
+Body::getReferenceStar() const
+{
+    const Body* body = this;
+
+    while (body->orbitBarycenter != NULL)
+        body = body->orbitBarycenter;
+
+    if (body->getSystem() != NULL)
+        return body->getSystem()->getStar();
+    else
+        return NULL;
+}
+
+
+// Get the star at the root of the frame hierarchy
+// NOTE: This method shouldn't be required after cleanup of frame and name
+// hierarchies post 1.5.0.
+Star*
+Body::getFrameReferenceStar() const
+{
+    const Body* body = this;
+
+    for (;;)
+    {
+        // Body has an orbit frame, parent is the frame center
+        if (body->orbitFrame != NULL)
+        {
+            if (body->orbitFrame->getCenter().star() != NULL)
+            {
+                return body->orbitFrame->getCenter().star();
+            }
+            else if (body->orbitFrame->getCenter().body() != NULL)
+            {
+                body = body->orbitFrame->getCenter().body();
+            }
+            else
+            {
+                // Bad frame center: either a location, deep sky object, or
+                // null object.
+                return NULL;
+            }
+        }
+        // Otherwise, use object's parent in the name hierarchy
+        else
+        {
+            if (body->orbitBarycenter != NULL)
+            {
+                body = body->orbitBarycenter;
+            }
+            else
+            {
+                if (body->getSystem() != NULL)
+                    return body->getSystem()->getStar();
+                else
+                    return NULL;
+            }
+        }
+    }
+
+    return NULL;
+}
+
+
+
 /**** Implementation of PlanetarySystem ****/
 
 PlanetarySystem::PlanetarySystem(Body* _primary) : primary(_primary)
@@ -627,6 +847,8 @@ PlanetarySystem::PlanetarySystem(Star* _star) :
 void PlanetarySystem::addBody(Body* body)
 {
     satellites.insert(satellites.end(), body);
+    objectIndex.insert(make_pair(body->getName(), body));
+    i18nObjectIndex.insert(make_pair(body->getName(true), body));
 }
 
 
@@ -641,6 +863,28 @@ void PlanetarySystem::removeBody(Body* body)
             break;
         }
     }
+
+    // Erase the object from the object indices
+    for (ObjectIndex::iterator iter = objectIndex.begin();
+         iter != objectIndex.end(); iter++)
+    {
+        if (iter->second == body)
+        {
+            objectIndex.erase(iter);
+            break;
+        }
+    }
+
+    for (ObjectIndex::iterator iter = i18nObjectIndex.begin();
+         iter != i18nObjectIndex.end(); iter++)
+    {
+        if (iter->second == body)
+        {
+            i18nObjectIndex.erase(iter);
+            break;
+        }
+    }
+
 }
 
 
@@ -655,23 +899,58 @@ void PlanetarySystem::replaceBody(Body* oldBody, Body* newBody)
             break;
         }
     }
+
+    // Erase the object from the object indices
+    for (ObjectIndex::iterator iter = objectIndex.begin();
+         iter != objectIndex.end(); iter++)
+    {
+        if (iter->second == oldBody)
+        {
+            objectIndex.erase(iter);
+            break;
+        }
+    }
+
+    for (ObjectIndex::iterator iter = i18nObjectIndex.begin();
+         iter != i18nObjectIndex.end(); iter++)
+    {
+        if (iter->second == oldBody)
+        {
+            i18nObjectIndex.erase(iter);
+            break;
+        }
+    }
+
+    // Add the replacement to the object indices
+    objectIndex.insert(make_pair(newBody->getName(), newBody));
+    i18nObjectIndex.insert(make_pair(newBody->getName(true), newBody));
 }
 
 
-Body* PlanetarySystem::find(string _name, bool deepSearch, bool i18n) const
+Body* PlanetarySystem::find(const string& _name, bool deepSearch, bool i18n) const
 {
-    for (vector<Body*>::const_iterator iter = satellites.begin();
-         iter != satellites.end(); iter++)
+    ObjectIndex index = i18n?i18nObjectIndex:objectIndex;
+    ObjectIndex::const_iterator firstMatch = index.find(_name);
+    if (firstMatch != index.end())
     {
-        if (UTF8StringCompare((*iter)->getName(i18n), _name) == 0)
+        return firstMatch->second;
+    }
+
+    if (deepSearch)
+    {
+        for (vector<Body*>::const_iterator iter = satellites.begin();
+             iter != satellites.end(); iter++)
         {
-            return *iter;
-        }
-        else if (deepSearch && (*iter)->getSatellites() != NULL)
-        {
-            Body* body = (*iter)->getSatellites()->find(_name, deepSearch, i18n);
-            if (body != NULL)
-                return body;
+            if (UTF8StringCompare((*iter)->getName(i18n), _name) == 0)
+            {
+                return *iter;
+            }
+            else if (deepSearch && (*iter)->getSatellites() != NULL)
+            {
+                Body* body = (*iter)->getSatellites()->find(_name, deepSearch, i18n);
+                if (body != NULL)
+                    return body;
+            }
         }
     }
 
@@ -700,11 +979,12 @@ bool PlanetarySystem::traverse(TraversalFunc func, void* info) const
 std::vector<std::string> PlanetarySystem::getCompletion(const std::string& _name, bool rec) const
 {
     std::vector<std::string> completion;
-      
+    int _name_length = UTF8Length(_name);
+
     for (vector<Body*>::const_iterator iter = satellites.begin();
          iter != satellites.end(); iter++)
     {
-        if (UTF8StringCompare((*iter)->getName(true), _name, _name.length()) == 0)
+        if (UTF8StringCompare((*iter)->getName(true), _name, _name_length) == 0)
         {
             completion.push_back((*iter)->getName(true));
         }
