@@ -13,13 +13,15 @@
 #include <cstdio>
 
 #include "celestia.h"
-// Ugh . . . the C++ standard says that stringstream should be in
-// sstream, but the GNU C++ compiler uses strstream instead.
-#ifdef HAVE_SSTREAM
+
+// Older gcc versions used <strstream> instead of <sstream>.
+// This has been corrected in GCC 3.2, but name clashing must
+// be avoided
+#ifdef __GNUC__
+#undef min
+#undef max
+#endif
 #include <sstream>
-#else
-#include <strstream>
-#endif // HAVE_SSTREAM
 
 #include <celutil/util.h>
 #include <celutil/debug.h>
@@ -144,7 +146,7 @@ Command* CommandParser::parseCommand()
     }
 
     string commandName = tokenizer->getStringValue();
-    
+
     Value* paramListValue = parser->readValue();
     if (paramListValue == NULL || paramListValue->getType() != Value::HashType)
     {
@@ -575,7 +577,9 @@ Command* CommandParser::parseCommand()
         paramList->getNumber("size", size);
         Vec3d colorv(1.0f, 0.0f, 0.0f);
         paramList->getVector("color", colorv);
-        Color color((float) colorv.x, (float) colorv.y, (float) colorv.z, 0.9f);
+        double alpha = 0.9f;
+        paramList->getNumber("alpha", alpha);
+        Color color((float) colorv.x, (float) colorv.y, (float) colorv.z, (float) alpha);
 
         Marker::Symbol symbol = Marker::Diamond;
         string symbolString;
@@ -583,17 +587,34 @@ Command* CommandParser::parseCommand()
         {
             if (compareIgnoringCase(symbolString, "diamond") == 0)
                 symbol = Marker::Diamond;
-            else if (compareIgnoringCase(symbolString, "square") == 0)
-                symbol = Marker::Square;
             else if (compareIgnoringCase(symbolString, "triangle") == 0)
                 symbol = Marker::Triangle;
+            else if (compareIgnoringCase(symbolString, "square") == 0)
+                symbol = Marker::Square;
+            else if (compareIgnoringCase(symbolString, "filledsquare") == 0)
+                symbol = Marker::FilledSquare;
             else if (compareIgnoringCase(symbolString, "plus") == 0)
                 symbol = Marker::Plus;
             else if (compareIgnoringCase(symbolString, "x") == 0)
                 symbol = Marker::X;
+            else if (compareIgnoringCase(symbolString, "leftarrow") == 0)
+                symbol = Marker::LeftArrow;
+            else if (compareIgnoringCase(symbolString, "rightarrow") == 0)
+                symbol = Marker::RightArrow;
+            else if (compareIgnoringCase(symbolString, "uparrow") == 0)
+                symbol = Marker::UpArrow;
+            else if (compareIgnoringCase(symbolString, "downarrow") == 0)
+                symbol = Marker::DownArrow;
+            else if (compareIgnoringCase(symbolString, "circle") == 0)
+                symbol = Marker::Circle;
+            else if (compareIgnoringCase(symbolString, "disk") == 0)
+                symbol = Marker::Disk;
         }
-        
-        cmd = new CommandMark(object, color, (float) size, symbol);
+
+        string label;
+        paramList->getString("label", label);
+
+        cmd = new CommandMark(object, color, (float) size, symbol, label);
     }
     else if (commandName == "unmark")
     {
@@ -610,7 +631,7 @@ Command* CommandParser::parseCommand()
         string type, filename;
         paramList->getString("type", type);
         paramList->getString("filename", filename);
-		
+
         cmd = new CommandCapture(type, filename);
     }
     else if (commandName == "renderpath")
@@ -629,7 +650,7 @@ Command* CommandParser::parseCommand()
             glcpath = GLContext::GLPath_NvCombiner_ARBVP;
         else if (compareIgnoringCase(path, "glsl") == 0)
             glcpath = GLContext::GLPath_GLSL;
-	
+
         cmd = new CommandRenderPath(glcpath);
     }
     else
@@ -646,11 +667,8 @@ Command* CommandParser::parseCommand()
 
 int parseRenderFlags(string s)
 {
-#ifdef HAVE_SSTREAM    
     istringstream in(s);
-#else
-    istrstream in(s.c_str());
-#endif
+
     Tokenizer tokenizer(&in);
     int flags = 0;
 
@@ -694,6 +712,10 @@ int parseRenderFlags(string s)
                 flags |= Renderer::ShowCelestialSphere;
             else if (compareIgnoringCase(name, "partialtrajectories") == 0)
                 flags |= Renderer::ShowPartialTrajectories;
+            else if (compareIgnoringCase(name, "openclusters") == 0)
+                flags |= Renderer::ShowOpenClusters;
+            else if (compareIgnoringCase(name, "cloudshadows") == 0)
+                flags |= Renderer::ShowCloudShadows;
 
             ttype = tokenizer.nextToken();
             if (ttype == Tokenizer::TokenBar)
@@ -712,11 +734,8 @@ int parseRenderFlags(string s)
 
 int parseLabelFlags(string s)
 {
-#ifdef HAVE_SSTREAM
     istringstream in(s);
-#else
-    istrstream in(s.c_str());
-#endif
+
     Tokenizer tokenizer(&in);
     int flags = 0;
 
@@ -742,6 +761,14 @@ int parseLabelFlags(string s)
                 flags |= Renderer::StarLabels;
 	    else if (compareIgnoringCase(name, "galaxies") == 0)
 		flags |= Renderer::GalaxyLabels;
+	    else if (compareIgnoringCase(name, "locations") == 0)
+		flags |= Renderer::LocationLabels;
+	    else if (compareIgnoringCase(name, "nebulae") == 0)
+		flags |= Renderer::NebulaLabels;
+	    else if (compareIgnoringCase(name, "openclusters") == 0)
+		flags |= Renderer::OpenClusterLabels;
+	    else if (compareIgnoringCase(name, "i18nconstellations") == 0)
+		flags |= Renderer::I18nConstellationLabels;
 
             ttype = tokenizer.nextToken();
             if (ttype == Tokenizer::TokenBar)
@@ -760,11 +787,8 @@ int parseLabelFlags(string s)
 
 int parseOrbitFlags(string s)
 {
-#ifdef HAVE_SSTREAM    
     istringstream in(s);
-#else
-    istrstream in(s.c_str());
-#endif
+
     Tokenizer tokenizer(&in);
     int flags = 0;
 
