@@ -30,7 +30,7 @@ CommandWait::~CommandWait()
 {
 }
 
-void CommandWait::process(ExecutionEnvironment& env, double t, double dt)
+void CommandWait::process(ExecutionEnvironment&, double, double)
 {
 }
 
@@ -79,7 +79,7 @@ void CommandGoto::process(ExecutionEnvironment& env)
 
 
 ////////////////
-// GotoLongLat command: go to the selected body and hover over 
+// GotoLongLat command: go to the selected body and hover over
 
 CommandGotoLongLat::CommandGotoLongLat(double t,
                                        double dist,
@@ -189,7 +189,7 @@ void CommandSynchronous::process(ExecutionEnvironment& env)
 
 
 ////////////////
-// Chase command: 
+// Chase command:
 
 CommandChase::CommandChase()
 {
@@ -202,7 +202,7 @@ void CommandChase::process(ExecutionEnvironment& env)
 
 
 ////////////////
-// Track command: 
+// Track command:
 
 CommandTrack::CommandTrack()
 {
@@ -304,7 +304,7 @@ CommandClearScreen::CommandClearScreen()
 {
 }
 
-void CommandClearScreen::process(ExecutionEnvironment& env)
+void CommandClearScreen::process(ExecutionEnvironment&)
 {
 }
 
@@ -316,7 +316,7 @@ CommandExit::CommandExit()
 {
 }
 
-void CommandExit::process(ExecutionEnvironment& env)
+void CommandExit::process(ExecutionEnvironment&)
 {
     exit(0);
 }
@@ -357,7 +357,7 @@ CommandChangeDistance::CommandChangeDistance(double _duration, double _rate) :
 {
 }
 
-void CommandChangeDistance::process(ExecutionEnvironment& env, double t, double dt)
+void CommandChangeDistance::process(ExecutionEnvironment& env, double, double dt)
 {
     env.getSimulation()->changeOrbitDistance((float) (rate * dt));
 }
@@ -372,7 +372,7 @@ CommandOrbit::CommandOrbit(double _duration, const Vec3f& axis, float rate) :
 {
 }
 
-void CommandOrbit::process(ExecutionEnvironment& env, double t, double dt)
+void CommandOrbit::process(ExecutionEnvironment& env, double, double dt)
 {
     float v = spin.length();
     if (v != 0.0f)
@@ -390,7 +390,7 @@ CommandRotate::CommandRotate(double _duration, const Vec3f& axis, float rate) :
 {
 }
 
-void CommandRotate::process(ExecutionEnvironment& env, double t, double dt)
+void CommandRotate::process(ExecutionEnvironment& env, double, double dt)
 {
     float v = spin.length();
     if (v != 0.0f)
@@ -408,7 +408,7 @@ CommandMove::CommandMove(double _duration, const Vec3d& _velocity) :
 {
 }
 
-void CommandMove::process(ExecutionEnvironment& env, double t, double dt)
+void CommandMove::process(ExecutionEnvironment& env, double, double dt)
 {
     env.getSimulation()->setObserverPosition(env.getSimulation()->getObserver().getPosition() + (velocity * dt));
 }
@@ -445,7 +445,7 @@ void CommandSetOrientation::process(ExecutionEnvironment& env)
 ////////////////
 // Look back command: reverse observer orientation
 
-CommandLookBack::CommandLookBack() 
+CommandLookBack::CommandLookBack()
 {
 }
 
@@ -522,12 +522,6 @@ CommandSetVisibilityLimit::CommandSetVisibilityLimit(double mag) :
 
 void CommandSetVisibilityLimit::process(ExecutionEnvironment& env)
 {
-    Renderer* r = env.getRenderer();
-    if (r != NULL)
-    {
-        r->setBrightnessBias(0.05f);
-        r->setSaturationMagnitude(1.0f);
-    }
     env.getSimulation()->setFaintestVisible((float) magnitude);
 }
 ////////////////
@@ -606,11 +600,12 @@ void CommandSet::process(ExecutionEnvironment& env)
 // Mark object command
 
 CommandMark::CommandMark(const string& _target, Color _color, float _size,
-                         Marker::Symbol _symbol) :
+                         Marker::Symbol _symbol, const string& _label) :
     target(_target),
     color(_color),
     size(_size),
-    symbol(_symbol)
+    symbol(_symbol),
+    label(_label)
 {
 }
 
@@ -621,7 +616,7 @@ void CommandMark::process(ExecutionEnvironment& env)
         return;
 
     if (env.getSimulation()->getUniverse() != NULL)
-        env.getSimulation()->getUniverse()->markObject(sel, size, color, symbol, 1);
+        env.getSimulation()->getUniverse()->markObject(sel, size, color, symbol, 1, label);
 }
 
 
@@ -687,15 +682,15 @@ CommandCapture::CommandCapture(const std::string& _type,
 {
 }
 
-void CommandCapture::process(ExecutionEnvironment& env)
+void CommandCapture::process(ExecutionEnvironment&)
 {
     bool success = false;
-#ifndef MACOSX
+#ifndef TARGET_OS_MAC
 
     // Get the dimensions of the current viewport
     int viewport[4];
     glGetIntegerv(GL_VIEWPORT, viewport);
-    
+
     if (compareIgnoringCase(type, "jpeg") == 0)
     {
         success = CaptureGLBufferToJPEG(filename,
@@ -737,8 +732,8 @@ void CommandRenderPath::process(ExecutionEnvironment& env)
 
 RepeatCommand::RepeatCommand(CommandSequence* _body, int _repeatCount) :
     body(_body),
-    repeatCount(_repeatCount),
     bodyDuration(0.0),
+    repeatCount(_repeatCount),
     execution(NULL)
 {
     for (CommandSequence::const_iterator iter = body->begin();
