@@ -14,6 +14,7 @@
 #include "celestia.h"
 #include <celmath/mathlib.h>
 #include <celmath/perlin.h>
+#include <celmath/intersect.h>
 #include "astro.h"
 #include "galaxy.h"
 #include <celutil/util.h>
@@ -25,9 +26,10 @@
 
 using namespace std;
 
-static Color colorTable[256];
 
-static const unsigned int GALAXY_POINTS  = 7000;
+static int width = 128, height = 128;
+static Color colorTable[256];
+static const unsigned int GALAXY_POINTS  = 3500;
 
 static bool formsInitialized = false;
 
@@ -38,9 +40,14 @@ static GalacticForm*  irregularForm   = NULL;
 static Texture* galaxyTex = NULL;
 
 static void InitializeForms();
+static GalacticForm* buildGalacticForms(const std::string& filename);
 
 float Galaxy::lightGain  = 0.0f;
 
+bool operator < (const Blob& b1, const Blob& b2)
+{
+  return (b1.position.distanceFromOrigin() < b2.position.distanceFromOrigin());
+}
 
 struct GalaxyTypeName
 {
@@ -48,7 +55,7 @@ struct GalaxyTypeName
     Galaxy::GalaxyType type;
 };
 
-static GalaxyTypeName GalaxyTypeNames[] = 
+static GalaxyTypeName GalaxyTypeNames[] =
 {
     { "S0",  Galaxy::S0 },
     { "Sa",  Galaxy::Sa },
@@ -69,10 +76,9 @@ static GalaxyTypeName GalaxyTypeNames[] =
 };
 
 
-static void GalaxyTextureEval(float u, float v, float w,
-                              unsigned char *pixel)
+static void GalaxyTextureEval(float u, float v, float /*w*/, unsigned char *pixel)
 {
-    float r = 0.9f - (float) sqrt(u * u + v * v);
+    float r = 0.9f - (float) sqrt(u * u + v * v );
     if (r < 0)
         r = 0;
 
@@ -86,6 +92,7 @@ static void GalaxyTextureEval(float u, float v, float w,
 
 Galaxy::Galaxy() :
     detail(1.0f),
+    customTmpName(NULL),
     form(NULL)
 {
 }
@@ -103,11 +110,30 @@ void Galaxy::setDetail(float d)
 }
 
 
+void Galaxy::setCustomTmpName(const string& tmpNameStr)
+{
+    if (customTmpName == NULL)
+        customTmpName = new string(tmpNameStr);
+    else
+        *customTmpName = tmpNameStr;
+}
+
+
+string Galaxy::getCustomTmpName() const
+{
+    if (customTmpName == NULL)
+        return "";
+    else
+        return *customTmpName;
+}
+
+
 const char* Galaxy::getType() const
 {
     return GalaxyTypeNames[(int) type].name;
 }
 
+
 void Galaxy::setType(const string& typeStr)
 {
     type = Galaxy::Irr;
@@ -122,40 +148,46 @@ void Galaxy::setType(const string& typeStr)
 
     if (!formsInitialized)
         InitializeForms();
-    switch (type)
-    {
-    case S0:
-    case Sa:
-    case Sb:
-    case Sc:
-    case SBa:
-    case SBb:
-    case SBc:
-        form = spiralForms[type - S0];
-        break;
-    case E0:
-    case E1:
-    case E2:
-    case E3:
-    case E4:
-    case E5:
-    case E6:
-    case E7:
-        form = ellipticalForms[type - E0];
-        //form = NULL;
-        break;
-    case Irr:
-        form = irregularForm;
-        break;    
-    }
-} 
+
+	if (customTmpName != NULL)
+	{
+			form = buildGalacticForms("models/" + *customTmpName);
+	}
+	else
+	{
+		switch (type)
+		{
+		case S0:
+		case Sa:
+		case Sb:
+		case Sc:
+		case SBa:
+		case SBb:
+		case SBc:
+			form = spiralForms[type - S0];
+			break;
+		case E0:
+		case E1:
+		case E2:
+		case E3:
+		case E4:
+		case E5:
+		case E6:
+		case E7:
+			form = ellipticalForms[type - E0];
+			//form = NULL;
+			break;
+		case Irr:
+			form = irregularForm;
+			break;
+		}
+	}
+}
 
 
 size_t Galaxy::getDescription(char* buf, size_t bufLength) const
 {
-    // Should use snprintf, but it's not available on Windows; the
-    // buffer *must* be large enough.
-    return sprintf(buf, _("Galaxy (Hubble type: %s)"), getType());
+    return snprintf(buf, bufLength, _("Galaxy (Hubble type: %s)"), getType());
 }
 
 
@@ -164,6 +196,38 @@ GalacticForm* Galaxy::getForm() const
     return form;
 }
 
+const char* Galaxy::getObjTypeName() const
+{
+    return "galaxy";
+}
+
+
+// TODO: This value is just a guess.
+// To be optimal, it should actually be computed:
+static const float RADIUS_CORRECTION     = 0.025f;
+static const float MAX_SPIRAL_THICKNESS  = 0.06f;
+
+bool Galaxy::pick(const Ray3d& ray,
+                  double& distanceToPicker,
+                  double& cosAngleToBoundCenter) const
+{
+    // The ellipsoid should be slightly larger to compensate for the fact
+    // that blobs are considered points when galaxies are built, but have size
+    // when they are drawn.
+    float yscale = (type < E0 )? MAX_SPIRAL_THICKNESS: form->scale.y + RADIUS_CORRECTION;
+    Vec3d ellipsoidAxes(getRadius()*(form->scale.x + RADIUS_CORRECTION),
+                        getRadius()* yscale,
+                        getRadius()*(form->scale.z + RADIUS_CORRECTION));
+
+    Quatf qf= getOrientation();
+    Quatd qd(qf.w, qf.x, qf.y, qf.z);
+
+    return testIntersection(Ray3d(Point3d() + (ray.origin - getPosition()), ray.direction) * conjugate(qd).toMatrix3(),
+                            Ellipsoidd(ellipsoidAxes),
+                            distanceToPicker,
+                            cosAngleToBoundCenter);
+}
+
 
 bool Galaxy::load(AssociativeArray* params, const string& resPath)
 {
@@ -171,10 +235,14 @@ bool Galaxy::load(AssociativeArray* params, const string& resPath)
     params->getNumber("Detail", detail);
     setDetail((float) detail);
 
+	string customTmpName;
+	if(params->getString("CustomTemplate",customTmpName))
+		setCustomTmpName(customTmpName);
+
     string typeName;
     params->getString("Type", typeName);
     setType(typeName);
-    
+
     return DeepSkyObject::load(params, resPath);
 }
 
@@ -192,7 +260,7 @@ void Galaxy::render(const GLContext& context,
 }
 
 
-void Galaxy::renderGalaxyPointSprites(const GLContext& context,
+void Galaxy::renderGalaxyPointSprites(const GLContext&,
                                       const Vec3f& offset,
                                       const Quatf& viewerOrientation,
                                       float brightness,
@@ -201,9 +269,23 @@ void Galaxy::renderGalaxyPointSprites(const GLContext& context,
     if (form == NULL)
         return;
 
+    /* We'll first see if the galaxy's apparent size is big enough to
+       be noticeable on screen; if it's not we'll break right here,
+       avoiding all the overhead of the matrix transformations and
+       GL state changes: */
+        float distanceToDSO = offset.length() - getRadius();
+        if (distanceToDSO < 0)
+            distanceToDSO = 0;
+
+        float minimumFeatureSize = pixelSize * distanceToDSO;
+        float size  = 2 * getRadius();
+
+        if (size < minimumFeatureSize)
+            return;
+
     if (galaxyTex == NULL)
     {
-        galaxyTex = CreateProceduralTexture(128, 128, GL_RGBA,
+        galaxyTex = CreateProceduralTexture(width, height, GL_RGBA,
                                             GalaxyTextureEval);
     }
     assert(galaxyTex != NULL);
@@ -217,17 +299,10 @@ void Galaxy::renderGalaxyPointSprites(const GLContext& context,
     Vec3f v2 = Vec3f( 1,  1, 0) * viewMat;
     Vec3f v3 = Vec3f(-1,  1, 0) * viewMat;
 
-    float distanceToDSO = offset.length() - getRadius();
-    if (distanceToDSO < 0)
-        distanceToDSO = 0;
-
-    float minimumFeatureSize = pixelSize * distanceToDSO;
-
     //Mat4f m = (getOrientation().toMatrix4() *
     //           Mat4f::scaling(form->scale) *
     //           Mat4f::scaling(getRadius()));
 
-    float  size  = 2 * getRadius();
     Mat3f m =
         Mat3f::scaling(form->scale)*getOrientation().toMatrix3()*Mat3f::scaling(size);
 
@@ -238,7 +313,6 @@ void Galaxy::renderGalaxyPointSprites(const GLContext& context,
 
     vector<Blob>* points = form->blobs;
     unsigned int nPoints = (unsigned int) (points->size() * clamp(getDetail()));
-
     // corrections to avoid excessive brightening if viewed e.g. edge-on
 
     float brightness_corr = 1.0f;
@@ -252,46 +326,41 @@ void Galaxy::renderGalaxyPointSprites(const GLContext& context,
         if (brightness_corr < 0.2f)
             brightness_corr = 0.2f;
     }
-
     if (type > E3) // only elliptics with higher ellipticities
     {
         cosi = Vec3f(1,0,0) * getOrientation().toMatrix3()
                             * offset/offset.length();
-        brightness_corr = brightness_corr * (float) sqrt(abs((cosi)));
-        if (brightness_corr < 0.65f)
-            brightness_corr = 0.65f;
+        brightness_corr = brightness_corr * (float) abs((cosi));
+        if (brightness_corr < 0.45f)
+            brightness_corr = 0.45f;
     }
 
     glBegin(GL_QUADS);
     for (unsigned int i = 0; i < nPoints; ++i)
     {
-        Blob    b    = (*points)[i];
-        Point3f p    = b.position * m;            
-        float   br   = b.brightness;
-
-        // Reddening for elliptical galaxies
-
-        b.colorIndex = type < E0 ? b.colorIndex : (unsigned int) ceil(0.3f*b.colorIndex);
-        Color   c    = colorTable[b.colorIndex];     // lookup static color table
-          
-        Point3f relPos = p + offset;
-  
         if ((i & pow2) != 0)
         {
             pow2 <<= 1;
-            size /= 1.57f;
+            size /= 1.55f;
             if (size < minimumFeatureSize)
                 break;
-        }          
-            
+        }
+
+        Blob    b  = (*points)[i];
+        Point3f p  = b.position * m;
+        float   br = b.brightness / 255.0f;
+
+        Color   c      = colorTable[b.colorIndex];     // lookup static color table
+        Point3f relPos = p + offset;
+
         float screenFrac = size / relPos.distanceFromOrigin();
-          
         if (screenFrac < 0.1f)
         {
-            float a  = 3.5f * (0.1f - screenFrac) * brightness_corr * brightness * br;
-            
+            float btot = ((type > SBc) && (type < Irr))? 2.5f: 5.0f;
+            float a  = btot * (0.1f - screenFrac) * brightness_corr * brightness * br;
+
             glColor4f(c.red(), c.green(), c.blue(), (4.0f*lightGain + 1.0f)*a);
-              
+
             glTexCoord2f(0, 0);          glVertex(p + (v0 * size));
             glTexCoord2f(1, 0);          glVertex(p + (v1 * size));
             glTexCoord2f(1, 1);          glVertex(p + (v2 * size));
@@ -304,8 +373,8 @@ void Galaxy::renderGalaxyPointSprites(const GLContext& context,
 
 void Galaxy::renderGalaxyEllipsoid(const GLContext& context,
                                    const Vec3f& offset,
-                                   const Quatf& viewerOrientation,
-                                   float brightness,
+                                   const Quatf&,
+                                   float,
                                    float pixelSize)
 {
     float discSizeInPixels = pixelSize * getRadius() / offset.length();
@@ -318,7 +387,7 @@ void Galaxy::renderGalaxyEllipsoid(const GLContext& context,
     if (vproc == NULL)
         return;
 
-    int e = min(max((int) type - (int) E0, 0), 7);
+    //int e = min(max((int) type - (int) E0, 0), 7);
     Vec3f scale = Vec3f(1.0f, 0.9f, 1.0f) * getRadius();
     Vec3f eyePos_obj = -offset * (~getOrientation()).toMatrix3();
 
@@ -346,7 +415,7 @@ void Galaxy::renderGalaxyEllipsoid(const GLContext& context,
             float theta = (float) (PI * 2) * (float) j / (float) nSlices;
             float sinTheta = (float) sin(theta);
             float cosTheta = (float) cos(theta);
-            
+
             glVertex3f((float) cos(phi0) * cosTheta * scale.x,
                        (float) sin(phi0)            * scale.y,
                        (float) cos(phi0) * sinTheta * scale.z);
@@ -379,21 +448,21 @@ void Galaxy::increaseLightGain()
     lightGain  += 0.05f;
     if (lightGain > 1.0f)
         lightGain  = 1.0f;
-} 
+}
 
 
 void Galaxy::decreaseLightGain()
 {
     lightGain  -= 0.05f;
     if (lightGain < 0.0f)
-        lightGain  = 0.0f;  
-} 
+        lightGain  = 0.0f;
+}
 
 
 float Galaxy::getLightGain()
 {
     return lightGain;
-} 
+}
 
 
 void Galaxy::setLightGain(float lg)
@@ -454,7 +523,8 @@ void Galaxy::hsv2rgb( float *r, float *g, float *b, float h, float s, float v )
      *g = p;
      *b = v;
      break;
-   default:              *r = v;
+   default:
+     *r = v;
      *g = p;
      *b = q;
      break;
@@ -464,64 +534,127 @@ void Galaxy::hsv2rgb( float *r, float *g, float *b, float h, float s, float v )
 
 GalacticForm* buildGalacticForms(const std::string& filename)
 {
-    unsigned int galaxySize = GALAXY_POINTS;
-    Blob b;    
-    vector<Blob>* galacticPoints = new vector<Blob>;
-    galacticPoints->reserve(galaxySize);
+	Blob b;
+	vector<Blob>* galacticPoints = new vector<Blob>;
 
-    ifstream inFile(filename.c_str());    
-    while (inFile.good())
-    {
-        float x, y, z, br;
-        inFile >> x;
-        inFile >> y;
-        inFile >> z;
-        inFile >> br;
-      
-        b.position    = Point3f(x, y, z);
-        b.brightness  = br;
-        b.colorIndex  = (unsigned int) (b.position.distanceFromOrigin() * 255);
-      
-        galacticPoints->push_back(b);
-    }          
-    
-    GalacticForm* galacticForm  = new GalacticForm();
-    galacticForm->blobs         = galacticPoints;
-    galacticForm->scale         = Vec3f(1.0f, 1.0f, 1.0f);
-    
-    return galacticForm;
-};
+	// Load templates in standard .png format
+	int width, height, rgb, j = 0, kmin = 9;
+	unsigned char value;
+	float h = 0.75f;
+	Image* img;
+	img = LoadPNGImage(filename);
+	if (img == NULL)
+	{
+	cout<<"\nThe galaxy template *** "<<filename<<" *** could not be loaded!\n\n";
+	return NULL;
+	}
+	width  = img->getWidth();
+	height = img->getHeight();
+	rgb    = img->getComponents();
+
+		for (int i = 0; i < width * height; i++)
+		{
+			value = img->getPixels()[rgb * i];
+			if (value > 10)
+			{
+				float x, y, z, r2, yy, prob;
+				z  = floor(i /(float) width);
+				x  = (i - width * z - 0.5f * (width - 1)) / (float) width;
+				z  = (0.5f * (height - 1) - z) / (float) height;
+				x  += Mathf::sfrand() * 0.008f;
+				z  += Mathf::sfrand() * 0.008f;
+				r2 = x * x + z * z;
+
+				if ( strcmp ( filename.c_str(), "models/E0.png") != 0 )
+				{
+					float y0 = 0.5f * MAX_SPIRAL_THICKNESS * sqrt((float)value/256.0f) * exp(- 5.0f * r2);
+					float B, yr;
+					B = (r2 > 0.35f)? 1.0f: 0.75f; // the darkness of the "dust lane", 0 < B < 1
+					float p0 = 1.0f - B * exp(-h * h); // the uniform reference probability, envelopping prob*p0.
+					do
+					{
+						// generate "thickness" y of spirals with emulation of a dust lane
+						// in galctic plane (y=0)
+
+						yr =  Mathf::sfrand() * h;
+						prob = (1.0f - B * exp(-yr * yr))/p0;
+
+					} while (Mathf::frand() > prob);
+					b.brightness  = value * prob;
+					y = y0 * yr / h;
+				}
+				else
+				{
+					// generate spherically symmetric distribution from E0.png
+					do
+					{
+						yy = Mathf::sfrand();
+						float ry2 = 1.0f - yy * yy;
+						prob = ry2 > 0? sqrt(ry2): 0.0f;
+					} while (Mathf::frand() > prob);
+					y = yy * sqrt(0.25f - r2) ;
+					b.brightness  = value;
+					kmin = 12;
+				}
+
+				b.position    = Point3f(x, y, z);
+				unsigned int rr =  (unsigned int) (b.position.distanceFromOrigin() * 511);
+				b.colorIndex  = rr < 256? rr: 255;
+				galacticPoints->push_back(b);
+				j++;
+			 }
+		}
+
+    delete img;
+	galacticPoints->reserve(j);
+
+	// sort to start with the galaxy center region (x^2 + y^2 + z^2 ~ 0), such that
+	// the biggest (brightest) sprites will be localized there!
+
+	sort(galacticPoints->begin(), galacticPoints->end());
+
+	// reshuffle the galaxy points randomly...except the first kmin+1 in the center!
+	// the higher that number the stronger the central "glow"
+
+	random_shuffle( galacticPoints->begin() + kmin, galacticPoints->end());
+
+	GalacticForm* galacticForm  = new GalacticForm();
+	galacticForm->blobs         = galacticPoints;
+	galacticForm->scale         = Vec3f(1.0f, 1.0f, 1.0f);
+
+	return galacticForm;
+}
 
 
 void InitializeForms()
 {
-    unsigned int i = 0;
-
     // build color table:
-    for (i = 0; i < 256; ++i)
+
+    for (unsigned int i = 0; i < 256; i++)
     {
-        float rr,gg,bb;
-        float rho  = (float) i / 255.0f;
+        float rr, gg, bb;
+        //
         // generic Hue profile as deduced from true-color imaging for spirals
-        float h = 30.0f * (float) tanh(15.68 * (0.1086 - rho));
-        if (rho > 0.1086f)
-            h += 260.0f;
+        // Hue in degrees
+
+        float hue = (i < 28)? 25 * tanh(0.0615f * (27 - i)): 25 * tanh(0.0615f * (27 - i)) + 220;
+
         //convert Hue to RGB
-        Galaxy::hsv2rgb( &rr, &gg, &bb, h , 0.15f, 1.0f);
-        colorTable[i]  = Color( rr,gg,bb);
+
+        Galaxy::hsv2rgb(&rr, &gg, &bb, hue, 0.20f, 1.0f);
+        colorTable[i]  = Color(rr, gg, bb);
     }
     // Spiral Galaxies, 7 classical Hubble types
-    
-    spiralForms   = new GalacticForm*[7];
-	
-    spiralForms[Galaxy::S0]   = buildGalacticForms("models/S0.pts");
-    spiralForms[Galaxy::Sa]   = buildGalacticForms("models/Sa.pts");
-    spiralForms[Galaxy::Sb]   = buildGalacticForms("models/Sb.pts");
-    spiralForms[Galaxy::Sc]   = buildGalacticForms("models/Sc.pts");
-    spiralForms[Galaxy::SBa]  = buildGalacticForms("models/SBa.pts");
 
-    spiralForms[Galaxy::SBb]  = buildGalacticForms("models/SBb.pts");
-    spiralForms[Galaxy::SBc]  = buildGalacticForms("models/SBc.pts");
+    spiralForms   = new GalacticForm*[7];
+
+    spiralForms[Galaxy::S0]   = buildGalacticForms("models/S0.png");
+    spiralForms[Galaxy::Sa]   = buildGalacticForms("models/Sa.png");
+    spiralForms[Galaxy::Sb]   = buildGalacticForms("models/Sb.png");
+    spiralForms[Galaxy::Sc]   = buildGalacticForms("models/Sc.png");
+    spiralForms[Galaxy::SBa]  = buildGalacticForms("models/SBa.png");
+    spiralForms[Galaxy::SBb]  = buildGalacticForms("models/SBb.png");
+    spiralForms[Galaxy::SBc]  = buildGalacticForms("models/SBc.png");
 
     // Elliptical Galaxies , 8 classical Hubble types, E0..E7,
     //
@@ -529,27 +662,36 @@ void InitializeForms()
     // via rescaling by (1.0f, 3.8f, 1.0f).
 
     ellipticalForms = new GalacticForm*[8];
-
     for (unsigned int eform  = 0; eform <= 7; ++eform)
     {
         float ell = 1.0f - (float) eform / 8.0f;
-        ellipticalForms[eform] = new GalacticForm();
-        ellipticalForms[eform]->blobs = spiralForms[Galaxy::S0]->blobs;
-        // note the correct x,y-alignment of 'ell' scaling!!
-        ellipticalForms[eform]->scale = Vec3f(ell, 3.8*ell, 1.0f);
-    }
 
+        // note the correct x,y-alignment of 'ell' scaling!!
+   		// build all elliptical templates from rescaling E0
+
+   		ellipticalForms[eform] = buildGalacticForms("models/E0.png");
+   		if (*ellipticalForms)
+   			ellipticalForms[eform]->scale = Vec3f(ell, ell, 1.0f);
+
+        // account for reddening of ellipticals rel.to spirals
+        if (*ellipticalForms)
+        {
+        	unsigned int nPoints = (unsigned int) (ellipticalForms[eform]->blobs->size());
+			for (unsigned int i = 0; i < nPoints; ++i)
+    		{
+            	(*ellipticalForms[eform]->blobs)[i].colorIndex = (unsigned int) ceil(0.76f * (*ellipticalForms[eform]->blobs)[i].colorIndex);
+        	}
+        }
+    }
     //Irregular Galaxies
-    unsigned int galaxySize = GALAXY_POINTS;
-    Blob b;    
-    Point3f p;    
-    
-    
+    unsigned int galaxySize = GALAXY_POINTS, ip = 0;
+    Blob b;
+    Point3f p;
+
     vector<Blob>* irregularPoints = new vector<Blob>;
     irregularPoints->reserve(galaxySize);
-    
-    i = 0;
-    while (i < galaxySize)
+
+    while (ip < galaxySize)
     {
         p        = Point3f(Mathf::sfrand(), Mathf::sfrand(), Mathf::sfrand());
         float r  = p.distanceFromOrigin();
@@ -559,16 +701,17 @@ void InitializeForms()
             if (Mathf::frand() < prob)
             {
                 b.position   = p;
-                b.brightness = 1.0f;  
-                b.colorIndex = (unsigned int) (r*255);
+                b.brightness = 64u;
+                unsigned int rr =  (unsigned int) (r * 511);
+        	    b.colorIndex  = rr < 256? rr: 255;
                 irregularPoints->push_back(b);
-                ++i;
+                ++ip;
             }
         }
     }
     irregularForm        = new GalacticForm();
     irregularForm->blobs = irregularPoints;
-    irregularForm->scale = Vec3f(1.0f, 0.1f, 1.0f);
+    irregularForm->scale = Vec3f(0.5f, 0.5f, 0.5f);
 
     formsInitialized = true;
 }
