@@ -1,6 +1,6 @@
 // lodspheremesh.cpp
-// 
-// Copyright (C) 2000, Chris Laurel <claurel@shatters.net>
+//
+// Copyright (C) 2000-2007, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@ using namespace std;
 
 //#define SHOW_PATCH_VISIBILITY
 //#define SHOW_FRUSTUM
+//#define VERTEX_BUFFER_OBJECTS_ENABLED
 
 static bool trigArraysInitialized = false;
 static int maxDivisions = 16384;
@@ -218,10 +219,10 @@ void LODSphereMesh::render(const GLContext& context,
     int minSplit = 1;
     for (i = 0; i < nTextures; i++)
     {
-        float pixelsPerTexel = pixWidth * 2.0f / 
+        float pixelsPerTexel = pixWidth * 2.0f /
             ((float) tex[i]->getWidth() / 2.0f);
         double l = log(pixelsPerTexel) / log(2.0);
-        
+
         ri.texLOD[i] = max(min(tex[i]->getLODCount() - 1, (int) l), 0);
         if (tex[i]->getUTileCount(ri.texLOD[i]) > minSplit)
             minSplit = tex[i]->getUTileCount(ri.texLOD[i]);
@@ -274,7 +275,7 @@ void LODSphereMesh::render(const GLContext& context,
             glx::glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
 
             glx::glGenBuffersARB(1, &indexBuffer);
-            
+
             useVertexBuffers = true;
 
             // HACK: delete the user arrays--we shouldn't need to allocate
@@ -290,7 +291,7 @@ void LODSphereMesh::render(const GLContext& context,
         glx::glBindBufferARB(GL_ARRAY_BUFFER_ARB, vertexBuffers[currentVB]);
     }
 
-    // Set up the mesh vertices 
+    // Set up the mesh vertices
     int nRings = phiExtent / ri.step;
     int nSlices = thetaExtent / ri.step;
 
@@ -347,7 +348,7 @@ void LODSphereMesh::render(const GLContext& context,
     else
     {
         // Render the sphere section by section.
-        int reject = 0;
+        /*int reject = 0;   Unused*/
 
         // Compute the vertices of the view frustum.  These will be used for
         // culling patches.
@@ -529,7 +530,7 @@ void LODSphereMesh::render(const GLContext& context,
 }
 
 
-int LODSphereMesh::renderPatches(int phi0, int theta0, 
+int LODSphereMesh::renderPatches(int phi0, int theta0,
                                  int extent,
                                  int level,
                                  const RenderInfo& ri)
@@ -669,8 +670,8 @@ void LODSphereMesh::renderSection(int phi0, int theta0, int extent,
     int phiExtent = extent / 2;
     int theta1 = theta0 + thetaExtent;
     int phi1 = phi0 + phiExtent;
-    int n3 = 0;
-    int n2 = 0;
+    /*int n3 = 0;   Unused*/
+    /*int n2 = 0;   Unused*/
 
     float du[MAX_SPHERE_MESH_TEXTURES];
     float dv[MAX_SPHERE_MESH_TEXTURES];
@@ -680,6 +681,15 @@ void LODSphereMesh::renderSection(int phi0, int theta0, int extent,
 
     if (useVertexBuffers)
     {
+        // Calling glBufferDataARB() with NULL before mapping the buffer
+        // is a hint to OpenGL that previous contents of vertex buffer will
+        // be discarded and overwritten. It enables renaming in the driver,
+        // hopefully resulting in performance gains.
+        glx::glBufferDataARB(GL_ARRAY_BUFFER_ARB,
+                             maxVertices * vertexSize * sizeof(float),
+                             NULL,
+                             GL_STREAM_DRAW_ARB);
+
         vertices = reinterpret_cast<float*>(glx::glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB));
         if (vertices == NULL)
             return;
@@ -822,4 +832,3 @@ void LODSphereMesh::renderSection(int phi0, int theta0, int extent,
         glx::glBindBufferARB(GL_ARRAY_BUFFER_ARB, vertexBuffers[currentVB]);
     }
 }
-
