@@ -1,6 +1,6 @@
 // meshmanager.cpp
 //
-// Copyright (C) 2001 Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2006 Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -96,18 +96,36 @@ Model* ModelInfo::load(const string& filename)
         model = LoadCelestiaMesh(filename);
     }
 
+    // Condition the model for optimal rendering
     if (model != NULL)
     {
+        // Many models tend to have a lot of duplicate materials; eliminate
+        // them, since unnecessarily setting material parameters can adversely
+        // impact rendering performance. Ideally uniquification of materials
+        // would be performed just once when the model was created, but
+        // that's not the case.
+        uint32 originalMaterialCount = model->getMaterialCount();
+        model->uniquifyMaterials();
+
         // Sort the submeshes roughly by opacity.  This will eliminate a
         // good number of the errors caused when translucent triangles are
         // rendered before geometry that they cover.
-        model->sortMeshes(Model::OpacityComparator(*model));
+        model->sortMeshes(Model::OpacityComparator());
+
+        model->determineOpacity();
+
+        // Display some statics for the model
+        clog << _("   Model statistics: ")
+             << model->getVertexCount() << _(" vertices, ")
+             << model->getPrimitiveCount() << _(" primitives, ")
+             << originalMaterialCount << _(" materials ")
+             << "(" << model->getMaterialCount() << _(" unique)\n");
     }
     else
     {
         cerr << _("Error loading model '") << filename << "'\n";
     }
- 
+
     return model;
 }
 
@@ -139,6 +157,7 @@ static float NoiseDisplacementFunc(float u, float v, void* info)
 }
 
 
+// TODO: The Celestia mesh format is deprecated
 Model* LoadCelestiaMesh(const string& filename)
 {
     ifstream meshFile(filename.c_str(), ios::in);
@@ -180,9 +199,9 @@ Model* LoadCelestiaMesh(const string& filename)
     }
 
     Hash* meshDef = meshDefValue->getHash();
-    
+
     NoiseMeshParameters params;
-    
+
     params.size = Vec3f(1, 1, 1);
     params.offset = Vec3f(10, 10, 10);
     params.featureHeight = 0.0f;
@@ -229,7 +248,7 @@ static VertexList* ConvertToVertexList(M3DTriangleMesh& mesh,
     if (nTexCoords == nVertices)
         parts |= VertexList::TexCoord0;
     VertexList* vl = new VertexList(parts);
-    
+
     Vec3f* faceNormals = new Vec3f[nFaces];
     Vec3f* vertexNormals = new Vec3f[nFaces * 3];
     int* faceCounts = new int[nVertices];
@@ -361,13 +380,17 @@ static VertexList* ConvertToVertexList(M3DTriangleMesh& mesh,
                     M3DColor specular = material->getSpecularColor();
                     vl->setSpecularColor(Color(specular.red, specular.green, specular.blue));
                     float shininess = material->getShininess();
-                    
-                    // Map the shininess from the 3DS file into the 0-128
-                    // range that OpenGL uses for the specular exponent.
-                    shininess = (float) pow(2.0, 10.0 * shininess);
+
+                    // Map the 3DS file's shininess from percentage (0-100) to
+                    // range that OpenGL uses for the specular exponent. The
+                    // current equation is just a guess at the mapping that
+                    // 3DS actually uses.
+                    shininess = (float) pow(2.0, 1.0 + 0.1 * shininess);
+                    //shininess = 2.0f + shininess;
+                    //clog << materialName << ": shininess=" << shininess << ", color=" << specular.red << "," << specular.green << "," << specular.blue << '\n';
                     if (shininess > 128.0f)
                         shininess = 128.0f;
-                    vl->setShininess(128.0f);
+                    vl->setShininess(shininess);
 
                     if (material->getTextureMap() != "")
                     {
@@ -403,7 +426,9 @@ static VertexList* ConvertToVertexList(M3DTriangleMesh& mesh,
 
 
 static Mesh*
-ConvertVertexListToMesh(VertexList* vlist, const string& texPath, uint32 material)
+ConvertVertexListToMesh(VertexList* vlist,
+                        const string& /*texPath*/,      //TODO: remove parameter??
+                        uint32 material)
 {
     Mesh::VertexAttribute attributes[8];
     uint32 nAttributes = 0;
