@@ -26,6 +26,7 @@ static Color DefaultSpecular(0.0f, 0.0f, 0.0f);
 static Color DefaultEmissive(0.0f, 0.0f, 0.0f);
 static float DefaultSpecularPower = 1.0f;
 static float DefaultOpacity = 1.0f;
+static Mesh::BlendMode DefaultBlend = Mesh::NormalBlend;
 
 
 /*!
@@ -55,12 +56,15 @@ defined here--they have the obvious definitions.
                           emissive <color>  |
                           specpower <float> |
                           opacity <float>   |
+                          blend <blendmode> |
                           <texture>
 
 <color>               ::= <float> <float> <float>
 
 <string>              ::= """ { letter } """
 
+<blendmode>           ::= normal | add
+
 <mesh_definition>     ::= mesh
                           <vertex_description>
                           <vertex_pool>
@@ -74,7 +78,8 @@ defined here--they have the obvious definitions.
 <vertex_attribute>    ::= <vertex_semantic> <vertex_format>
 
 <vertex_semantic>     ::= position | normal | color0 | color1 | tangent |
-                          texcoord0 | texcoord1 | texcoord2 | texcoord3
+                          texcoord0 | texcoord1 | texcoord2 | texcoord3 |
+                          pointsize
 
 <vertex_format>       ::= f1 | f2 | f3 | f4 | ub4
 
@@ -87,7 +92,8 @@ defined here--they have the obvious definitions.
                           { <unsigned_int> }
 
 <prim_group_type>     ::= trilist | tristrip | trifan |
-                          linelist | linestrip | points
+                          linelist | linestrip | points |
+                          sprites
 
 <material_index>      :: <unsigned_int> | -1
 \endcode
@@ -129,7 +135,7 @@ private:
                        uint32 nVertices,
                        uint32 stride,
                        const Mesh::VertexDescription& desc);
-    
+
     ostream& out;
 };
 
@@ -171,7 +177,7 @@ private:
                        uint32 nVertices,
                        uint32 stride,
                        const Mesh::VertexDescription& desc);
-    
+
     ostream& out;
 };
 
@@ -193,7 +199,7 @@ ModelLoader::reportError(const string& msg)
 }
 
 
-const string& 
+const string&
 ModelLoader::getErrorMessage() const
 {
     return errorMessage;
@@ -340,6 +346,28 @@ AsciiModelLoader::loadMaterial()
 
             material->maps[texType] = tex;
         }
+        else if (property == "blend")
+        {
+            Mesh::BlendMode blendMode = Mesh::InvalidBlend;
+
+            if (tok.nextToken() == Tokenizer::TokenName)
+            {
+                string blendModeName = tok.getNameValue();
+                if (blendModeName == "normal")
+                    blendMode = Mesh::NormalBlend;
+                else if (blendModeName == "add")
+                    blendMode = Mesh::AdditiveBlend;
+            }
+
+            if (blendMode == Mesh::InvalidBlend)
+            {
+                reportError("Bad blend mode in material");
+                delete material;
+                return NULL;
+            }
+
+            material->blend = blendMode;
+        }
         else
         {
             // All non-texture material properties are 3-vectors except
@@ -363,7 +391,7 @@ AsciiModelLoader::loadMaterial()
             Color colorVal;
             if (nValues == 3)
                 colorVal = Color((float) data[0], (float) data[1], (float) data[2]);
-            
+
             if (property == "diffuse")
                 material->diffuse = colorVal;
             else if (property == "specular")
@@ -375,7 +403,7 @@ AsciiModelLoader::loadMaterial()
             else if (property == "specpower")
                 material->specularPower = (float) data[0];
         }
-    }    
+    }
 
     if (tok.getTokenType() != Tokenizer::TokenName)
     {
@@ -445,7 +473,7 @@ AsciiModelLoader::loadVertexDescription()
             return NULL;
         }
 
-        Mesh::VertexAttributeFormat format = 
+        Mesh::VertexAttributeFormat format =
             Mesh::parseVertexAttributeFormat(formatName);
         if (format == Mesh::InvalidFormat)
         {
@@ -477,7 +505,10 @@ AsciiModelLoader::loadVertexDescription()
         return NULL;
     }
 
-    return new Mesh::VertexDescription(offset, nAttributes, attributes);
+    Mesh::VertexDescription *vertexDesc =
+        new Mesh::VertexDescription(offset, nAttributes, attributes);
+    delete[] attributes;
+    return vertexDesc;
 }
 
 
@@ -522,7 +553,7 @@ AsciiModelLoader::loadVertices(const Mesh::VertexDescription& vertexDesc,
         for (uint32 attr = 0; attr < vertexDesc.nAttributes; attr++)
         {
             Mesh::VertexAttributeFormat fmt = vertexDesc.attributes[attr].format;
-            uint32 nBytes = Mesh::getVertexAttributeSize(fmt);
+            /*uint32 nBytes = Mesh::getVertexAttributeSize(fmt);    Unused*/
             int readCount = 0;
             switch (fmt)
             {
@@ -550,10 +581,12 @@ AsciiModelLoader::loadVertices(const Mesh::VertexDescription& vertexDesc,
                 if (tok.nextToken() != Tokenizer::TokenNumber)
                 {
                     reportError("Error in vertex data");
-                    delete[] vertexData;
+                    data[j] = 0.0;
+                }
+                else
+                {
+                    data[j] = tok.getNumberValue();
                 }
-                data[j] = tok.getNumberValue();
-
                 // TODO: range check unsigned byte values
             }
 
@@ -577,7 +610,7 @@ AsciiModelLoader::loadVertices(const Mesh::VertexDescription& vertexDesc,
     return vertexData;
 }
 
-                               
+
 Mesh*
 AsciiModelLoader::loadMesh()
 {
@@ -587,7 +620,7 @@ AsciiModelLoader::loadMesh()
         reportError("Mesh definition expected");
         return NULL;
     }
-    
+
     Mesh::VertexDescription* vertexDesc = loadVertexDescription();
     if (vertexDesc == NULL)
         return NULL;
@@ -595,11 +628,15 @@ AsciiModelLoader::loadMesh()
     uint32 vertexCount = 0;
     char* vertexData = loadVertices(*vertexDesc, vertexCount);
     if (vertexData == NULL)
+    {
+        delete vertexDesc;
         return NULL;
+    }
 
     Mesh* mesh = new Mesh();
     mesh->setVertexDescription(*vertexDesc);
     mesh->setVertices(vertexCount, vertexData);
+    delete vertexDesc;
 
     while (tok.nextToken() == Tokenizer::TokenName &&
            tok.getNameValue() != "end_mesh")
@@ -622,7 +659,7 @@ AsciiModelLoader::loadMesh()
 
         uint32 materialIndex;
         if (tok.getNumberValue() == -1.0)
-            materialIndex = ~0;
+            materialIndex = ~0u;
         else
             materialIndex = (uint32) tok.getNumberValue();
 
@@ -634,7 +671,7 @@ AsciiModelLoader::loadMesh()
         }
 
         uint32 indexCount = (uint32) tok.getNumberValue();
-        
+
         uint32* indices = new uint32[indexCount];
         if (indices == NULL)
         {
@@ -742,7 +779,7 @@ AsciiModelLoader::load()
             return NULL;
         }
     }
-     
+
     return model;
 }
 
@@ -797,10 +834,12 @@ AsciiModelWriter::writeGroup(const Mesh::PrimitiveGroup& group)
         out << "linestrip"; break;
     case Mesh::PointList:
         out << "points"; break;
+    case Mesh::SpriteList:
+        out << "sprites"; break;
     default:
         return;
     }
-    
+
     out << ' ' << group.materialIndex << ' ' << group.nIndices << '\n';
 
     // Print the indices, twelve per line
@@ -928,6 +967,9 @@ AsciiModelWriter::writeVertexDescription(const Mesh::VertexDescription& desc)
         case Mesh::Texture3:
             out << "texcoord3";
             break;
+        case Mesh::PointSize:
+            out << "pointsize";
+            break;
         default:
             assert(0);
             break;
@@ -997,6 +1039,24 @@ AsciiModelWriter::writeMaterial(const Mesh::Material& material)
     if (material.opacity != DefaultOpacity)
         out << "opacity " << material.opacity << '\n';
 
+    if (material.blend != DefaultBlend)
+    {
+        out << "blend ";
+        switch (material.blend)
+        {
+        case Mesh::NormalBlend:
+            out << "normal";
+            break;
+        case Mesh::AdditiveBlend:
+            out << "add";
+            break;
+        default:
+            assert(0);
+            break;
+        }
+        out << "\n";
+    }
+
     for (int i = 0; i < Mesh::TextureSemanticMax; i++)
     {
         const TextureInfo* texInfo = GetTextureManager()->getResourceInfo(material.maps[i]);
@@ -1019,7 +1079,7 @@ AsciiModelWriter::writeMaterial(const Mesh::Material& material)
             default:
                 assert(0);
             }
-            
+
             out << " \"" << texInfo->source << "\"\n";
         }
     }
@@ -1079,8 +1139,10 @@ static int32 readUint(istream& in)
 
 static float readFloat(istream& in)
 {
-    int i = readUint(in);
-    return *((float*) &i);
+    float f;
+    in.read((char*) &f, sizeof(float));
+    LE_TO_CPU_FLOAT(f, f);
+    return f;
 }
 
 
@@ -1256,7 +1318,7 @@ BinaryModelLoader::load()
             return NULL;
         }
     }
-     
+
     return model;
 }
 
@@ -1322,6 +1384,19 @@ BinaryModelLoader::loadMaterial()
             }
             break;
 
+        case CMOD_Blend:
+            {
+                int16 blendMode = readInt16(in);
+                if (blendMode < 0 || blendMode >= Mesh::BlendMax)
+                {
+                    reportError("Bad blend mode");
+                    delete material;
+                    return NULL;
+                }
+                material->blend = (Mesh::BlendMode) blendMode;
+            }
+            break;
+
         case CMOD_Texture:
             {
                 int16 texType = readInt16(in);
@@ -1348,11 +1423,11 @@ BinaryModelLoader::loadMaterial()
                 }
 
                 ResourceHandle tex = GetTextureManager()->getHandle(TextureInfo(texfile, getTexturePath(), TextureInfo::WrapTexture));
-                
+
                 material->maps[texType] = tex;
             }
             break;
-            
+
         case CMOD_EndMaterial:
             return material;
 
@@ -1385,7 +1460,7 @@ BinaryModelLoader::loadVertexDescription()
     for (;;)
     {
         int16 tok = readInt16(in);
-        
+
         if (tok == CMOD_EndVertexDesc)
         {
             break;
@@ -1410,7 +1485,7 @@ BinaryModelLoader::loadVertexDescription()
 
                 attributes[nAttributes].semantic =
                     static_cast<Mesh::VertexAttributeSemantic>(tok);
-                attributes[nAttributes].format = 
+                attributes[nAttributes].format =
                     static_cast<Mesh::VertexAttributeFormat>(fmt);
                 attributes[nAttributes].offset = offset;
 
@@ -1433,7 +1508,10 @@ BinaryModelLoader::loadVertexDescription()
         return NULL;
     }
 
-    return new Mesh::VertexDescription(offset, nAttributes, attributes);
+    Mesh::VertexDescription *vertexDesc =
+        new Mesh::VertexDescription(offset, nAttributes, attributes);
+    delete[] attributes;
+    return vertexDesc;
 }
 
 
@@ -1447,16 +1525,20 @@ BinaryModelLoader::loadMesh()
     uint32 vertexCount = 0;
     char* vertexData = loadVertices(*vertexDesc, vertexCount);
     if (vertexData == NULL)
+    {
+        delete vertexDesc;
         return NULL;
+    }
 
     Mesh* mesh = new Mesh();
     mesh->setVertexDescription(*vertexDesc);
     mesh->setVertices(vertexCount, vertexData);
+    delete vertexDesc;
 
     for (;;)
     {
         int16 tok = readInt16(in);
-        
+
         if (tok == CMOD_EndMesh)
         {
             break;
@@ -1530,7 +1612,7 @@ BinaryModelLoader::loadVertices(const Mesh::VertexDescription& vertexDesc,
         {
             uint32 base = offset + vertexDesc.attributes[attr].offset;
             Mesh::VertexAttributeFormat fmt = vertexDesc.attributes[attr].format;
-            int readCount = 0;
+            /*int readCount = 0;    Unused*/
             switch (fmt)
             {
             case Mesh::Float1:
@@ -1781,6 +1863,12 @@ BinaryModelWriter::writeMaterial(const Mesh::Material& material)
         writeTypeFloat1(out, material.opacity);
     }
 
+    if (material.blend != DefaultBlend)
+    {
+        writeToken(out, CMOD_Blend);
+        writeInt16(out, (int16) material.blend);
+    }
+
     for (int i = 0; i < Mesh::TextureSemanticMax; i++)
     {
         if (material.maps[i] != InvalidResource)
