@@ -1,6 +1,6 @@
 // observer.cpp
 //
-// Copyright (C) 2001, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2006, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -12,6 +12,9 @@
 #include "observer.h"
 #include "simulation.h"
 
+static const double maximumSimTime = 730486721060.00073; // 2000000000 Jan 01 12:00:00 UTC
+static const double minimumSimTime = -730498278941.99951; // -2000000000 Jan 01 12:00:00 UTC
+
 using namespace std;
 
 
@@ -50,7 +53,7 @@ Observer::Observer() :
     trackingOrientation(1.0f, 0.0f, 0.0f, 0.0f),
     fov((float) (PI / 4.0)),
     reverseFlag(false),
-    locationFilter(~0)
+    locationFilter(~0u)
 {
 }
 
@@ -152,7 +155,7 @@ RigidTransform Observer::getSituation() const
 {
     return frame.toUniversal(situation, getTime());
 }
-                        
+
 
 
 void Observer::setSituation(const RigidTransform& xform)
@@ -174,15 +177,15 @@ Vec3d toUniversal(const Vec3d& v,
             Quatf q = observer.getOrientation();
             Quatd qd(q.w, q.x, q.y, q.z);
             return v * qd.toMatrix3();
-        }                        
+        }
+
 
-        
     case astro::Geographic:
         if (sel.getType() != Selection::Type_Body)
             return v;
         else
-            return v * sel.body()->getGeographicToHeliocentric(t);
-        
+            return v * sel.body()->getBodyFixedToHeliocentric(t);
+
     case astro::Equatorial:
         if (sel.getType() != Selection::Type_Body)
             return v;
@@ -204,14 +207,22 @@ Vec3d toUniversal(const Vec3d& v,
 }
 
 
-static Quatf lookAt(Point3f from, Point3f to, Vec3f up)
+/*! Determine an orientation that will make the negative z-axis point from
+ *  from the observer to the target, with the y-axis pointing in direction
+ *  of the component of 'up' that is orthogonal to the z-axis.
+ */
+// TODO: This is a generally useful function that should be moved to
+// the celmath package.
+template<class T> static Quaternion<T> 
+lookAt(Point3<T> from, Point3<T> to, Vector3<T> up)
 {
-    Vec3f n = to - from;
+    Vector3<T> n = to - from;
     n.normalize();
-    Vec3f v = n ^ up;
+    Vector3<T> v = n ^ up;
     v.normalize();
-    Vec3f u = v ^ n;
-    return Quatf(Mat3f(v, u, -n));
+    Vector3<T> u = v ^ n;
+
+    return Quaternion<T>::matrixToQuaternion(Matrix3<T>(v, u, -n));
 }
 
 
@@ -230,9 +241,18 @@ void Observer::update(double dt, double timeScale)
     realTime += dt;
     simTime += (dt / 86400.0) * timeScale;
 
+    if (simTime >= maximumSimTime)
+        simTime = maximumSimTime;
+    if (simTime <= minimumSimTime)
+        simTime = minimumSimTime;
+
     if (observerMode == Travelling)
     {
-        float t = (float) clamp((realTime - journey.startTime) / journey.duration);
+        // Compute the fraction of the trip that has elapsed; handle zero
+        // durations correctly by skipping directly to the destination.
+        float t = 1.0;
+        if (journey.duration > 0)
+            t = (float) clamp((realTime - journey.startTime) / journey.duration);
 
         Vec3d jv = journey.to - journey.from;
         UniversalCoord p;
@@ -332,7 +352,7 @@ void Observer::update(double dt, double timeScale)
                     Quatd q1(journey.rotation1.w, journey.rotation1.x,
                              journey.rotation1.y, journey.rotation1.z);
                     p = origin + v0 * Quatd::slerp(q0, q1, t).toMatrix3();
-                                              
+
                     p = frame.fromUniversal(RigidTransform(p, Quatf(1.0f)),
                                             simTime).translation;
                 }
@@ -397,7 +417,13 @@ void Observer::update(double dt, double timeScale)
     if (getVelocity() != targetVelocity)
     {
         double t = clamp((realTime - beginAccelTime) / VELOCITY_CHANGE_TIME);
-        setVelocity(getVelocity() * (1.0 - t) + targetVelocity * t);
+        Vec3d v = getVelocity() * (1.0 - t) + targetVelocity * t;
+
+        // At some threshold, we just set the velocity to zero; otherwise,
+        // we'll end up with ridiculous velocities like 10^-40 m/s.
+        if (v.length() < 1.0e-12)
+            v = Vec3d(0.0, 0.0, 0.0);
+        setVelocity(v);
     }
 
     // Update the position
@@ -416,9 +442,11 @@ void Observer::update(double dt, double timeScale)
     if (!trackObject.empty())
     {
         Vec3f up = Vec3f(0, 1, 0) * getOrientation().toMatrix3();
-        Vec3d vn = trackObject.getPosition(getTime()) - getPosition();
-        Point3f to((float) vn.x, (float) vn.y, (float) vn.z);
-        setOrientation(lookAt(Point3f(0, 0, 0), to, up));
+        Vec3d viewDir = trackObject.getPosition(getTime()) - getPosition();
+        
+        setOrientation(lookAt(Point3d(0, 0, 0),
+                              Point3d(viewDir.x, viewDir.y, viewDir.z),
+                              Vec3d(up.x, up.y, up.z)));
     }
 }
 
@@ -679,44 +707,18 @@ void Observer::computeCenterCOParameters(const Selection& destination,
     Vec3d w(wf.x, wf.y, wf.z);
     v.normalize();
 
-    Vec3d n = w ^ v;
-    double nl = n.length();
-    double angle = 0.0;
-    if (nl > 0.0)
-    {
-        double cosAngle = w * v;
-        if (cosAngle < 1.0 - 1e-8)
-        {
-            if (cosAngle > 1e-8 - 1.0)
-                angle = acos(cosAngle);
-            else
-                angle = PI;
-        }
-        else
-        {
-            angle = 0.0;
-        }
-
-        n = n / nl;
-    }
-    else
-    {
-        n = Vec3d(1.0, 0.0, 0.0);
-    }
-
     Selection centerObj = frame.refObject;
     UniversalCoord centerPos = centerObj.getPosition(getTime());
     UniversalCoord targetPosition = destination.getPosition(getTime());
 
-    Quatd qd;
-    qd.setAxisAngle(n, -angle);
+    Quatd qd = Quatd::vecToVecRotation(v, w);
     Quatf q((float) qd.w, (float) qd.x, (float) qd.y, (float) qd.z);
 
     jparams.from = getPosition();
     jparams.to = centerPos + ((getPosition() - centerPos) * qd.toMatrix3());
     jparams.initialOrientation = getOrientation();
     jparams.finalOrientation = getOrientation() * q;
-    
+
     jparams.startInterpolation = 0.0;
     jparams.endInterpolation = 1.0;
 
@@ -873,7 +875,7 @@ void Observer::changeOrbitDistance(const Selection& selection, float d)
     if (!center.empty())
     {
         UniversalCoord focusPosition = center.getPosition(getTime());
-        
+
         double size = center.radius();
 
         // Somewhat arbitrary parameters to chosen to give the camera movement
@@ -965,9 +967,29 @@ static double getPreferredDistance(const Selection& selection)
         return 5.0 * selection.radius();
     case Selection::Type_Star:
         if (selection.star()->getVisibility())
+        {
             return 100.0 * selection.radius();
+        }
         else
-            return astro::AUtoKilometers(1.0);
+        {
+            double maxOrbitRadius = 0.0;
+            const vector<Star*>* orbitingStars = selection.star()->getOrbitingStars();
+            if (orbitingStars != NULL)
+            {
+                for (vector<Star*>::const_iterator iter = orbitingStars->begin();
+                     iter != orbitingStars->end(); iter++)
+                {
+                    Orbit* orbit = (*iter)->getOrbit();
+                    if (orbit != NULL)
+                        maxOrbitRadius = max(orbit->getBoundingRadius(), maxOrbitRadius);
+                }
+            }
+
+            if (maxOrbitRadius == 0.0)
+                return astro::AUtoKilometers(1.0);
+            else
+                return maxOrbitRadius * 5.0;
+        }
     case Selection::Type_Location:
         {
             double maxDist = getPreferredDistance(selection.location()->getParentBody());
@@ -1026,8 +1048,8 @@ void Observer::gotoSelection(const Selection& selection,
 // through the middle of a planet.
 void Observer::gotoSelectionGC(const Selection& selection,
                                double gotoTime,
-                               double startInter,
-                               double endInter,
+                               double /*startInter*/,       //TODO: remove parameter??
+                               double /*endInter*/,         //TODO: remove parameter??
                                Vec3f up,
                                astro::CoordinateSystem upFrame)
 {
@@ -1141,13 +1163,13 @@ void Observer::gotoLocation(const RigidTransform& transform,
 {
     journey.startTime = realTime;
     journey.duration = duration;
-    
+
     RigidTransform from(getPosition(), getOrientation());
     from = frame.fromUniversal(from, getTime());
     journey.from = from.translation;
     journey.initialOrientation= Quatf((float) from.rotation.w, (float) from.rotation.x,
                                       (float) from.rotation.y, (float) from.rotation.z);
-    
+
     journey.to = transform.translation;
     journey.finalOrientation = Quatf((float) transform.rotation.w,
                                      (float) transform.rotation.x,
@@ -1208,7 +1230,7 @@ void Observer::gotoSurface(const Selection& sel, double duration)
     else
     {
     }
-    
+
     FrameOfReference frame(astro::Geographic, sel);
     RigidTransform rt = frame.fromUniversal(RigidTransform(getPosition(), q),
                                             getTime());
