@@ -1,6 +1,6 @@
 // rendcontext.h
 //
-// Copyright (C) 2004, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2004-2006, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -11,45 +11,7 @@
 #define _CELENGINE_RENDCONTEXT_H_
 
 #include "mesh.h"
-
-
-static const unsigned int MaxLights = 8;
-
-struct DirectionalLight
-{
-    Color color;
-    float irradiance;
-    Vec3f direction_eye;
-    Vec3f direction_obj;
-
-    // Required for eclipse shadows only--may be able to use
-    // distance instead of position.
-    Point3d position;
-    float apparentSize;
-};
-
-struct EclipseShadow
-{
-    Point3f origin;
-    Vec3f direction;
-    float penumbraRadius;
-    float umbraRadius;
-};
-
-struct LightingState
-{
-    LightingState() : nLights(0),
-                      eyeDir_obj(0.0f, 0.0f, -1.0f),
-                      eyePos_obj(0.0f, 0.0f, -1.0f)
-    { shadows[0] = NULL; };
-
-    unsigned int nLights;
-    DirectionalLight lights[MaxLights];
-    std::vector<EclipseShadow>* shadows[MaxLights];
-
-    Vec3f eyeDir_obj;
-    Point3f eyePos_obj;
-};
+#include "shadermanager.h"
 
 
 class RenderContext
@@ -57,12 +19,14 @@ class RenderContext
  public:
     RenderContext(const Mesh::Material*);
     RenderContext();
+    virtual ~RenderContext() {};
 
     virtual void makeCurrent(const Mesh::Material&) = 0;
     virtual void setVertexArrays(const Mesh::VertexDescription& desc,
                                  void* vertexData) = 0;
     virtual void drawGroup(const Mesh::PrimitiveGroup& group);
 
+    const Mesh::Material* getMaterial() const;
     void setMaterial(const Mesh::Material*);
     void lock() { locked = true; }
     void unlock() { locked = false; }
@@ -76,11 +40,20 @@ class RenderContext
 
     RenderPass getRenderPass() const { return renderPass; }
     void setRenderPass(RenderPass rp) { renderPass = rp; }
+
+    void setPointScale(float);
+    float getPointScale() const;
     
  private:
     const Mesh::Material* material;
     bool locked;
     RenderPass renderPass;
+    float pointScale;
+
+ protected:
+    bool usePointSize;
+    bool useNormals;
+    bool useColors;
 };
 
 
@@ -89,14 +62,17 @@ class FixedFunctionRenderContext : public RenderContext
  public:
     FixedFunctionRenderContext(const Mesh::Material*);
     FixedFunctionRenderContext();
+    virtual ~FixedFunctionRenderContext();
 
     virtual void makeCurrent(const Mesh::Material&);
     virtual void setVertexArrays(const Mesh::VertexDescription& desc,
                                  void* vertexData);
+    void setLighting(bool);
 
  private:
-    bool blendOn;
+    Mesh::BlendMode blendMode;
     bool specularOn;
+    bool lightingEnabled;
 };
 
 
@@ -127,17 +103,56 @@ class VP_Combiner_RenderContext : public RenderContext
 class GLSL_RenderContext : public RenderContext
 {
  public:
-    GLSL_RenderContext(const LightingState& ls);
-    GLSL_RenderContext(const LightingState& ls,
-		       const Mesh::Material*);
+    GLSL_RenderContext(const LightingState& ls, float _objRadius, const Mat4f& _xform);
+    virtual ~GLSL_RenderContext();
     
     virtual void makeCurrent(const Mesh::Material&);
     virtual void setVertexArrays(const Mesh::VertexDescription& desc,
-				 void* vertexData);
+                                 void* vertexData);
+                 
+    virtual void setLunarLambert(float);
+    virtual void setAtmosphere(const Atmosphere*);
 
+ private:
+     void initLightingEnvironment();
+     void setLightingParameters(CelestiaGLProgram& prog, Color diffuseColor, Color specularColor);
+     void setShadowParameters(CelestiaGLProgram& prog);
+     
  private:
     const LightingState& lightingState;
+    const Atmosphere* atmosphere;
+    Mesh::BlendMode blendMode;
+    float objRadius;
+    Mat4f xform;
+    
+    // extended material properties
+    float lunarLambert;
+    
+    ShaderProperties shaderProps;
 };
 
+
+class GLSLUnlit_RenderContext : public RenderContext
+{
+ public:
+    GLSLUnlit_RenderContext(float _objRadius);
+    virtual ~GLSLUnlit_RenderContext();
+    
+    virtual void makeCurrent(const Mesh::Material&);
+    virtual void setVertexArrays(const Mesh::VertexDescription& desc,
+                                 void* vertexData);
+
+ private:
+     void initLightingEnvironment();
+     void setLightingParameters(CelestiaGLProgram& prog, Color diffuseColor, Color specularColor);
+     
+ private:
+    Mesh::BlendMode blendMode;
+    float objRadius;
+    
+    ShaderProperties shaderProps;
+};
+
+
 #endif // _CELENGINE_RENDCONTEXT_H_
 
