@@ -1,5 +1,5 @@
 // simulation.h
-// 
+//
 // Copyright (C) 2001, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
@@ -41,7 +41,7 @@ class Simulation
     void render(Renderer&);
     void render(Renderer&, Observer&);
 
-    Selection pickObject(Vec3f pickRay, float tolerance = 0.0f);
+    Selection pickObject(Vec3f pickRay, int renderFlags, float tolerance = 0.0f);
 
     Universe* getUniverse() const;
 
@@ -98,6 +98,8 @@ class Simulation
     bool getSyncTime() const;
     void setSyncTime(bool);
     void synchronizeTime();
+    bool getPauseState() const;
+    void setPauseState(bool);
 
     float getFaintestVisible() const;
     void setFaintestVisible(float);
@@ -115,6 +117,7 @@ class Simulation
  private:
     double realTime;
     double timeScale;
+    double storedTimeScale;
     bool syncTime;
 
     Universe* universe;
@@ -126,6 +129,7 @@ class Simulation
     std::vector<Observer*> observers;
 
     float faintestVisible;
+    bool pauseState;
 };
 
 #endif // _CELENGINE_SIMULATION_H_
