@@ -1,6 +1,6 @@
 // solarsys.cpp
 //
-// Copyright (C) 2001-2004 Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2006 Chris Laurel <claurel@shatters.net>
 //
 // Solar system catalog parser.
 //
@@ -14,9 +14,9 @@
 #include <cstdio>
 
 #ifndef _WIN32
-#ifndef MACOSX_PB
+#ifndef TARGET_OS_MAC
 #include <config.h>
-#endif /* ! MACOSX_PB */
+#endif /* ! TARGET_OS_MAC */
 #endif /* ! _WIN32 */
 
 #include <celutil/debug.h>
@@ -88,6 +88,16 @@ static void sscError(const Tokenizer& tok,
 }
 
 
+//! Maximum depth permitted for nested frames.
+static unsigned int MaxFrameDepth = 50;
+
+static bool isFrameCircular(const ReferenceFrame& frame, ReferenceFrame::FrameType frameType)
+{
+    return frame.nestingDepth(MaxFrameDepth, frameType) > MaxFrameDepth;
+}
+
+
+
 static Location* CreateLocation(Hash* locationData,
                                 Body* body)
 {
@@ -96,10 +106,10 @@ static Location* CreateLocation(Hash* locationData,
     Vec3d longlat(0.0, 0.0, 0.0);
     locationData->getVector("LongLat", longlat);
 
-    Vec3f position = body->planetocentricToCartesian((float) longlat.x,
-                                                     (float) longlat.y,
-                                                     (float) longlat.z);
-    location->setPosition(position);
+    Vec3d position = body->planetocentricToCartesian(longlat.x,
+                                                     longlat.y,
+                                                     longlat.z);
+    location->setPosition(Vec3f((float) position.x, (float) position.y, (float) position.z));
 
     double size = 1.0;
     locationData->getNumber("Size", size);
@@ -123,6 +133,7 @@ static void FillinSurface(Hash* surfaceData,
 {
     surfaceData->getColor("Color", surface->color);
 
+    // Haze is deprecated; used only in pre-OpenGL 2.0 render paths
     Color hazeColor = surface->hazeColor;
     float hazeDensity = hazeColor.alpha();
     if (surfaceData->getColor("HazeColor", hazeColor) | surfaceData->getNumber("HazeDensity", hazeDensity))
@@ -134,6 +145,8 @@ static void FillinSurface(Hash* surfaceData,
     surfaceData->getColor("SpecularColor", surface->specularColor);
     surfaceData->getNumber("SpecularPower", surface->specularPower);
 
+    surfaceData->getNumber("LunarLambert", surface->lunarLambert);
+
     string baseTexture;
     string bumpTexture;
     string nightTexture;
@@ -153,7 +166,7 @@ static void FillinSurface(Hash* surfaceData,
     unsigned int bumpFlags = TextureInfo::WrapTexture | TextureInfo::AllowSplitting;
     unsigned int nightFlags = TextureInfo::WrapTexture | TextureInfo::AllowSplitting;
     unsigned int specularFlags = TextureInfo::WrapTexture | TextureInfo::AllowSplitting;
-    
+
     float bumpHeight = 2.5f;
     surfaceData->getNumber("BumpHeight", bumpHeight);
 
@@ -203,18 +216,53 @@ static void FillinSurface(Hash* surfaceData,
 }
 
 
+// Set up the orbit barycenter for a body. By default, it is the parent of the
+// object
+static Selection GetOrbitBarycenter(const string& name,
+                                    PlanetarySystem* system)
+{
+    Selection orbitBarycenter;
+    Body* primary = system->getPrimaryBody();
+    if (primary != NULL)
+        orbitBarycenter = Selection(primary);
+    else
+        orbitBarycenter = Selection(system->getStar());
+
+    // The barycenter must be in the same star system as the object we're creating
+    if (orbitBarycenter.body())
+    {
+        if (system->getStar() != orbitBarycenter.body()->getSystem()->getStar())
+        {
+            cerr << "OrbitBarycenter" << _(" of ") << name << _(" must be in same star system\n");
+            return Selection();
+        }
+    }
+    else if (orbitBarycenter.star())
+    {
+        if (system->getStar() != orbitBarycenter.star())
+        {
+            cerr << "OrbitBarycenter" << _(" of ") << name << _(" must be in same star system\n");
+            return Selection();
+        }
+    }
+
+    return orbitBarycenter;
+}
+
+
 // Create a body (planet or moon) using the values from a hash
 // The usePlanetsUnits flags specifies whether period and semi-major axis
 // are in years and AU rather than days and kilometers
-static Body* CreatePlanet(PlanetarySystem* system,
+static Body* CreatePlanet(const string& name,
+                          PlanetarySystem* system,
+                          Universe& universe,
                           Body* existingBody,
                           Hash* planetData,
                           const string& path,
-                          Disposition disposition,
-                          bool usePlanetUnits = true)
+                          Disposition disposition)
 {
     Body* body = NULL;
-  
+
     if (disposition == ModifyObject)
     {
         body = existingBody;
@@ -225,8 +273,73 @@ static Body* CreatePlanet(PlanetarySystem* system,
         body = new Body(system);
     }
 
-    Orbit* orbit = CreateOrbit(system, planetData, path, usePlanetUnits);
+    Selection orbitBarycenter = GetOrbitBarycenter(name, system);
+    bool orbitsPlanet = false;
+    if (orbitBarycenter.body())
+    {
+        body->setOrbitBarycenter(orbitBarycenter.body());
+        orbitsPlanet = true;
+    }
+    else if (orbitBarycenter.star())
+    {
+        body->setOrbitBarycenter(NULL);
+    }
+    else
+    {
+        // Bad orbit barycenter specified
+        if (body != existingBody)
+            delete body;
+        return NULL;
+    }
 
+    // Set the reference frame of the orbit
+    Value* frameValue = planetData->getValue("OrbitFrame");
+    if (frameValue != NULL)
+    {
+        ReferenceFrame* frame = CreateReferenceFrame(universe, frameValue);
+        if (frame != NULL)
+        {
+            body->setOrbitFrame(frame);
+
+            // If the center of the is a star, orbital element units are
+            // in AU; otherwise, use kilometers.
+            if (frame->getCenter().star() != NULL)
+                orbitsPlanet = false;
+            else
+                orbitsPlanet = true;
+        }
+    }
+
+    Value* bodyFrameValue = planetData->getValue("BodyFrame");
+    if (bodyFrameValue != NULL)
+    {
+        ReferenceFrame* frame = CreateReferenceFrame(universe, bodyFrameValue);
+        if (frame != NULL)
+        {
+            body->setBodyFrame(frame);
+        }
+    }
+
+    if (body->getOrbitFrame() != NULL)
+    {
+        if (isFrameCircular(*body->getOrbitFrame(), ReferenceFrame::PositionFrame))
+        {
+            clog << "Orbit frame for " << body->getName() << " is nested too deep (probably circular)\n";
+            body->setOrbitFrame(NULL);
+        }
+    }
+
+    if (body->getBodyFrame() != NULL)
+    {
+        if (isFrameCircular(*body->getBodyFrame(), ReferenceFrame::OrientationFrame))
+        {
+            clog << "Body frame for " << body->getName() << " is nested too deep (probably circular)\n";
+            body->setBodyFrame(NULL);
+        }
+    }
+
+
+    Orbit* orbit = CreateOrbit(system, planetData, path, !orbitsPlanet);
     if (orbit != NULL)
     {
         body->setOrbit(orbit);
@@ -236,7 +349,8 @@ static Body* CreatePlanet(PlanetarySystem* system,
     {
         DPRINTF(0, "No valid orbit specified for object '%s'; skipping . . .\n",
                 body->getName().c_str());
-        delete body;
+        if (body != existingBody)
+            delete body;
         return NULL;
     }
 
@@ -264,7 +378,7 @@ static Body* CreatePlanet(PlanetarySystem* system,
 
     if (classification == Body::Unknown)
     {
-        //Try to guess the type
+        // Try to guess the type
         if (system->getPrimaryBody() != NULL)
         {
             if(radius > 0.1)
@@ -294,8 +408,20 @@ static Body* CreatePlanet(PlanetarySystem* system,
 
     string infoURL;
     if (planetData->getString("InfoURL", infoURL))
+    {
+        if (infoURL.find(':') == string::npos)
+        {
+            // Relative URL, the base directory is the current one,
+            // not the main installation directory
+            if (path[1] == ':')
+                // Absolute Windows path, file:/// is required
+                infoURL = "file:///" + path + "/" + infoURL;
+            else if (!path.empty())
+                infoURL = path + "/" + infoURL;
+        }
         body->setInfoURL(infoURL);
-    
+    }
+
     double albedo = 0.5;
     if (planetData->getNumber("Albedo", albedo))
         body->setAlbedo((float) albedo);
@@ -303,7 +429,7 @@ static Body* CreatePlanet(PlanetarySystem* system,
     double oblateness = 0.0;
     if (planetData->getNumber("Oblateness", oblateness))
         body->setOblateness((float) oblateness);
-    
+
     double mass = 0.0;
     if (planetData->getNumber("Mass", mass))
         body->setMass((float) mass);
@@ -312,11 +438,28 @@ static Body* CreatePlanet(PlanetarySystem* system,
     if (planetData->getRotation("Orientation", orientation))
         body->setOrientation(orientation);
 
-    RotationElements re = body->getRotationElements();
-    if (disposition != ModifyObject)
-        re.period = (float) body->getOrbit()->getPeriod();
-    FillinRotationElements(planetData, re);
-    body->setRotationElements(re);
+    // Get the rotation model for this body
+    double syncRotationPeriod = body->getOrbit()->getPeriod();
+    RotationModel* rm = CreateRotationModel(planetData, path,
+                                            syncRotationPeriod);
+    if (rm != NULL)
+    {
+        // TODO: Free old rotation model and replace it with the new one;
+        // should reference count rotation model objects.
+        body->setRotationModel(rm);
+    }
+    else
+    {
+        // If no rotation model is provided, use a default rotation model--
+        // a uniform rotation that's synchronous with the orbit (appropriate
+        // for nearly all natural satellites in the solar system.) If the
+        // disposition is modify, we do not want to replace the existing
+        // rotation model with a default one.
+        if (disposition != ModifyObject)
+        {
+            body->setRotationModel(CreateDefaultRotationModel(syncRotationPeriod));
+        }
+    }
 
     Surface surface;
     if (disposition == ModifyObject)
@@ -361,7 +504,7 @@ static Body* CreatePlanet(PlanetarySystem* system,
             {
                 Hash* atmosData = atmosDataValue->getHash();
                 assert(atmosData != NULL);
-                
+
                 Atmosphere* atmosphere = NULL;
                 if (disposition == ModifyObject)
                 {
@@ -382,6 +525,15 @@ static Body* CreatePlanet(PlanetarySystem* system,
                 atmosData->getColor("Upper", atmosphere->upperColor);
                 atmosData->getColor("Sky", atmosphere->skyColor);
                 atmosData->getColor("Sunset", atmosphere->sunsetColor);
+
+                atmosData->getNumber("Mie", atmosphere->mieCoeff);
+                atmosData->getNumber("MieScaleHeight", atmosphere->mieScaleHeight);
+                atmosData->getNumber("MieAsymmetry", atmosphere->miePhaseAsymmetry);
+                atmosData->getVector("Rayleigh", atmosphere->rayleighCoeff);
+                //atmosData->getNumber("RayleighScaleHeight", atmosphere->rayleighScaleHeight);
+                atmosData->getVector("Absorption", atmosphere->absorptionCoeff);
+
+                // Get the cloud map settings
                 atmosData->getNumber("CloudHeight", atmosphere->cloudHeight);
                 if (atmosData->getNumber("CloudSpeed", atmosphere->cloudSpeed))
                     atmosphere->cloudSpeed = degToRad(atmosphere->cloudSpeed);
@@ -394,12 +546,18 @@ static Body* CreatePlanet(PlanetarySystem* system,
                                                         TextureInfo::WrapTexture);
                 }
 
+                string cloudNormalMap;
+                if (atmosData->getString("CloudNormalMap", cloudNormalMap))
+                {
+                    atmosphere->cloudNormalMap.setTexture(cloudNormalMap,
+                                                           path,
+                                                           TextureInfo::WrapTexture);
+                }
+
                 body->setAtmosphere(*atmosphere);
                 if (disposition != ModifyObject)
                     delete atmosphere;
             }
-
-            delete atmosDataValue;
         }
     }
 
@@ -437,8 +595,6 @@ static Body* CreatePlanet(PlanetarySystem* system,
 
                 body->setRings(rings);
             }
-
-            delete ringsDataValue;
         }
     }
 
@@ -446,11 +602,102 @@ static Body* CreatePlanet(PlanetarySystem* system,
 }
 
 
+// Create a barycenter object using the values from a hash
+static Body* CreateReferencePoint(const string& name,
+                                  PlanetarySystem* system,
+                                  Universe& universe,
+                                  Body* existingBody,
+                                  Hash* refPointData,
+                                  const string& path,
+                                  Disposition disposition)
+{
+    Body* body = NULL;
+
+    if (disposition == ModifyObject)
+    {
+        body = existingBody;
+    }
+
+    if (body == NULL)
+    {
+        body = new Body(system);
+    }
+
+    body->setRadius(1.0f);
+    body->setClassification(Body::Invisible);
+
+    Selection orbitBarycenter = GetOrbitBarycenter(name, system);
+    bool orbitsPlanet = false;
+    if (orbitBarycenter.body())
+    {
+        body->setOrbitBarycenter(orbitBarycenter.body());
+        orbitsPlanet = true;
+    }
+    else if (orbitBarycenter.star())
+    {
+        body->setOrbitBarycenter(NULL);
+    }
+    else
+    {
+        // Bad orbit barycenter specified
+        if (body != existingBody)
+            delete body;
+        return NULL;
+    }
+
+   // Set the reference frame of the orbit
+    Value* frameValue = refPointData->getValue("OrbitFrame");
+    if (frameValue != NULL)
+    {
+        ReferenceFrame* frame = CreateReferenceFrame(universe, frameValue);
+        if (frame != NULL)
+        {
+            body->setOrbitFrame(frame);
+
+            // If the center of the is a star, orbital element units are
+            // in AU; otherwise, use kilometers.
+            if (frame->getCenter().star() != NULL)
+                orbitsPlanet = false;
+            else
+                orbitsPlanet = true;
+        }
+    }
+
+    if (body->getOrbitFrame() != NULL)
+    {
+        if (isFrameCircular(*body->getOrbitFrame(), ReferenceFrame::PositionFrame))
+        {
+            clog << "Orbit frame for " << body->getName() << " is nested too deep (probably circular)\n";
+            body->setOrbitFrame(NULL);
+        }
+    }
+
+    Orbit* orbit = CreateOrbit(system, refPointData, path, !orbitsPlanet);
+    if (orbit != NULL)
+    {
+        body->setOrbit(orbit);
+    }
+
+    if (body->getOrbit() == NULL)
+    {
+        DPRINTF(0, "No valid orbit specified for barycenter '%s'; skipping . . .\n",
+                name.c_str());
+        if (body != existingBody)
+            delete body;
+        return NULL;
+    }
+
+    body->setRotationModel(new ConstantOrientation(Quatd(1.0)));
+
+    return body;
+}
+
+
 bool LoadSolarSystemObjects(istream& in,
                             Universe& universe,
                             const std::string& directory)
 {
-    Tokenizer tokenizer(&in); 
+    Tokenizer tokenizer(&in);
     Parser parser(&tokenizer);
 
     while (tokenizer.nextToken() != Tokenizer::TokenEnd)
@@ -508,6 +755,7 @@ bool LoadSolarSystemObjects(istream& in,
         if (objectDataValue->getType() != Value::HashType)
         {
             sscError(tokenizer, "{ expected");
+            delete objectDataValue;
             return false;
         }
         Hash* objectData = objectDataValue->getHash();
@@ -515,9 +763,9 @@ bool LoadSolarSystemObjects(istream& in,
         Selection parent = universe.findPath(parentName, NULL, 0);
         PlanetarySystem* parentSystem = NULL;
 
-        if (itemType == "Body")
+        if (itemType == "Body" || itemType == "ReferencePoint")
         {
-            bool orbitsPlanet = false;
+            //bool orbitsPlanet = false;
             if (parent.star() != NULL)
             {
                 SolarSystem* solarSystem = universe.getSolarSystem(parent.star());
@@ -540,7 +788,7 @@ bool LoadSolarSystemObjects(istream& in,
                     parentSystem = new PlanetarySystem(parent.body());
                     parent.body()->setSatellites(parentSystem);
                 }
-                orbitsPlanet = true;
+                //orbitsPlanet = true;
             }
             else
             {
@@ -557,8 +805,13 @@ bool LoadSolarSystemObjects(istream& in,
                     cerr << _("warning duplicate definition of ") <<
                         parentName << " " <<  name << '\n';
                 }
-                
-                Body* body = CreatePlanet(parentSystem, existingBody, objectData, directory, disposition, !orbitsPlanet);
+
+                Body* body;
+                if (itemType == "ReferencePoint")
+                    body = CreateReferencePoint(name, parentSystem, universe, existingBody, objectData, directory, disposition);
+                else
+                    body = CreatePlanet(name, parentSystem, universe, existingBody, objectData, directory, disposition);
+
                 if (body != NULL)
                 {
                     body->setName(name);
@@ -566,7 +819,7 @@ bool LoadSolarSystemObjects(istream& in,
                     {
                         parentSystem->replaceBody(existingBody, body);
                         delete existingBody;
-                    } 
+                    }
                     else if (disposition == AddObject)
                     {
                         parentSystem->addBody(body);
@@ -606,6 +859,7 @@ bool LoadSolarSystemObjects(istream& in,
                 cerr << _("parent body '") << parentName << _("' of '") << name << _("' not found.\n");
             }
         }
+        delete objectDataValue;
     }
 
     // TODO: Return some notification if there's an error parsing the file
