@@ -22,20 +22,19 @@ class SolarSystem
  public:
     SolarSystem(Star*);
 
-    const Star* getStar() const;
+    Star* getStar() const;
     Point3f getCenter() const;
     PlanetarySystem* getPlanets() const;
     
  private:
     PlanetarySystem* planets;
-    const Star* star;
+    Star* star;
 };
 
 typedef std::map<uint32, SolarSystem*> SolarSystemCatalog;
 
 class Universe;
 
-bool ReadSolarSystems(std::istream&, const StarDatabase&, SolarSystemCatalog&);
 bool LoadSolarSystemObjects(std::istream& in,
                             Universe& universe,
                             const std::string& dir = "");
