@@ -9,12 +9,15 @@
 
 #include <celmath/mathlib.h>
 #include <cstring>
+#include <cassert>
 #include "celestia.h"
 #include "astro.h"
 #include "orbit.h"
 #include "star.h"
 #include "texmanager.h"
 
+using namespace std;
+
 
 // The value of the temperature of the sun is actually 5780, but the
 // stellar class tables list the temperature of a G2V star as 5860.  We
@@ -33,11 +36,6 @@ struct SpectralTypeInfo
     float rotationPeriod;
 };
 
-ResourceHandle StarDetails::starTexB = InvalidResource;
-ResourceHandle StarDetails::starTexA = InvalidResource;
-ResourceHandle StarDetails::starTexG = InvalidResource;
-ResourceHandle StarDetails::starTexM = InvalidResource;
-ResourceHandle StarDetails::starTexL = InvalidResource;
 
 static StarDetails** normalStarDetails = NULL;
 static StarDetails** whiteDwarfDetails = NULL;
@@ -45,6 +43,7 @@ static StarDetails*  neutronStarDetails = NULL;
 static StarDetails*  blackHoleDetails = NULL;
 static StarDetails*  barycenterDetails = NULL;
 
+StarDetails::StarTextureSet StarDetails::starTextures;
 
 // Star temperature data from Lang's _Astrophysical Data: Planets and Stars_
 // Temperatures from missing (and typically not used) types in those
@@ -121,13 +120,20 @@ static float tempT[10] =
     1425, 1350, 1275, 1200, 1140, 1080, 1020, 900, 800, 750
 };
 
+// White dwarf temperaturs
+static float tempWD[10] =
+{
+    100000.0f, 50400.0f, 25200.0f, 16800.0f, 12600.0f,
+    10080.0f, 8400.0f, 7200.0f, 6300.0f, 5600.0f,
+};
+
 
 // Tables with adjustments for estimating absolute bolometric magnitude from
 // visual magnitude, from Lang's "Astrophysical Data: Planets and Stars".
 // Gaps in the tables from unused spectral classes were filled in with linear
 // interpolation--not accurate, but these shouldn't appear in real catalog
 // data anyway.
-static float bmag_correctionO[3][10] = 
+static float bmag_correctionO[3][10] =
 {
     // Lum class V (main sequence)
     {
@@ -146,7 +152,7 @@ static float bmag_correctionO[3][10] =
     }
 };
 
-static float bmag_correctionB[3][10] = 
+static float bmag_correctionB[3][10] =
 {
     // Lum class V (main sequence)
     {
@@ -165,7 +171,7 @@ static float bmag_correctionB[3][10] =
     }
 };
 
-static float bmag_correctionA[3][10] = 
+static float bmag_correctionA[3][10] =
 {
     // Lum class V (main sequence)
     {
@@ -184,7 +190,7 @@ static float bmag_correctionA[3][10] =
     }
 };
 
-static float bmag_correctionF[3][10] = 
+static float bmag_correctionF[3][10] =
 {
     // Lum class V (main sequence)
     {
@@ -203,7 +209,7 @@ static float bmag_correctionF[3][10] =
     }
 };
 
-static float bmag_correctionG[3][10] = 
+static float bmag_correctionG[3][10] =
 {
     // Lum class V (main sequence)
     {
@@ -222,7 +228,7 @@ static float bmag_correctionG[3][10] =
     }
 };
 
-static float bmag_correctionK[3][10] = 
+static float bmag_correctionK[3][10] =
 {
     // Lum class V (main sequence)
     {
@@ -241,7 +247,7 @@ static float bmag_correctionK[3][10] =
     }
 };
 
-static float bmag_correctionM[3][10] = 
+static float bmag_correctionM[3][10] =
 {
     // Lum class V (main sequence)
     {
@@ -262,15 +268,24 @@ static float bmag_correctionM[3][10] =
 
 // Brown dwarf data from Grant Hutchison
 static float bmag_correctionL[10] =
-{ 
+{
     -4.6f, -4.9f, -5.0f, -5.2f, -5.4f, -5.9f, -6.1f, -6.7f, -7.4f, -8.2f,
 };
 
 static float bmag_correctionT[10] =
-{ 
+{
     -8.9f, -9.6f, -10.8f, -11.9f, -13.1f, -14.4f, -16.1f, -17.9f, -19.6f, -19.6f,
 };
 
+// White dwarf data from Grant Hutchison; value for hypothetical
+// 0 subclass is just duplicated from subclass 1.
+static float bmag_correctionWD[10] =
+{
+    -4.15f, -4.15f, -2.22f, -1.24f, -0.67f,
+    -0.32f, -0.13f, -0.04f, -0.03f, -0.09f,
+};
+
+
 // Stellar rotation by spectral and luminosity class.
 // Tables from Grant Hutchison:
 // "Most data are from Lang's _Astrophysical Data: Planets and Stars_ (I
@@ -339,20 +354,20 @@ static float rotperiod_M[3][10] =
 };
 
 
-char* LumClassNames[StellarClass::Lum_Count] = {
+const char* LumClassNames[StellarClass::Lum_Count] = {
     "I-a0", "I-a", "I-b", "II", "III", "IV", "V", "VI", ""
 };
 
-char* SubclassNames[11] = {
+const char* SubclassNames[11] = {
     "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", ""
 };
 
-char* SpectralClassNames[StellarClass::NormalClassCount] = {
+const char* SpectralClassNames[StellarClass::NormalClassCount] = {
     "O", "B", "A", "F", "G", "K", "M", "R",
     "S", "N", "WC", "WN", "?", "L", "T", "C",
 };
 
-char* WDSpectralClassNames[StellarClass::WDClassCount] = {
+const char* WDSpectralClassNames[StellarClass::WDClassCount] = {
     "DA", "DB", "DC", "DO", "DQ", "DZ", "D", "DX",
 };
 
@@ -366,7 +381,7 @@ StarDetails::GetStarDetails(const StellarClass& sc)
         return GetNormalStarDetails(sc.getSpectralClass(),
                                     sc.getSubclass(),
                                     sc.getLuminosityClass());
-                                    
+
     case StellarClass::WhiteDwarf:
         return GetWhiteDwarfDetails(sc.getSpectralClass(),
                                     sc.getSubclass());
@@ -384,16 +399,18 @@ StarDetails*
 StarDetails::CreateStandardStarType(const std::string& specTypeName,
                                     float _temperature,
                                     float _rotationPeriod)
-                                    
+
 {
     StarDetails* details = new StarDetails();
 
     details->setTemperature(_temperature);
     details->setSpectralType(specTypeName);
 
-    RotationElements re = details->getRotationElements();
-    re.period = _rotationPeriod;
-    details->setRotationElements(re);
+    details->setRotationModel(new UniformRotationModel(_rotationPeriod,
+                                                       0.0f,
+                                                       astro::J2000,
+                                                       0.0f,
+                                                       0.0f));
 
     return details;
 }
@@ -406,7 +423,7 @@ StarDetails::GetNormalStarDetails(StellarClass::SpectralClass specClass,
 {
     if (normalStarDetails == NULL)
     {
-        unsigned int nTypes = StellarClass::Spectral_Count * 11 * 
+        unsigned int nTypes = StellarClass::Spectral_Count * 11 *
             StellarClass::Lum_Count;
         normalStarDetails = new StarDetails*[nTypes];
         for (unsigned int i = 0; i < nTypes; i++)
@@ -463,6 +480,8 @@ StarDetails::GetNormalStarDetails(StellarClass::SpectralClass specClass,
         case StellarClass::Lum_Unknown:
             lumIndex = 0;
             break;
+
+        default: break;  // Do nothing, but prevent GCC4 warnings (Beware: potentially dangerous)
         }
 
         float temp = 0.0f;
@@ -514,6 +533,8 @@ StarDetails::GetNormalStarDetails(StellarClass::SpectralClass specClass,
         case StellarClass::Spectral_T:
             temp = tempT[subclass];
             break;
+
+        default: break;  // Do nothing, but prevent GCC4 warnings (Beware: potentially dangerous)
         }
 
         float bmagCorrection = 0.0f;
@@ -575,20 +596,17 @@ StarDetails::GetNormalStarDetails(StellarClass::SpectralClass specClass,
             period = 0.2f;
             bmagCorrection = bmag_correctionT[subclass];
             break;
+
+        default: break;  // Do nothing, but prevent GCC4 warnings (Beware: potentially dangerous)
         }
 
         normalStarDetails[index] = CreateStandardStarType(name, temp, period);
         normalStarDetails[index]->setBolometricCorrection(bmagCorrection);
-        
-        if (specClass == StellarClass::Spectral_L ||
-            specClass == StellarClass::Spectral_T)
-        {
-            normalStarDetails[index]->setTexture(MultiResTexture(starTexL, starTexL, starTexL));
-        }
-        else
-        {
-            normalStarDetails[index]->setTexture(MultiResTexture(starTexA, starTexA, starTexA));
-        }
+
+        MultiResTexture starTex = starTextures.starTex[specClass];
+        if (!starTex.isValid())
+            starTex = starTextures.defaultTex;
+        normalStarDetails[index]->setTexture(starTex);
     }
 
     return normalStarDetails[index];
@@ -605,7 +623,7 @@ StarDetails::GetWhiteDwarfDetails(StellarClass::SpectralClass specClass,
 
     if (whiteDwarfDetails == NULL)
     {
-        unsigned int nTypes = 
+        unsigned int nTypes =
             StellarClass::WDClassCount * StellarClass::SubclassCount;
         whiteDwarfDetails = new StarDetails*[nTypes];
         for (unsigned int i = 0; i < nTypes; i++)
@@ -624,19 +642,30 @@ StarDetails::GetWhiteDwarfDetails(StellarClass::SpectralClass specClass,
                 SubclassNames[subclass]);
 
         float temp;
-        if (subclass == 0)
-            temp = 100000.0f;
-        if (subclass == StellarClass::Subclass_Unknown)
-            temp = 10080.0f;  // Treat unknown as subclass 5
+        float bmagCorrection;
+        // subclass is always >= 0:
+        if (subclass <= 9)
+        {
+            temp = tempWD[subclass];
+            bmagCorrection = bmag_correctionWD[subclass];
+        }
         else
-            temp = 50400.0f / subclass;
+        {
+            // Treat unknown as subclass 5
+            temp = tempWD[5];
+            bmagCorrection = bmag_correctionWD[5];
+        }
 
         // Assign white dwarfs a rotation period of half an hour; very
         // rough, as white rotation rates vary a lot.
         float period = 1.0f / 48.0f;
-        
+
         whiteDwarfDetails[index] = CreateStandardStarType(name, temp, period);
-        whiteDwarfDetails[index]->setTexture(MultiResTexture(starTexA, starTexA, starTexA));
+        MultiResTexture starTex = starTextures.starTex[StellarClass::Spectral_D];
+        if (!starTex.isValid())
+            starTex = starTextures.defaultTex;
+        whiteDwarfDetails[index]->setTexture(starTex);
+        whiteDwarfDetails[index]->setBolometricCorrection(bmagCorrection);
     }
 
     return whiteDwarfDetails[index];
@@ -654,7 +683,10 @@ StarDetails::GetNeutronStarDetails()
                                                     1.0f / 86400.0f);
         neutronStarDetails->setRadius(10.0f);
         neutronStarDetails->addKnowledge(KnowRadius);
-        neutronStarDetails->setTexture(MultiResTexture(starTexA, starTexA, starTexA));
+        MultiResTexture starTex = starTextures.neutronStarTex;
+        if (!starTex.isValid())
+            starTex = starTextures.defaultTex;
+        neutronStarDetails->setTexture(starTex);
     }
 
     return neutronStarDetails;
@@ -697,13 +729,9 @@ StarDetails::GetBarycenterDetails()
 
 
 void
-StarDetails::InitializeStarTextures()
+StarDetails::SetStarTextures(const StarTextureSet& _starTextures)
 {
-    starTexB = GetTextureManager()->getHandle(TextureInfo("bstar.jpg", 0));
-    starTexA = GetTextureManager()->getHandle(TextureInfo("astar.jpg", 0));
-    starTexG = GetTextureManager()->getHandle(TextureInfo("gstar.jpg", 0));
-    starTexM = GetTextureManager()->getHandle(TextureInfo("mstar.jpg", 0));
-    starTexL = GetTextureManager()->getHandle(TextureInfo("browndwarf.jpg", 0));
+    starTextures = _starTextures;
 }
 
 
@@ -713,16 +741,47 @@ StarDetails::StarDetails() :
     bolometricCorrection(0.0f),
     knowledge(0u),
     visible(true),
+    texture(texture),
     model(InvalidResource),
     orbit(NULL),
     orbitalRadius(0.0f),
     barycenter(NULL),
-    semiAxes(1.0f, 1.0f, 1.0f)
+    rotationModel(NULL),
+    semiAxes(1.0f, 1.0f, 1.0f),
+    orbitingStars(NULL),
+    isShared(true)
 {
     spectralType[0] = '\0';
 }
 
 
+StarDetails::StarDetails(const StarDetails& sd) :
+    radius(sd.radius),
+    temperature(sd.temperature),
+    bolometricCorrection(sd.bolometricCorrection),
+    knowledge(sd.knowledge),
+    visible(sd.visible),
+    texture(texture),
+    model(sd.model),
+    orbit(sd.orbit),
+    orbitalRadius(sd.orbitalRadius),
+    barycenter(sd.barycenter),
+    rotationModel(sd.rotationModel),
+    semiAxes(sd.semiAxes),
+    orbitingStars(NULL),
+    isShared(false)
+{
+    assert(sd.isShared);
+    memcpy(spectralType, sd.spectralType, sizeof(spectralType));
+}
+
+
+StarDetails::~StarDetails()
+{
+    delete orbitingStars;
+}
+
+
 void
 StarDetails::setRadius(float _radius)
 {
@@ -828,9 +887,21 @@ StarDetails::setVisibility(bool b)
 
 
 void
-StarDetails::setRotationElements(const RotationElements& re)
+StarDetails::setRotationModel(const RotationModel* rm)
 {
-    rotationElements = re;
+    rotationModel = rm;
+}
+
+
+
+Star::~Star()
+{
+    // TODO: Implement reference counting for StarDetails objects so that
+    // we can enable this.
+#if 0
+    if (!details->shared())
+        delete details;
+#endif
 }
 
 
@@ -839,7 +910,7 @@ float Star::getRadius() const
 {
     if (details->getKnowledge(StarDetails::KnowRadius))
         return details->getRadius();
-        
+
 #ifdef NO_BOLOMETRIC_MAGNITUDE_CORRECTION
     // Use the Stefan-Boltzmann law to estimate the radius of a
     // star from surface temperature and luminosity
@@ -867,6 +938,23 @@ StarDetails::setEllipsoidSemiAxes(const Vec3f& v)
 }
 
 
+bool
+StarDetails::shared() const
+{
+    return isShared;
+}
+
+
+void
+StarDetails::addOrbitingStar(Star* star)
+{
+    assert(!shared());
+    if (orbitingStars == NULL)
+        orbitingStars = new vector<Star*>();
+    orbitingStars->push_back(star);
+}
+
+
 UniversalCoord
 Star::getPosition(double t) const
 {
@@ -901,6 +989,26 @@ Star::getPosition(double t) const
 }
 
 
+UniversalCoord
+Star::getOrbitBarycenterPosition(double t) const
+{
+    const Star* barycenter = getOrbitBarycenter();
+
+    if (barycenter == NULL)
+    {
+        Point3d barycenterPos(position.x * 1.0e6,
+                              position.y * 1.0e6,
+                              position.z * 1.0e6);
+
+        return UniversalCoord(barycenterPos);
+    }
+    else
+    {
+        return barycenter->getPosition(t);
+    }
+}
+
+
 MultiResTexture
 Star::getTexture() const
 {
@@ -954,11 +1062,14 @@ void Star::setLuminosity(float lum)
 
 void Star::setDetails(StarDetails* sd)
 {
+    // TODO: delete existing details if they aren't shared
     details = sd;
 }
 
 void Star::setOrbitBarycenter(Star* s)
 {
+    if (details->shared())
+        details = new StarDetails(*details);
     details->setOrbitBarycenter(s);
 }
 
@@ -968,7 +1079,15 @@ void Star::computeOrbitalRadius()
 }
 
 void
-Star::setRotationElements(const RotationElements& re)
+Star::setRotationModel(const RotationModel* rm)
 {
-    details->setRotationElements(re);
+    details->setRotationModel(rm);
+}
+
+void
+Star::addOrbitingStar(Star* star)
+{
+    if (details->shared())
+        details = new StarDetails(*details);
+    details->addOrbitingStar(star);
 }
