@@ -7,12 +7,12 @@
 // as published by the Free Software Foundation; either version 2
 // of the License, or (at your option) any later version.
 
-#ifndef MACOSX
+#ifndef TARGET_OS_MAC
 #define JPEG_SUPPORT
 #define PNG_SUPPORT
 #endif
 
-#ifdef MACOSX
+#ifdef TARGET_OS_MAC
 #include <unistd.h>
 #include "CGBuffer.h"
 #ifndef PNG_SUPPORT
@@ -30,14 +30,15 @@
 #include <cassert>
 
 #ifndef _WIN32
-#ifndef MACOSX
+#ifndef TARGET_OS_MAC
 #include <config.h>
-#endif /* ! MACOSX */
+#endif /* ! TARGET_OS_MAC */
 #endif /* ! _WIN32 */
 
 #include <celmath/vecmath.h>
 #include <celutil/filetype.h>
 #include <celutil/debug.h>
+#include <celutil/util.h>
 
 #include "gl.h"
 #include "glext.h"
@@ -66,11 +67,11 @@ extern "C" {
 #endif // JPEG_SUPPORT
 
 #ifdef PNG_SUPPORT // PNG_SUPPORT
-#ifdef MACOSX
+#ifdef TARGET_OS_MAC
 #include "../../macosx/png.h"
 #else
 #include "png.h"
-#endif // MACOSX
+#endif // TARGET_OS_MAC
 
 // Define png_jmpbuf() in case we are using a pre-1.0.6 version of libpng
 #ifndef png_jmpbuf
@@ -101,6 +102,7 @@ struct TextureCaps
     bool autoMipMapSupported;
     bool maxLevelSupported;
     GLint maxTextureSize;
+    bool nonPow2Supported;
 };
 
 static TextureCaps texCaps;
@@ -147,6 +149,7 @@ static const TextureCaps& GetTextureCaps()
         texCaps.autoMipMapSupported = ExtensionSupported("GL_SGIS_generate_mipmap");
         texCaps.maxLevelSupported = testMaxLevel();
         glGetIntegerv(GL_MAX_TEXTURE_SIZE, &texCaps.maxTextureSize);
+        texCaps.nonPow2Supported = ExtensionSupported("GL_ARB_texture_non_power_of_two");
     }
 
     return texCaps;
@@ -264,7 +267,7 @@ static void LoadMipmapSet(Image& img, GLenum target)
     {
         uint mipWidth  = max((uint) img.getWidth() >> mip, 1u);
         uint mipHeight = max((uint) img.getHeight() >> mip, 1u);
-            
+
         if (img.isCompressed())
         {
             glx::glCompressedTexImage2DARB(target,
@@ -340,7 +343,12 @@ static int CalcMipLevelCount(int w, int h)
 
 
 Texture::Texture(int w, int h, int d) :
-    width(w), height(h), depth(d), alpha(false)
+    alpha(false),
+    compressed(false),
+    width(w),
+    height(h),
+    depth(d),
+    formatOptions(0)
 {
 }
 
@@ -397,6 +405,17 @@ int Texture::getDepth() const
 }
 
 
+unsigned int Texture::getFormatOptions() const
+{
+    return formatOptions;
+}
+
+
+void Texture::setFormatOptions(unsigned int opts)
+{
+    formatOptions = opts;
+}
+
 
 ImageTexture::ImageTexture(Image& img,
                            AddressMode addressMode,
@@ -404,22 +423,26 @@ ImageTexture::ImageTexture(Image& img,
     Texture(img.getWidth(), img.getHeight()),
     glName(0)
 {
-    glGenTextures(1, (GLuint*)&glName);
+    glGenTextures(1, (GLuint*) &glName);
     glBindTexture(GL_TEXTURE_2D, glName);
 
-    bool mipmap = mipMapMode == DefaultMipMaps;
+
+    bool mipmap = mipMapMode != NoMipMaps;
     bool precomputedMipMaps = false;
 
-    // Require a complete set of mipmaps
+    // Use precomputed mipmaps only if a complete set is supplied
     int mipLevelCount = img.getMipLevelCount();
-    if (mipmap &&
-        mipLevelCount == CalcMipLevelCount(img.getWidth(), img.getHeight()))
+    if (mipmap && mipLevelCount == CalcMipLevelCount(img.getWidth(), img.getHeight()))
+    {
         precomputedMipMaps = true;
+    }
 
     // We can't automatically generate mipmaps for compressed textures.
-    // If a precomputed mipmap set isn't provided, turn of mipmapping entirely.
+    // If a precomputed mipmap set isn't provided, turn off mipmapping entirely.
     if (!precomputedMipMaps && img.isCompressed())
+    {
         mipmap = false;
+    }
 
     GLenum texAddress = GetGLTexAddressMode(addressMode);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, texAddress);
@@ -427,7 +450,7 @@ ImageTexture::ImageTexture(Image& img,
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     mipmap ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
-            
+
     if (mipMapMode == AutoMipMaps)
         glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);
 
@@ -435,15 +458,11 @@ ImageTexture::ImageTexture(Image& img,
 
     if (mipmap)
     {
-#if 0
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL,
-                        maxMipMapLevel);
-#endif
         if (precomputedMipMaps)
         {
             LoadMipmapSet(img, GL_TEXTURE_2D);
         }
-        else
+        else if (mipMapMode == DefaultMipMaps)
         {
             gluBuild2DMipmaps(GL_TEXTURE_2D,
                               internalFormat,
@@ -452,6 +471,11 @@ ImageTexture::ImageTexture(Image& img,
                               GL_UNSIGNED_BYTE,
                               img.getPixels());
         }
+        else
+        {
+            assert(mipMapMode == AutoMipMaps);
+            LoadMiplessTexture(img, GL_TEXTURE_2D);
+        }
     }
     else
     {
@@ -459,6 +483,7 @@ ImageTexture::ImageTexture(Image& img,
     }
 
     alpha = img.hasAlpha();
+    compressed = img.isCompressed();
 }
 
 
@@ -497,7 +522,7 @@ void ImageTexture::setBorderColor(Color borderColor)
 }
 
 
-TiledTexture::TiledTexture(Image& img, 
+TiledTexture::TiledTexture(Image& img,
                            int _uSplit, int _vSplit,
                            MipMapMode mipMapMode) :
     Texture(img.getWidth(), img.getHeight()),
@@ -512,8 +537,9 @@ TiledTexture::TiledTexture(Image& img,
     }
 
     alpha = img.hasAlpha();
+    compressed = img.isCompressed();
 
-    bool mipmap = mipMapMode == DefaultMipMaps;
+    bool mipmap = mipMapMode != NoMipMaps;
     bool precomputedMipMaps = false;
 
     // Require a complete set of mipmaps
@@ -647,7 +673,7 @@ TiledTexture::TiledTexture(Image& img,
             }
         }
     }
-            
+
     delete tile;
 }
 
@@ -699,7 +725,7 @@ int TiledTexture::getVTileCount(int) const
 const TextureTile TiledTexture::getTile(int lod, int u, int v)
 {
     if (lod != 0 || u >= uSplit || u < 0 || v >= vSplit || v < 0)
-        return TextureTile(0); 
+        return TextureTile(0);
     else
     {
         return TextureTile(glNames[v * uSplit + u]);
@@ -739,20 +765,20 @@ CubeMap::CubeMap(Image* faces[]) :
     if (!precomputedMipMaps && faces[0]->isCompressed())
         mipmap = false;
 
-    glGenTextures(1, (GLuint*)&glName);
-    glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, glName);
+    glGenTextures(1, (GLuint*) &glName);
+    glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, glName);
 
-    glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
+    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER,
                     mipmap ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
 
     int internalFormat = getInternalFormat(format);
 
     for (i = 0; i < 6; i++)
     {
-        GLenum targetFace = (GLenum) ((int) GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT + i);
+        GLenum targetFace = (GLenum) ((int) GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB + i);
         Image* face = faces[i];
 
         if (mipmap)
@@ -788,7 +814,7 @@ CubeMap::~CubeMap()
 
 void CubeMap::bind()
 {
-    glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, glName);
+    glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, glName);
 }
 
 
@@ -804,7 +830,7 @@ const TextureTile CubeMap::getTile(int lod, int u, int v)
 void CubeMap::setBorderColor(Color borderColor)
 {
     bind();
-    SetBorderColor(borderColor, GL_TEXTURE_CUBE_MAP_EXT);
+    SetBorderColor(borderColor, GL_TEXTURE_CUBE_MAP_ARB);
 }
 
 
@@ -863,7 +889,8 @@ Texture* CreateProceduralTexture(int width, int height,
 }
 
 
-// Helper function for CreateNormalizationCubeMap
+// Helper function for CreateProceduralCubeMap; return the normalized
+// vector pointing to (s, t) on the specified face.
 static Vec3f cubeVector(int face, float s, float t)
 {
     Vec3f v;
@@ -944,17 +971,18 @@ extern Texture* CreateProceduralCubeMap(int size, int format,
     return tex;
 }
 
-
+#if 0
 static bool isPow2(int x)
 {
     return ((x & (x - 1)) == 0);
 }
-
+#endif
 
 static Texture* CreateTextureFromImage(Image& img,
                                        Texture::AddressMode addressMode,
                                        Texture::MipMapMode mipMode)
 {
+#if 0
     // Require texture dimensions to be powers of two.  Even though the
     // OpenGL driver will automatically rescale textures with non-power of
     // two sizes, some quality loss may result.  The power of two requirement
@@ -966,23 +994,34 @@ static Texture* CreateTextureFromImage(Image& img,
         clog << "Texture has non-power of two dimensions.\n";
         return NULL;
     }
+#endif
+
+    // If non power of two textures are supported switch mipmap generation to
+    // automatic. gluBuildMipMaps may rescale the texture to a power of two
+    // on some drivers even when the hardware supports non power of two textures,
+    // whereas auto mipmap generation will properly deal with the dimensions.
+    if (GetTextureCaps().nonPow2Supported)
+    {
+        if (mipMode == Texture::DefaultMipMaps)
+            mipMode = Texture::AutoMipMaps;
+    }
 
     bool splittingAllowed = true;
     Texture* tex = NULL;
 
     int maxDim = GetTextureCaps().maxTextureSize;
-    //int maxDim = 256;
     if ((img.getWidth() > maxDim || img.getHeight() > maxDim) &&
         splittingAllowed)
     {
         // The texture is too large; we need to split it.
         int uSplit = max(1, img.getWidth() / maxDim);
         int vSplit = max(1, img.getHeight() / maxDim);
-        
+        clog << _("Creating tiled texture. Width=") << img.getWidth() << _(", max=") << maxDim << "\n";
         tex = new TiledTexture(img, uSplit, vSplit, mipMode);
     }
     else
     {
+        clog << _("Creating ordinary texture: ") << img.getWidth() << "x" << img.getHeight() << "\n";
         // The image is small enough to fit in a single texture; or, splitting
         // was disallowed so we'll scale the large image down to fit in
         // an ordinary texture.
@@ -998,7 +1037,9 @@ Texture* LoadTextureFromFile(const string& filename,
                              Texture::MipMapMode mipMode)
 {
     // Check for a Celestia texture--these need to be handled specially.
-    if (DetermineFileType(filename) == Content_CelestiaTexture)
+    ContentType contentType = DetermineFileType(filename);
+
+    if (contentType == Content_CelestiaTexture)
         return LoadVirtualTexture(filename);
 
     // All other texture types are handled by first loading an image, then
@@ -1008,6 +1049,19 @@ Texture* LoadTextureFromFile(const string& filename,
         return NULL;
 
     Texture* tex = CreateTextureFromImage(*img, addressMode, mipMode);
+
+    if (contentType == Content_DXT5NormalMap)
+    {
+        // If the texture came from a .dxt5nm file then mark it as a dxt5
+        // compressed normal map. There's no separate OpenGL format for dxt5
+        // normal maps, so the file extension is the only thing that
+        // distinguishes it from a plain old dxt5 texture.
+        if (img->getFormat() == GL_COMPRESSED_RGBA_S3TC_DXT5_EXT)
+        {
+            tex->setFormatOptions(Texture::DXT5NormalMap);
+        }
+    }
+
     delete img;
 
     return tex;
