@@ -15,6 +15,7 @@
 #include <cstdio>
 #include "celutil/debug.h"
 #include "celutil/directory.h"
+#include "celutil/filetype.h"
 #include "virtualtex.h"
 #include "gl.h"
 #include "parser.h"
@@ -58,8 +59,8 @@ VirtualTexture::VirtualTexture(const string& _tilePath,
                                const string& _tileType) :
     Texture(_tileSize << (_baseSplit + 1), _tileSize << _baseSplit),
     tilePath(_tilePath),
-    baseSplit(_baseSplit),
     tilePrefix(_tilePrefix),
+    baseSplit(_baseSplit),
     tileSize(_tileSize),
     ticks(0),
     nResolutionLevels(0)
@@ -69,6 +70,9 @@ VirtualTexture::VirtualTexture(const string& _tilePath,
     tileTree[1] = new TileQuadtreeNode();
     tileExt = string(".") + _tileType;
     populateTileTree();
+
+    if (DetermineFileType(tileExt) == Content_DXT5NormalMap)
+        setFormatOptions(Texture::DXT5NormalMap);
 }
 
 
@@ -85,8 +89,8 @@ const TextureTile VirtualTexture::getTile(int lod, int u, int v)
 #endif
 
     lod += baseSplit;
-    
-    if (lod < 0 || (uint) lod >= nResolutionLevels || 
+
+    if (lod < 0 || (uint) lod >= nResolutionLevels ||
         u < 0 || u >= (2 << lod) ||
         v < 0 || v >= (1 << lod))
     {
@@ -215,10 +219,10 @@ ImageTexture* VirtualTexture::loadTileTexture(uint lod, uint u, uint v)
     lod >>= baseSplit;
 
     assert(lod < (unsigned)MaxResolutionLevels);
-    
+
     char filename[64];
     sprintf(filename, "level%d/%s%d_%d", lod, tilePrefix.c_str(), u, v);
-    
+
     string pathname = tilePath + filename + tileExt;
     Image* img = LoadImageFromFile(pathname);
     if (img == NULL)
@@ -232,6 +236,12 @@ ImageTexture* VirtualTexture::loadTileTexture(uint lod, uint u, uint v)
 
     if (isPow2(img->getWidth()) && isPow2(img->getHeight()))
         tex = new ImageTexture(*img, EdgeClamp, mipMapMode);
+
+    // TODO: Virtual textures can have tiles in different formats, some
+    // compressed and some not. The compression flag doesn't make much
+    // sense for them.
+    compressed = img->isCompressed();
+
     delete img;
 
     return tex;
@@ -275,7 +285,7 @@ void VirtualTexture::populateTileTree()
                 maxLevel = i + baseSplit;
                 int uLimit = 2 << maxLevel;
                 int vLimit = 1 << maxLevel;
-                
+
                 string filename;
                 while (dir->nextFile(filename))
                 {
@@ -290,6 +300,7 @@ void VirtualTexture::populateTileTree()
                         }
                     }
                 }
+                delete dir;
             }
         }
     }
@@ -364,8 +375,6 @@ static VirtualTexture* CreateVirtualTexture(Hash* texParams,
                               (unsigned int) tileSize,
                               tilePrefix,
                               tileType);
-
-    return NULL;
 }
 
 
@@ -385,12 +394,16 @@ static VirtualTexture* LoadVirtualTexture(istream& in, const string& path)
     if (texParamsValue == NULL || texParamsValue->getType() != Value::HashType)
     {
         DPRINTF(0, "Error parsing virtual texture\n");
+        delete texParamsValue;
         return NULL;
     }
 
     Hash* texParams = texParamsValue->getHash();
-    
-    return CreateVirtualTexture(texParams, path);
+
+    VirtualTexture* virtualTex  = CreateVirtualTexture(texParams, path);
+    delete texParamsValue;
+
+    return virtualTex;
 }
 
 
