@@ -35,6 +35,8 @@ class Url;
 // class CelestiaWatcher;
 class CelestiaCore;
 
+class astro::Date;
+
 typedef Watcher<CelestiaCore> CelestiaWatcher;
 
 class ProgressNotifier
@@ -187,6 +189,15 @@ class CelestiaCore // : public Watchable<CelestiaCore>
         KbPassToScript   = 2,
     };        
 
+    enum
+    {
+        ShowNoElement   = 0x001,
+        ShowTime = 0x002,
+        ShowVelocity  = 0x004,
+        ShowSelection    = 0x008,
+        ShowFrame   = 0x010,
+    };
+
     typedef void (*ContextMenuFunc)(float, float, Selection);
 
  public:
@@ -243,6 +254,9 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     void addFavoriteFolder(std::string, FavoritesList::iterator* iter=NULL);
     FavoritesList* getFavorites();
 
+    bool viewUpdateRequired() const;
+    void setViewChanged();
+
     const DestinationList* getDestinations();
 
     int getTimeZoneBias() const;
@@ -266,6 +280,10 @@ class CelestiaCore // : public Watchable<CelestiaCore>
 
     int getHudDetail();
     void setHudDetail(int);
+    astro::Date::Format getDateFormat() const;
+    void setDateFormat(astro::Date::Format format);
+    int getOverlayElements() const;
+    void setOverlayElements(int);
 
     void setContextMenuCallback(ContextMenuFunc);
 
@@ -284,6 +302,8 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     void setActiveFrameVisible(bool);
     bool getLightDelayActive() const;
     void setLightDelayActive(bool);
+    bool getAltAzimuthMode() const;
+    void setAltAzimuthMode(bool);
     int getScreenDpi() const;
     void setScreenDpi(int);
     int getDistanceToScreen() const;
@@ -323,6 +343,9 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     bool readStars(const CelestiaConfig&, ProgressNotifier*);
     void renderOverlay();
     void fatalError(const std::string&);
+#ifdef CELX
+    bool initLuaHook(ProgressNotifier*);
+#endif // CELX
 
  private:
     CelestiaConfig* config;
@@ -353,6 +376,9 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     int typedTextCompletionIdx;
     int textEnterMode;
     int hudDetail;
+    astro::Date::Format dateFormat;
+    int dateStrWidth;
+    int overlayElements;
     bool wireframe;
     bool editMode;
     bool altAzimuthMode;
@@ -367,7 +393,17 @@ class CelestiaCore // : public Watchable<CelestiaCore>
 
 #ifdef CELX
     LuaState* celxScript;
-#endif // CELX    
+    LuaState* luaHook;     // Lua hook context
+    LuaState* luaSandbox;  // Safe Lua context for ssc scripts
+#endif // CELX
+
+    enum ScriptState
+    {
+        ScriptCompleted,
+        ScriptRunning,
+        ScriptPaused,
+    };
+    ScriptState scriptState;
 
     int timeZoneBias;              // Diff in secs between local time and GMT
     std:: string timeZoneName;	   // Name of the current time zone
@@ -387,9 +423,8 @@ class CelestiaCore // : public Watchable<CelestiaCore>
 
     double sysTime;
     double currentTime;
-    double timeScale;
-    bool paused;
-    bool scriptPaused;
+
+    bool viewChanged;
 
     Vec3f joystickRotation;
     bool joyButtonsPressed[JoyButtonCount];
@@ -428,6 +463,8 @@ class CelestiaCore // : public Watchable<CelestiaCore>
 #ifdef CELX
     friend View* getViewByObserver(CelestiaCore*, Observer*);
     friend void getObservers(CelestiaCore*, std::vector<Observer*>&);
+    friend TextureFont* getFont(CelestiaCore*);
+    friend TextureFont* getTitleFont(CelestiaCore*);
 #endif
 };
 
