@@ -7,7 +7,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  $Id: settings-gconf.cpp,v 1.1 2005/12/06 03:19:36 suwalski Exp $
+ *  $Id: settings-gconf.cpp,v 1.4 2008/01/18 04:36:11 suwalski Exp $
  */
 
 #include <gtk/gtk.h>
@@ -32,11 +32,14 @@ static void confWinY(GConfClient*, guint, GConfEntry* e, AppData* app);
 static void confAmbientLight(GConfClient*, guint, GConfEntry* e, AppData* app);
 static void confVisualMagnitude(GConfClient*, guint, GConfEntry* e, AppData* app);
 static void confGalaxyLightGain(GConfClient*, guint, GConfEntry* e, AppData*);
+static void confDistanceLimit(GConfClient*, guint, GConfEntry* e, AppData*);
 static void confShowLocalTime(GConfClient*, guint, GConfEntry* e, AppData* app);
 static void confVerbosity(GConfClient*, guint, GConfEntry* e, AppData* app);
 static void confFullScreen(GConfClient*, guint, GConfEntry* e, AppData* app);
 static void confStarStyle(GConfClient*, guint, GConfEntry* e, AppData* app);
+static void confTextureResolution(GConfClient*, guint, GConfEntry* e, AppData* app);
 static void confAltSurfaceName(GConfClient*, guint, GConfEntry* e, AppData* app);
+static void confVideoSync(GConfClient*, guint, GConfEntry* e, AppData* app);
 
 /* Definitions: Helpers */
 static int readGConfRender(GConfClient* client);
@@ -67,11 +70,14 @@ void initSettingsGConfNotifiers(AppData* app)
 	gconf_client_notify_add (app->client, "/apps/celestia/ambientLight", (GConfClientNotifyFunc)confAmbientLight, app, NULL, NULL);
 	gconf_client_notify_add (app->client, "/apps/celestia/visualMagnitude", (GConfClientNotifyFunc)confVisualMagnitude, app, NULL, NULL);
 	gconf_client_notify_add (app->client, "/apps/celestia/galaxyLightGain", (GConfClientNotifyFunc)confGalaxyLightGain, app, NULL, NULL);
+	gconf_client_notify_add (app->client, "/apps/celestia/distanceLimit", (GConfClientNotifyFunc)confDistanceLimit, app, NULL, NULL);
 	gconf_client_notify_add (app->client, "/apps/celestia/showLocalTime", (GConfClientNotifyFunc)confShowLocalTime, app, NULL, NULL);
 	gconf_client_notify_add (app->client, "/apps/celestia/verbosity", (GConfClientNotifyFunc)confVerbosity, app, NULL, NULL);
 	gconf_client_notify_add (app->client, "/apps/celestia/fullScreen", (GConfClientNotifyFunc)confFullScreen, app, NULL, NULL);
 	gconf_client_notify_add (app->client, "/apps/celestia/starStyle", (GConfClientNotifyFunc)confStarStyle, app, NULL, NULL);
+	gconf_client_notify_add (app->client, "/apps/celestia/textureResolution", (GConfClientNotifyFunc)confTextureResolution, app, NULL, NULL);
 	gconf_client_notify_add (app->client, "/apps/celestia/altSurfaceName", (GConfClientNotifyFunc)confAltSurfaceName, app, NULL, NULL);
+	gconf_client_notify_add (app->client, "/apps/celestia/videoSync", (GConfClientNotifyFunc)confVideoSync, app, NULL, NULL);
 }
 
 
@@ -101,11 +107,15 @@ void applySettingsGConfMain(AppData* app, GConfClient* client)
 	setSaneAmbientLight(app, gconf_client_get_float(client, "/apps/celestia/ambientLight", NULL));
 	setSaneVisualMagnitude(app, gconf_client_get_float(client, "/apps/celestia/visualMagnitude", NULL));
 	setSaneGalaxyLightGain(gconf_client_get_float(client, "/apps/celestia/galaxyLightGain", NULL));
+	setSaneDistanceLimit(app, gconf_client_get_int(client, "/apps/celestia/distanceLimit", NULL));
 	setSaneVerbosity(app, gconf_client_get_int(client, "/apps/celestia/verbosity", NULL));
 	setSaneStarStyle(app, (Renderer::StarStyle)gconf_client_get_int(client, "/apps/celestia/starStyle", NULL));
+	setSaneTextureResolution(app, gconf_client_get_int(client, "/apps/celestia/textureResolution", NULL));
 	setSaneAltSurface(app, gconf_client_get_string(client, "/apps/celestia/altSurfaceName", NULL));
 	
 	app->showLocalTime = gconf_client_get_bool(client, "/apps/celestia/showLocalTime", NULL);
+
+	app->renderer->setVideoSync(gconf_client_get_bool(client, "/apps/celestia/videoSync", NULL));
 	
 	/* Render Flags */
 	rf = readGConfRender(app->client);
@@ -128,9 +138,13 @@ void saveSettingsGConf(AppData* app)
 	gconf_client_set_int(app->client, "/apps/celestia/winX", getWinX(app), NULL);
 	gconf_client_set_int(app->client, "/apps/celestia/winY", getWinY(app), NULL);
 	
-	/* Save window  size */
+	/* Save window size */
 	gconf_client_set_int(app->client, "/apps/celestia/winWidth", getWinWidth(app), NULL);
 	gconf_client_set_int(app->client, "/apps/celestia/winHeight", getWinHeight(app), NULL);
+
+	/* These do not produce notification when changed */
+	gconf_client_set_int(app->client, "/apps/celestia/textureResolution", app->renderer->getResolution(), NULL);
+	gconf_client_set_int(app->client, "/apps/celestia/distanceLimit", (int)app->renderer->getDistanceLimit(), NULL);
 	
 	g_object_unref (G_OBJECT (app->client));
 }
@@ -320,6 +334,18 @@ static void confGalaxyLightGain(GConfClient*, guint, GConfEntry* e, AppData*)
 }
 
 
+/* GCONF CALLBACK: Sets texture resolution when changed */
+static void confDistanceLimit(GConfClient*, guint, GConfEntry* e, AppData* app)
+{
+	int value = gconf_value_get_int(e->value);
+	
+	if (value == app->renderer->getDistanceLimit())
+		return;
+	
+	setSaneDistanceLimit(app, value);
+}
+
+
 /* GCONF CALLBACK: Sets "show local time" setting upon change */
 static void confShowLocalTime(GConfClient*, guint, GConfEntry* e, AppData* app)
 {
@@ -375,6 +401,18 @@ static void confStarStyle(GConfClient*, guint, GConfEntry* e, AppData* app)
 }
 
 
+/* GCONF CALLBACK: Sets texture resolution when changed */
+static void confTextureResolution(GConfClient*, guint, GConfEntry* e, AppData* app)
+{
+	int value = gconf_value_get_int(e->value);
+	
+	if (value == (int)app->renderer->getResolution())
+		return;
+	
+	setSaneTextureResolution(app, value);
+}
+
+
 /* GCONF CALLBACK: Sets alternate surface texture when changed */
 static void confAltSurfaceName(GConfClient*, guint, GConfEntry* e, AppData* app)
 {
@@ -387,6 +425,18 @@ static void confAltSurfaceName(GConfClient*, guint, GConfEntry* e, AppData* app)
 }
 
 
+/* GCONF CALLBACK: Sets video framerate sync when changed */
+static void confVideoSync(GConfClient*, guint, GConfEntry* e, AppData* app)
+{
+	gboolean value = gconf_value_get_bool(e->value);
+	
+	if (value == app->renderer->getVideoSync())
+		return;
+	
+	app->renderer->setVideoSync(value);
+}
+
+
 /* HELPER: Reads in GConf->Render preferences and returns as int mask */
 static int readGConfRender(GConfClient* client)
 {
