@@ -15,6 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
+
 #include <fstream>
 #include <sstream>
 
@@ -33,6 +34,8 @@
 #include <qfont.h>
 #include <qlineedit.h>
 #include <qvalidator.h>
+#include <qurl.h>
+#include <qevent.h>
 
 #include <qmenubar.h>
 #include <qpopupmenu.h>
@@ -48,7 +51,7 @@
 #include <klocale.h>
 #include <kconfig.h>
 #include <qtextedit.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 
 #include <qdatetime.h>
 #include <kshortcut.h>
@@ -65,7 +68,7 @@
 #include <kcelbookmarkmanager.h>
 #include <kdeversion.h>
 
-#if KDE_VERSION >= 0x030200 
+#if KDE_VERSION >= 0x030200
 #include <ksplashscreen.h>
 #endif
 
@@ -79,6 +82,10 @@
 #include "selectionpopup.h"
 #include "celsplashscreen.h"
 
+#ifdef THEORA
+#include "videocapturedlg.h"
+#endif
+
 #include "celengine/glext.h"
 
 #define MENUMAXSIZE 100
@@ -113,7 +120,7 @@ KdeApp::KdeApp(std::string config, std::string dir, std::vector<std::string> ext
                 images.append(*i + *j);
             }
         }
-        
+
         if (images.size() > 0) {
             int index = (int)(random()*1./RAND_MAX*images.size());
             splash = new CelSplashScreen(images[index], this);
@@ -122,7 +129,7 @@ KdeApp::KdeApp(std::string config, std::string dir, std::vector<std::string> ext
         }
     }
 #endif
-    
+
     appCore=new CelestiaCore();
     if (appCore == NULL)
     {
@@ -142,6 +149,7 @@ KdeApp::KdeApp(std::string config, std::string dir, std::vector<std::string> ext
     setAcceptDrops(true);
 
     // Create our OpenGL widget
+    startDir = QDir::current().path();
     if ( (dir.length() > 1 ? chdir(dir.c_str()):chdir(CONFIG_DATA_DIR)) == -1)
     {
         ::std::cout << "Cannot chdir to '" << CONFIG_DATA_DIR << "', probably due to improper installation" << ::std::endl;
@@ -191,7 +199,7 @@ KdeApp::KdeApp(std::string config, std::string dir, std::vector<std::string> ext
         delete splash;
     }
 #endif
-    
+
     // We use a timer with a null timeout value
     // to add appCore->tick to Qt's event loop
     QTimer *t = new QTimer( dynamic_cast<QObject *>(this) );
@@ -204,7 +212,7 @@ void KdeApp::setStartURL(KURL url) {
     if (url.protocol() == "cel") {
         appCore->setStartURL(url.url().latin1());
     }
-    if (url.protocol() == "file") { 
+    if (url.protocol() == "file") {
         appCore->setStartURL(url.path().latin1());
     }
 }
@@ -214,7 +222,7 @@ void KdeApp::goToURL(const KURL& url) {
         appCore->addToHistory();
         appCore->goToUrl(url.prettyURL().latin1());
     }
-    if (url.protocol() == "file") { 
+    if (url.protocol() == "file") {
         appCore->addToHistory();
         slotOpenFileURL(url);
     }
@@ -246,7 +254,7 @@ QString KdeApp::currentIcon() const {
     return iconName;
 }
 
-void KdeWatcher::notifyChange(CelestiaCore * core, int property)
+void KdeWatcher::notifyChange(CelestiaCore*, int property)
 {
     if ((property & (CelestiaCore::RenderFlagsChanged|
                      CelestiaCore::LabelFlagsChanged|
@@ -265,16 +273,18 @@ void KdeWatcher::notifyChange(CelestiaCore * core, int property)
         static std::vector<KAction*> actions;
         if (kdeapp->appCore->getTextEnterMode() != CelestiaCore::KbNormal) {
             for (unsigned int n=0; n < kdeapp->getActionCollection()->count(); n++) {
-                if (kdeapp->getActionCollection()->action(n)->shortcut().count() > 0
-                    && (kdeapp->getActionCollection()->action(n)->shortcut().seq(0).key(0).modFlags()
-                    & (KKey::CTRL | KKey::ALT | KKey::WIN )) == 0 ) {
+                KAction* action = kdeapp->getActionCollection()->action(n);
+                if (action->shortcut().count() > 0
+                    && (action->shortcut().seq(0).key(0).modFlags()
+                        & ( KKey::ALT | KKey::WIN )) == 0
+                    && action->isEnabled()) {
                     actions.push_back(kdeapp->getActionCollection()->action(n));
                     kdeapp->getActionCollection()->action(n)->setEnabled(false);
                 }
             }
         } else {
             for (std::vector<KAction*>::iterator n=actions.begin(); n<actions.end(); n++) {
-                 (*n)->setEnabled(true);
+                (*n)->setEnabled(true);
             }
             actions.clear();
         }
@@ -294,7 +304,7 @@ void KdeApp::resyncHistory() {
         action("go_back")->setEnabled(false);
     } else {
         action("go_back")->setEnabled(true);
-    } 
+    }
 }
 
 void KdeApp::resyncMenus() {
@@ -308,10 +318,12 @@ void KdeApp::resyncMenus() {
     ((KToggleAction*)action("showOpenClusters"))->setChecked(rFlags & Renderer::ShowOpenClusters);
     ((KToggleAction*)action("showDiagrams"))->setChecked(rFlags & Renderer::ShowDiagrams);
     ((KToggleAction*)action("showCloudMaps"))->setChecked(rFlags & Renderer::ShowCloudMaps);
+    ((KToggleAction*)action("showCloudShadows"))->setChecked(rFlags & Renderer::ShowCloudShadows);
     ((KToggleAction*)action("showOrbits"))->setChecked(rFlags & Renderer::ShowOrbits);
     ((KToggleAction*)action("showAsteroidOrbits"))->setChecked(orbitMask & Body::Asteroid);
     ((KToggleAction*)action("showCometOrbits"))->setChecked(orbitMask & Body::Comet);
     ((KToggleAction*)action("showMoonOrbits"))->setChecked(orbitMask & Body::Moon);
+    ((KToggleAction*)action("showStarOrbits"))->setChecked(orbitMask & Body::Stellar);
     ((KToggleAction*)action("showPlanetOrbits"))->setChecked(orbitMask & Body::Planet);
     ((KToggleAction*)action("showSpacecraftOrbits"))->setChecked(orbitMask & Body::Spacecraft);
     ((KToggleAction*)action("showCelestialSphere"))->setChecked(rFlags & Renderer::ShowCelestialSphere);
@@ -339,6 +351,8 @@ void KdeApp::resyncMenus() {
     ((KToggleAction*)action("showSpacecraftLabels"))->setChecked(lMode & Renderer::SpacecraftLabels);
     ((KToggleAction*)action("showLocationLabels"))->setChecked(lMode & Renderer::LocationLabels);
 
+    ((KToggleAction*)action("toggleVideoSync"))->setChecked(renderer->getVideoSync());
+
     switch (renderer->getGLContext()->getRenderPath()) {
     case GLContext::GLPath_Basic:
         ((KToggleAction*)action("renderPathBasic"))->setChecked(true);
@@ -391,10 +405,10 @@ void KdeApp::initActions()
     new KAction(i18n("Go to &Long/Lat..."), 0, ALT + Key_L, this, SLOT(slotGoToLongLat()), actionCollection(), "go_to_long_lat");
 
     backAction = new KToolBarPopupAction( i18n("&Back"), "back",
-                                           KStdAccel::key(KStdAccel::Back), this, SLOT( slotBack() ),
+                                           KStdAccel::shortcut(KStdAccel::Back), this, SLOT( slotBack() ),
                                            actionCollection(), KStdAction::stdName( KStdAction::Back ) );
     forwardAction = new KToolBarPopupAction( i18n("&Forward"), "forward",
-                                           KStdAccel::key(KStdAccel::Forward), this, SLOT( slotForward() ),
+                                           KStdAccel::shortcut(KStdAccel::Forward), this, SLOT( slotForward() ),
                                            actionCollection(), KStdAction::stdName( KStdAction::Forward ) );
     connect( backAction->popupMenu(), SIGNAL( aboutToShow() ), SLOT( slotBackAboutToShow() ) );
     connect( backAction->popupMenu(), SIGNAL( activated( int ) ), SLOT( slotBackActivated( int ) ) );
@@ -430,6 +444,8 @@ void KdeApp::initActions()
     new KAction(i18n("Set Time to Now"), "player_eject", Key_Exclam, this, SLOT(slotSetTimeNow()), actionCollection(), "setTimeNow");
     new KAction(i18n("Accelerate Time"), "1uparrow", Key_L, this, SLOT(slotAccelerateTime()), actionCollection(), "accelerateTime");
     new KAction(i18n("Decelerate Time"), "1downarrow", Key_K, this, SLOT(slotSlowDownTime()), actionCollection(), "slowDownTime");
+    new KAction(i18n("Accelerate Time (x2)"), "1uparrow", SHIFT + Key_L, this, SLOT(slotAccelerateTimeFine()), actionCollection(), "accelerateTimeFine");
+    new KAction(i18n("Decelerate Time (/2)"), "1downarrow", SHIFT + Key_K, this, SLOT(slotSlowDownTimeFine()), actionCollection(), "slowDownTimeFine");
     new KAction(i18n("Pause Time"), "player_pause", Key_Space, this, SLOT(slotPauseTime()), actionCollection(), "pauseTime");
     new KAction(i18n("Reverse Time"), "reload", Key_J, this, SLOT(slotReverseTime()), actionCollection(), "reverseTime");
 
@@ -501,7 +517,26 @@ void KdeApp::initActions()
 
     if (KGlobal::config()->hasKey("TimeZoneBias"))
         isLocal = (KGlobal::config()->readNumEntry("TimeZoneBias") != 0);
-    
+
+    if (KGlobal::config()->hasKey("StarStyle")) 
+    {
+        int starStyle = KGlobal::config()->readNumEntry("StarStyle");
+        if (starStyle >= 0 && starStyle < Renderer::StarStyleCount)
+            appCore->getRenderer()->setStarStyle((Renderer::StarStyle)starStyle);
+    }
+
+    if (KGlobal::config()->hasKey("TextureResolution")) 
+    {
+        int textureResolution = KGlobal::config()->readNumEntry("TextureResolution");
+        appCore->getRenderer()->setResolution(textureResolution);
+    }
+
+    if (KGlobal::config()->hasKey("DateFormat")) 
+    {
+        astro::Date::Format dateFormat = (astro::Date::Format) KGlobal::config()->readNumEntry("DateFormat");
+        appCore->setDateFormat(dateFormat);
+    }
+
     /////////////////////////////////////////
     // Render Flags
     KToggleAction* showStars = new KToggleAction(i18n("Show Stars"), 0, this, SLOT(slotShowStars()), actionCollection(), "showStars");
@@ -528,6 +563,9 @@ void KdeApp::initActions()
     KToggleAction* showCloudMaps = new KToggleAction(i18n("Show CloudMaps"), Key_I, this, SLOT(slotShowCloudMaps()), actionCollection(), "showCloudMaps");
     showCloudMaps->setChecked(rFlags & Renderer::ShowCloudMaps);
 
+    KToggleAction* showCloudShadows = new KToggleAction(i18n("Show Cloud Shadows"), 0, this, SLOT(slotShowCloudShadows()), actionCollection(), "showCloudShadows");
+    showCloudShadows->setChecked(rFlags & Renderer::ShowCloudShadows);
+
     KToggleAction* showOrbits = new KToggleAction(i18n("Show Orbits"), Key_O, this, SLOT(slotShowOrbits()), actionCollection(), "showOrbits");
     showOrbits->setChecked(rFlags & Renderer::ShowOrbits);
 
@@ -540,6 +578,9 @@ void KdeApp::initActions()
     KToggleAction* showMoonOrbits = new KToggleAction(i18n("Show Moon Orbits"), 0, this, SLOT(slotShowMoonOrbits()), actionCollection(), "showMoonOrbits");
     showMoonOrbits->setChecked(oMask & Body::Moon);
 
+    KToggleAction* showStarOrbits = new KToggleAction(i18n("Show Star Orbits"), 0, this, SLOT(slotShowStarOrbits()), actionCollection(), "showStarOrbits");
+    showStarOrbits->setChecked(oMask & Body::Stellar);
+
     KToggleAction* showPlanetOrbits = new KToggleAction(i18n("Show Planet Orbits"), 0, this, SLOT(slotShowPlanetOrbits()), actionCollection(), "showPlanetOrbits");
     showPlanetOrbits->setChecked(oMask & Body::Planet);
 
@@ -572,8 +613,8 @@ void KdeApp::initActions()
     KToggleAction* showBoundaries = new KToggleAction(i18n("Show Boundaries"), CTRL + Key_B, this, SLOT(slotShowBoundaries()), actionCollection(), "showBoundaries");
     showBoundaries->setChecked(rFlags & Renderer::ShowBoundaries);
 
-    new KToggleAction(i18n("Auto Magnitudes"), CTRL + Key_Y, this, SLOT(slotShowAutoMag()), actionCollection(), "showAutoMag");
-    showBoundaries->setChecked(rFlags & Renderer::ShowAutoMag);
+    KToggleAction* showAutoMag = new KToggleAction(i18n("Auto Magnitudes"), CTRL + Key_Y, this, SLOT(slotShowAutoMag()), actionCollection(), "showAutoMag");
+    showAutoMag->setChecked(rFlags & Renderer::ShowAutoMag);
 
     KToggleAction* showCometTails = new KToggleAction(i18n("Show Comet Tails"), CTRL + Key_T, this, SLOT(slotShowCometTails()), actionCollection(), "showCometTails");
     showCometTails->setChecked(rFlags & Renderer::ShowCometTails);
@@ -590,7 +631,7 @@ void KdeApp::initActions()
     showMoonLabels->setChecked(lMode & Renderer::MoonLabels);
 
     KToggleAction* showCometLabels = new KToggleAction(i18n("Show Comet Labels"), SHIFT + Key_W, this, SLOT(slotShowCometLabels()), actionCollection(), "showCometLabels");
-    showMoonLabels->setChecked(lMode & Renderer::CometLabels);
+    showCometLabels->setChecked(lMode & Renderer::CometLabels);
 
     KToggleAction* showConstellationLabels = new KToggleAction(i18n("Show Constellation Labels"), Key_Equal, this, SLOT(slotShowConstellationLabels()), actionCollection(), "showConstellationLabels");
     showConstellationLabels->setChecked(lMode & Renderer::ConstellationLabels);
@@ -675,29 +716,37 @@ void KdeApp::initActions()
     renderPath->setExclusiveGroup("renderPath");
     new KAction(i18n("Cycle OpenGL Render Path"), "reload", CTRL + Key_V, this, SLOT(slotCycleRenderPath()), actionCollection(), "cycleRenderPath");
 
+    KToggleAction *videoSync = new KToggleAction(i18n("Sync framerate to video refresh rate"), 0, this, SLOT(slotToggleVideoSync()), actionCollection(), "toggleVideoSync");
+    if (KGlobal::config()->hasKey("VideoSync"))
+        appCore->getRenderer()->setVideoSync(KGlobal::config()->readBoolEntry("VideoSync"));
+    videoSync->setChecked(appCore->getRenderer()->getVideoSync());
+
     new KAction(i18n("Grab Image"), "filesave", Key_F10, this, SLOT(slotGrabImage()), actionCollection(), "grabImage");
+    new KAction(i18n("Capture Video"), "filesave", Key_F11, this, SLOT(slotCaptureVideo()), actionCollection(), "captureVideo");
 
     new KAction(i18n("OpenGL info"), 0, this, SLOT(slotOpenGLInfo()),
                       actionCollection(), "opengl_info");
 
     toggleMenubar=KStdAction::showMenubar(this, SLOT(slotToggleMenubar()), actionCollection());
+    //NOTE: this one is deprecated, but it looks nicer than the alternative:
     toggleToolbar=KStdAction::showToolbar(this, SLOT(slotToggleToolbar()), actionCollection());
+    //setStandardToolBarMenuEnabled(true);
 
     new KToggleAction(i18n("Show Bookmark Toolbar"), 0, this,
         SLOT(slotShowBookmarkBar()), actionCollection(), "showBookmarkBar");
 
     createGUI();
-    
+
     bookmarkBarActionCollection = new KActionCollection( this );
     bookmarkBarActionCollection->setHighlightingEnabled( true );
 
     bookmarkBar = 0;
-    initBookmarkBar();    
+    initBookmarkBar();
 }
 
 void KdeApp::initBookmarkBar() {
     KToolBar *bar = new KToolBar(this, QMainWindow::Top, true, "bookmarkBar");
-    
+
     if (bookmarkBar) delete bookmarkBar;
     bookmarkBar = new KBookmarkBar( KCelBookmarkManager::self(), this, bar, bookmarkBarActionCollection, 0, "bookmarkBar");
     if (bar->count() == 0) bar->hide();
@@ -709,7 +758,7 @@ void KdeApp::initBookmarkBar() {
 }
 
 
-bool KdeApp::queryExit() { 
+bool KdeApp::queryExit() {
     KConfig* conf = kapp->config();
     saveMainWindowSettings(conf, "MainWindow");
     conf->setGroup("MainWindow");
@@ -729,6 +778,10 @@ bool KdeApp::queryExit() {
     conf->writeEntry("DistanceToScreen", appCore->getDistanceToScreen());
     conf->writeEntry("LocationFilter", appCore->getSimulation()->getActiveObserver()->getLocationFilter());
     conf->writeEntry("MinFeatureSize", appCore->getRenderer()->getMinimumFeatureSize());
+    conf->writeEntry("VideoSync", appCore->getRenderer()->getVideoSync());
+    conf->writeEntry("StarStyle", appCore->getRenderer()->getStarStyle());
+    conf->writeEntry("TextureResolution", appCore->getRenderer()->getResolution());
+    conf->writeEntry("DateFormat", appCore->getDateFormat());
     conf->setGroup(0);
     actionCollection()->writeShortcutSettings("Shortcuts", conf);
     openRecent->saveEntries(KGlobal::config());
@@ -760,7 +813,7 @@ void KdeApp::slotFullScreen() {
     } else {
         showFullScreen();
         action("fullScreen")->setIcon("window_nofullscreen");
-    }             
+    }
     isFullScreen = !isFullScreen;
 }
 
@@ -884,17 +937,17 @@ QString KdeApp::getOpenGLInfo() {
     char* version = (char*) glGetString(GL_VERSION);
     char* ext = (char*) glGetString(GL_EXTENSIONS);
     QString s;
-    s = "Vendor : ";
+    s = i18n("Vendor: ");
     if (vendor != NULL)
         s += vendor;
     s += "\n";
 
-    s += "Renderer : ";
+    s += i18n("Renderer: ");
     if (render != NULL)
         s += render;
     s += "\n";
 
-    s += "Version : ";
+    s += i18n("Version: ");
     if (version != NULL)
         s += version;
     s += "\n";
@@ -903,15 +956,22 @@ QString KdeApp::getOpenGLInfo() {
     GLint simTextures = 1;
     if (ExtensionSupported("GL_ARB_multitexture"))
         glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &simTextures);
-    sprintf(buf, "Max simultaneous textures: %d\n", simTextures);
+    sprintf(buf, "%s%d\n", _("Max simultaneous textures: "), simTextures);
     s += buf;
 
     GLint maxTextureSize = 0;
     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxTextureSize);
-    sprintf(buf, "Max texture size: %d\n\n", maxTextureSize);
+    sprintf(buf, "%s%d\n\n", _("Max texture size: "), maxTextureSize);
     s += buf;
 
-    s += "Supported Extensions:\n    ";
+    GLfloat pointSizeRange[2];
+    glGetFloatv(GL_POINT_SIZE_RANGE, pointSizeRange);
+    sprintf(buf, "%s%f - %f\n",
+            _("Point size range: "),
+            pointSizeRange[0], pointSizeRange[1]);
+    s += buf;
+
+    s += i18n("Supported Extensions:") + "\n    ";
     if (ext != NULL)
     {
         QString extString(ext);
@@ -932,6 +992,7 @@ void KdeApp::slotOpenGLInfo() {
     QTextEdit edit(&dlg);
     edit.append(getOpenGLInfo());
     edit.setFocusPolicy(QWidget::NoFocus);
+    edit.setCursorPosition(0, 0);
     dlg.setMainWidget(&edit);
     dlg.resize(400,430);
     dlg.exec();
@@ -965,6 +1026,10 @@ void KdeApp::slotAccelerateTime() {
     appCore->charEntered('l');
 }
 
+void KdeApp::slotAccelerateTimeFine() {
+    appCore->charEntered('L');
+}
+
 void KdeApp::slotPauseTime() {
     appCore->charEntered(' ');
 }
@@ -973,9 +1038,13 @@ void KdeApp::slotSlowDownTime() {
     appCore->charEntered('k');
 }
 
+void KdeApp::slotSlowDownTimeFine() {
+    appCore->charEntered('K');
+}
+
 void KdeApp::slotSetTimeNow() {
     time_t curtime=time(NULL);
-    appCore->getSimulation()->setTime((double) curtime / 86400.0 + (double) astro::Date(1970, 1, 1));
+    appCore->getSimulation()->setTime(astro::UTCtoTDB((double) curtime / 86400.0 + (double) astro::Date(1970, 1, 1)));
     appCore->getSimulation()->update(0.0);
 }
 
@@ -1019,6 +1088,11 @@ void KdeApp::slotShowCloudMaps() {
             appCore->getRenderer()->getRenderFlags() ^ Renderer::ShowCloudMaps);
 }
 
+void KdeApp::slotShowCloudShadows() {
+     appCore->getRenderer()->setRenderFlags(
+            appCore->getRenderer()->getRenderFlags() ^ Renderer::ShowCloudShadows);
+}
+
 void KdeApp::slotShowOrbits() {
      appCore->getRenderer()->setRenderFlags(
             appCore->getRenderer()->getRenderFlags() ^ Renderer::ShowOrbits);
@@ -1039,6 +1113,11 @@ void KdeApp::slotShowMoonOrbits() {
             appCore->getRenderer()->getOrbitMask() ^ Body::Moon);
 }
 
+void KdeApp::slotShowStarOrbits() {
+     appCore->getRenderer()->setOrbitMask(
+            appCore->getRenderer()->getOrbitMask() ^ Body::Stellar);
+}
+
 void KdeApp::slotShowPlanetOrbits() {
      appCore->getRenderer()->setOrbitMask(
             appCore->getRenderer()->getOrbitMask() ^ Body::Planet);
@@ -1093,8 +1172,7 @@ void KdeApp::slotShowBoundaries() {
 }
 
 void KdeApp::slotShowAutoMag() {
-     appCore->getRenderer()->setRenderFlags(
-            appCore->getRenderer()->getRenderFlags() ^ Renderer::ShowAutoMag);
+     appCore->charEntered('\031');
 }
 
 void KdeApp::slotShowCometTails() {
@@ -1217,7 +1295,7 @@ void KdeApp::slotShowOtherLocations() {
 }
 
 void KdeApp::slotMinFeatureSize(int size) {
-    appCore->getRenderer()->setMinimumFeatureSize((float)size);  
+    appCore->getRenderer()->setMinimumFeatureSize((float)size);
 }
 
 void KdeApp::slotSplitH() {
@@ -1319,18 +1397,53 @@ void KdeApp::slotSetRenderPathGLSL() {
         appCore->getRenderer()->getGLContext()->setRenderPath(GLContext::GLPath_GLSL);
 }
 
+void KdeApp::slotToggleVideoSync() {
+    appCore->getRenderer()->setVideoSync(!appCore->getRenderer()->getVideoSync());
+}
+
 void KdeApp::slotCycleRenderPath() {
     appCore->charEntered('\026');
 }
 
 void KdeApp::slotGrabImage() {
-    QString saveAsName = KFileDialog::getSaveFileName(0, "*.png");
+    QString dir;
+    KGlobal::config()->setGroup("Preferences");
+    if (KGlobal::config()->hasKey("GrabImageDir")) {
+        dir = KGlobal::config()->readEntry("GrabImageDir");
+    } else {
+        dir = startDir;
+    }
+    QString saveAsName = KFileDialog::getSaveFileName(dir, "*.png");
     if (saveAsName != "") {
         QImage grabedImage = glWidget->grabFrameBuffer();
         grabedImage.save(saveAsName, "PNG");
+        QUrl file(saveAsName);
+        KGlobal::config()->writeEntry("GrabImageDir", file.dirPath());
     }
 }
 
+void KdeApp::slotCaptureVideo() {
+#ifdef THEORA
+    QString dir;
+    KGlobal::config()->setGroup("Preferences");
+    if (KGlobal::config()->hasKey("CaptureVideoDir")) {
+        dir = KGlobal::config()->readEntry("CaptureVideoDir");
+    } else {
+        dir = startDir;
+    }
+    VideoCaptureDlg *dialog = new VideoCaptureDlg(this, dir);
+    if (dialog->exec() == QDialog::Accepted) {
+        appCore->initMovieCapture(dialog);
+        action("captureVideo")->setEnabled(false);
+        KGlobal::config()->writeEntry("CaptureVideoDir", dialog->getDir());
+    } else {
+        delete dialog;
+    }
+#else
+    KMessageBox::queuedMessageBox(this, KMessageBox::Sorry, i18n("This version of Celestia was not built with support for movie recording."));
+#endif
+}
+
 void KdeApp::slotShowBookmarkBar() {
     KToolBar * bar = static_cast<KToolBar *>( child( "bookmarkBar", "KToolBar" ) );
     if (bar->isVisible()) bar->hide();
@@ -1352,11 +1465,12 @@ void KdeApp::slotCopyUrl() {
 }
 
 void KdeApp::slotGoTo() {
-    KLineEditDlg dlg(i18n("Enter URL"), "", this);
+    bool ok;
+    QString url = KInputDialog::getText(i18n("Go to URL"), i18n("Enter URL"), "", &ok, this);
 
-    if (dlg.exec()) {
+    if (ok) {
         appCore->addToHistory();
-        appCore->goToUrl(dlg.text().latin1());
+        appCore->goToUrl(url.latin1());
     }
 }
 
@@ -1380,13 +1494,13 @@ void KdeApp::dropEvent(QDropEvent* event) {
 }
 
 void KdeApp::slotBackAboutToShow() {
-    int i; 
+    int i;
     KPopupMenu* menu = backAction->popupMenu();
     std::vector<Url>::size_type current = appCore->getHistoryCurrent();
     int pos;
-    std::vector<Url> history = appCore->getHistory(); 
+    std::vector<Url> history = appCore->getHistory();
 
-    menu->clear(); 
+    menu->clear();
     for (i=0, pos = current - 1 ; pos >= 0 && i < 15 ; pos--, i++) {
         menu->insertItem(QString(history[pos].getName().c_str()), pos);
     }
@@ -1438,6 +1552,10 @@ void KdeApp::popupMenu(float x, float y, Selection sel) {
     popup.process(id);
 }
 
+void KdeApp::resizeEvent(QResizeEvent* e) {
+    emit resized(e->size().width(), e->size().height());
+}
+
 LongLatDialog::LongLatDialog(QWidget* parent, CelestiaCore* appCore) :
    KDialogBase(parent, "long_lat", true, "Go to Long/Lat"), appCore(appCore)
 {
@@ -1466,17 +1584,17 @@ LongLatDialog::LongLatDialog(QWidget* parent, CelestiaCore* appCore) :
     latSign = new QComboBox( grid );
     latSign->insertItem(i18n("North", "N"));
     latSign->insertItem(i18n("South", "S"));
-    
+
     QLabel* altLab = new QLabel(i18n("Altitude: "), grid);
     altLab->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     altEdit = new QLineEdit(grid);
     altEdit->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     altEdit->setValidator(new QDoubleValidator(altEdit));
     new QLabel(i18n("km"), grid);
-    
+
     double distance, longitude, latitude;
     appCore->getSimulation()->getSelectionLongLat(distance, longitude, latitude);
-    
+
     if (longitude < 0) {
         longitude = -longitude;
         longSign->setCurrentItem(1);
@@ -1485,11 +1603,11 @@ LongLatDialog::LongLatDialog(QWidget* parent, CelestiaCore* appCore) :
         latitude = -latitude;
         latSign->setCurrentItem(1);
     }
-    
+
     Selection selection = appCore->getSimulation()->getSelection();
-    QString objName(selection.getName().c_str());
+    QString objName(selection.getName(true).c_str());
     objEdit->setText(objName.mid(objName.findRev('/') + 1));
-    
+
     latEdit->setText(QString("%1").arg(latitude, 0, 'f', 3));
     longEdit->setText(QString("%1").arg(longitude, 0, 'f', 3));
     altEdit->setText(QString("%1").arg(distance - selection.radius(), 0, 'f', 0));
@@ -1508,7 +1626,7 @@ void LongLatDialog::slotOk() {
 
 void LongLatDialog::slotApply() {
     Simulation* appSim = appCore->getSimulation();
-    Selection sel = appSim->findObjectFromPath(objEdit->text().latin1());
+    Selection sel = appSim->findObjectFromPath((const char*)objEdit->text().utf8(), true);
     if (!sel.empty())
     {
         appSim->setSelection(sel);
