@@ -1,8 +1,8 @@
 //{{NO_DEPENDENCIES}}
-// Microsoft Developer Studio generated include file.
-// Used by Celestia.rc
+// Microsoft Visual C++ generated include file.
+// Used by celestia.rc
 //
-#define VERSION_STRING                  "1.4.1"
+#define VERSION_STRING                  "1.5.0"
 #define IDR_MENU1                       101
 #define IDD_ABOUT                       102
 #define IDI_CELESTIA_ICON               103
@@ -52,52 +52,53 @@
 #define IDC_CALENDAR                    1006
 #define IDC_TIMEPICKER                  1007
 #define IDC_DATEPICKER                  1008
-#define IDC_BOOKMARK_EDIT               1009
-#define IDC_SSBROWSER_TREE              1010
-#define IDC_BUTTON_CENTER               1011
-#define IDC_BUTTON_GOTO                 1012
-#define IDC_STARBROWSER_LIST            1013
-#define IDC_BUTTON_REFRESH              1014
-#define IDC_RADIO_NEAREST               1015
-#define IDC_RADIO_BRIGHTEST             1016
-#define IDC_RADIO_WITHPLANETS           1017
-#define IDC_COMBO_TOURGUIDE             1018
-#define IDC_TEXT_DESCRIPTION            1019
-#define IDC_TEXT_CONTROLSHELP           1020
-#define IDC_EDIT_OBJECTNAME             1021
-#define IDC_EDIT_LONGITUDE              1022
-#define IDC_EDIT_LATITUDE               1023
-#define IDC_EDIT_DISTANCE               1024
-#define IDC_RADIO_KM                    1025
-#define IDC_RADIO_AU                    1026
-#define IDC_RADIO_RADII                 1027
-#define IDC_SETCURRENTTIME              1028
-#define IDC_COMBO_MOVIE_SIZE            1029
-#define IDC_COMBO_MOVIE_FRAMERATE       1030
-#define IDC_MAXSTARS_SLIDER             1031
-#define IDC_MAXSTARS_EDIT               1032
-#define IDC_COMBO_RESOLUTION            1033
-#define IDC_SHOWGALAXIES                1034
-#define IDC_SHOWATMOSPHERES             1035
-#define IDC_SHOWCLOUDS                  1036
-#define IDC_SHOWORBITS                  1037
-#define IDC_SHOWCONSTELLATIONS          1038
-#define IDC_SHOWCELESTIALGRID           1039
-#define IDC_SHOWNIGHTSIDELIGHTS         1040
-#define IDC_SHOWPLANETS                 1041
-#define IDC_SHOWSTARS                   1042
-#define IDC_SHOWECLIPSESHADOWS          1043
-#define IDC_LABELPLANETS                1044
-#define IDC_LABELMOONS                  1045
-#define IDC_LABELSTARS                  1046
-#define IDC_LABELCONSTELLATIONS         1047
-#define IDC_LABELGALAXIES               1048
-#define IDC_INFOTEXT0                   1049
-#define IDC_INFOTEXT1                   1050
-#define IDC_INFOTEXT2                   1051
-#define IDC_BOOKMARK_CREATEIN           1052
-#define IDC_BOOKMARK_NEWFOLDER          1053
-#define IDC_BOOKMARK_FOLDERTREE         1054
+#define IDC_JDPICKER                  1009
+#define IDC_BOOKMARK_EDIT               1010
+#define IDC_SSBROWSER_TREE              1011
+#define IDC_BUTTON_CENTER               1012
+#define IDC_BUTTON_GOTO                 1013
+#define IDC_STARBROWSER_LIST            1014
+#define IDC_BUTTON_REFRESH              1015
+#define IDC_RADIO_NEAREST               1016
+#define IDC_RADIO_BRIGHTEST             1017
+#define IDC_RADIO_WITHPLANETS           1018
+#define IDC_COMBO_TOURGUIDE             1019
+#define IDC_TEXT_DESCRIPTION            1020
+#define IDC_TEXT_CONTROLSHELP           1021
+#define IDC_EDIT_OBJECTNAME             1022
+#define IDC_EDIT_LONGITUDE              1023
+#define IDC_EDIT_LATITUDE               1024
+#define IDC_EDIT_DISTANCE               1025
+#define IDC_RADIO_KM                    1026
+#define IDC_RADIO_AU                    1027
+#define IDC_RADIO_RADII                 1028
+#define IDC_SETCURRENTTIME              1029
+#define IDC_COMBO_MOVIE_SIZE            1030
+#define IDC_COMBO_MOVIE_FRAMERATE       1031
+#define IDC_MAXSTARS_SLIDER             1032
+#define IDC_MAXSTARS_EDIT               1033
+#define IDC_COMBO_RESOLUTION            1034
+#define IDC_SHOWGALAXIES                1035
+#define IDC_SHOWATMOSPHERES             1036
+#define IDC_SHOWCLOUDS                  1037
+#define IDC_SHOWORBITS                  1038
+#define IDC_SHOWCONSTELLATIONS          1039
+#define IDC_SHOWCELESTIALGRID           1040
+#define IDC_SHOWNIGHTSIDELIGHTS         1041
+#define IDC_SHOWPLANETS                 1042
+#define IDC_SHOWSTARS                   1043
+#define IDC_SHOWECLIPSESHADOWS          1044
+#define IDC_LABELPLANETS                1045
+#define IDC_LABELMOONS                  1046
+#define IDC_LABELSTARS                  1047
+#define IDC_LABELCONSTELLATIONS         1048
+#define IDC_LABELGALAXIES               1049
+#define IDC_INFOTEXT0                   1050
+#define IDC_INFOTEXT1                   1051
+#define IDC_INFOTEXT2                   1052
+#define IDC_BOOKMARK_CREATEIN           1053
+#define IDC_BOOKMARK_NEWFOLDER          1054
+#define IDC_BOOKMARK_FOLDERTREE         1055
 #define IDC_BOOKMARKFOLDER              1057
 #define IDC_ORGANIZE_BOOKMARK_TREE      1058
 #define IDC_ORGANIZE_BOOKMARKS_NEWFOLDER 1059
@@ -117,12 +118,14 @@
 #define IDC_COMETORBITS                 1073
 #define IDC_LABELNEBULAE                1074
 #define IDC_LABELOPENCLUSTERS           1075
+#define IDC_LABELCONSTELLATIONSLATIN    1076
 #define IDCOMPUTE                       1078
 #define IDC_DATEFROM                    1079
 #define IDC_ECLIPSEFROM                 1079
 #define IDC_DATETO                      1080
 #define IDC_ECLIPSETO                   1080
 #define IDSETDATEANDGO                  1081
+#define IDC_STARORBITS                  1082
 #define IDC_EDIT_FILTER_DISTANCE        1084
 #define IDC_SLIDER_FILTER_DISTANCE      1085
 #define IDC_SHOWCOMETTAILS              1086
@@ -130,6 +133,7 @@
 #define IDC_SHOWRINGSHADOWS             1088
 #define IDC_ECLIPSETYPE                 1089
 #define IDC_ECLIPSETARGET               1090
+#define IDC_SHOWCLOUDSHADOWS            1091
 #define IDC_SHOW_CITIES                 1100
 #define IDC_SHOW_OBSERVATORIES          1101
 #define IDC_SHOW_LANDING_SITES          1102
@@ -140,11 +144,13 @@
 #define IDC_SHOW_TERRAE                 1107
 #define IDC_SHOW_OTHERS                 1108
 #define IDC_LABELFEATURES               1109
-#define IDC_MARKFEATURES                1110
 #define IDC_SLIDER_FEATURE_SIZE         1150
 #define IDC_EDIT_FEATURE_SIZE           1151
 #define IDC_CELESTIALINK                1152
 #define IDC_COMBOBOX_TIMEZONE           1153
+#define IDC_COMBOBOX_DATE_FORMAT           1154
+#define IDC_SOLARECLIPSE                1155
+#define IDC_LUNARECLIPSE                1156
 #define ID_SELECTSOL                    40001
 #define ID_GOTOSELECTION                40002
 #define ID_FOLLOWSELECTION              40003
@@ -204,11 +210,17 @@
 #define ID_VIEW_SINGLE                  40070
 #define ID_VIEW_SHOW_FRAMES             40071
 #define ID_VIEW_SYNC_TIME               40072
+#define ID_FILE_SCRIPTS                 40073
 #define ID_RENDER_STARSTYLE             40079
 #define ID_RENDER_STARSTYLE_FUZZY       40080
 #define ID_RENDER_STARSTYLE_POINTS      40081
 #define ID_RENDER_STARSTYLE_DISCS       40082
+#define ID_RENDER_BODY_AXES             40083
+#define ID_RENDER_FRAME_AXES            40084
+#define ID_RENDER_SUN_DIRECTION         40085
+#define ID_RENDER_VELOCITY_VECTOR       40086
 #define ID_BOOKMARKS_FIRSTBOOKMARK      41000
+#define ID_FIRST_SCRIPT                 42000
 
 // Next default values for new objects
 // 
@@ -216,7 +228,7 @@
 #ifndef APSTUDIO_READONLY_SYMBOLS
 #define _APS_NEXT_RESOURCE_VALUE        142
 #define _APS_NEXT_COMMAND_VALUE         40083
-#define _APS_NEXT_CONTROL_VALUE         1153
+#define _APS_NEXT_CONTROL_VALUE         1155
 #define _APS_NEXT_SYMED_VALUE           101
 #endif
 #endif
