@@ -68,6 +68,9 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
         case IDC_SHOWCLOUDS:
             renderer->setRenderFlags(renderFlags ^ Renderer::ShowCloudMaps);
             break;
+        case IDC_SHOWCLOUDSHADOWS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowCloudShadows);
+            break;
         case IDC_SHOWCONSTELLATIONS:
             renderer->setRenderFlags(renderFlags ^ Renderer::ShowDiagrams);
             break;
@@ -89,6 +92,9 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
         case IDC_PLANETORBITS:
             renderer->setOrbitMask(orbitMask ^ Body::Planet);
             break;
+        case IDC_STARORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::Stellar);
+            break;
         case IDC_MOONORBITS:
             renderer->setOrbitMask(orbitMask ^ Body::Moon);
             break;
@@ -122,6 +128,9 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
         case IDC_LABELCONSTELLATIONS:
             renderer->setLabelMode(labelMode ^ Renderer::ConstellationLabels);
             break;
+        case IDC_LABELCONSTELLATIONSLATIN:
+            renderer->setLabelMode(labelMode ^ Renderer::I18nConstellationLabels);
+            break;
         case IDC_LABELGALAXIES:
             renderer->setLabelMode(labelMode ^ Renderer::GalaxyLabels);
             break;
@@ -263,6 +272,8 @@ void ViewOptionsDialog::SetControls(HWND hDlg)
         (renderFlags & Renderer::ShowCelestialSphere)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCLOUDS, BM_SETCHECK,
         (renderFlags & Renderer::ShowCloudMaps)? BST_CHECKED:BST_UNCHECKED, 0);
+    SendDlgItemMessage(hDlg, IDC_SHOWCLOUDSHADOWS, BM_SETCHECK,
+        (renderFlags & Renderer::ShowCloudShadows)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCONSTELLATIONS, BM_SETCHECK,
         (renderFlags & Renderer::ShowDiagrams)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWECLIPSESHADOWS, BM_SETCHECK,
@@ -279,6 +290,7 @@ void ViewOptionsDialog::SetControls(HWND hDlg)
     dlgCheck(hDlg, IDC_ASTEROIDORBITS,   orbitMask,   Body::Asteroid);
     dlgCheck(hDlg, IDC_COMETORBITS,      orbitMask,   Body::Comet);
     dlgCheck(hDlg, IDC_SPACECRAFTORBITS, orbitMask,   Body::Spacecraft);
+    dlgCheck(hDlg, IDC_STARORBITS,       orbitMask,   Body::Stellar);
     SendDlgItemMessage(hDlg, IDC_SHOWPLANETS, BM_SETCHECK,
         (renderFlags & Renderer::ShowPlanets)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWSTARS, BM_SETCHECK,
@@ -293,6 +305,8 @@ void ViewOptionsDialog::SetControls(HWND hDlg)
         (renderFlags & Renderer::ShowMarkers)? BST_CHECKED:BST_UNCHECKED, 0);
 
     dlgCheck(hDlg, IDC_LABELCONSTELLATIONS, labelMode, Renderer::ConstellationLabels);
+    SendDlgItemMessage(hDlg, IDC_LABELCONSTELLATIONSLATIN, BM_SETCHECK,
+                       ((labelMode & Renderer::I18nConstellationLabels) == 0) ? BST_CHECKED : BST_UNCHECKED, 0);
     dlgCheck(hDlg, IDC_LABELGALAXIES,   labelMode, Renderer::GalaxyLabels);
 	dlgCheck(hDlg, IDC_LABELNEBULAE,    labelMode, Renderer::NebulaLabels);
 	dlgCheck(hDlg, IDC_LABELOPENCLUSTERS, labelMode, Renderer::OpenClusterLabels);
