@@ -14,9 +14,9 @@
 #include <cstdarg>
 
 #ifndef _WIN32
-#ifndef MACOSX_PB
+#ifndef TARGET_OS_MAC
 #include <config.h>
-#endif /* MACOSX_PB */
+#endif /* TARGET_OS_MAC */
 #endif /* _WIN32 */
 
 static int debugVerbosity = 0;
